@@ -18,10 +18,16 @@
 package org.apache.fluss.client.lookup;
 
 import org.apache.fluss.annotation.Internal;
+import org.apache.fluss.annotation.VisibleForTesting;
 import org.apache.fluss.client.metadata.MetadataUpdater;
+import org.apache.fluss.exception.ApiException;
 import org.apache.fluss.exception.FlussRuntimeException;
+import org.apache.fluss.exception.InvalidMetadataException;
 import org.apache.fluss.exception.LeaderNotAvailableException;
+import org.apache.fluss.exception.RetriableException;
+import org.apache.fluss.metadata.PhysicalTablePath;
 import org.apache.fluss.metadata.TableBucket;
+import org.apache.fluss.metadata.TablePartition;
 import org.apache.fluss.rpc.gateway.TabletServerGateway;
 import org.apache.fluss.rpc.messages.LookupRequest;
 import org.apache.fluss.rpc.messages.LookupResponse;
@@ -36,10 +42,14 @@ import org.apache.fluss.utils.types.Tuple2;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import javax.annotation.Nullable;
+
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.Semaphore;
 import java.util.stream.Collectors;
 
@@ -65,11 +75,26 @@ class LookupSender implements Runnable {
 
     private final Semaphore maxInFlightReuqestsSemaphore;
 
-    LookupSender(MetadataUpdater metadataUpdater, LookupQueue lookupQueue, int maxFlightRequests) {
+    private final int maxRetries;
+
+    private final int maxRequestTimeoutMs;
+
+    private final short acks;
+
+    LookupSender(
+            MetadataUpdater metadataUpdater,
+            LookupQueue lookupQueue,
+            int maxFlightRequests,
+            int maxRetries,
+            short acks,
+            int maxRequestTimeoutMs) {
         this.metadataUpdater = metadataUpdater;
         this.lookupQueue = lookupQueue;
         this.maxInFlightReuqestsSemaphore = new Semaphore(maxFlightRequests);
+        this.maxRetries = maxRetries;
         this.running = true;
+        this.acks = acks;
+        this.maxRequestTimeoutMs = maxRequestTimeoutMs;
     }
 
     @Override
@@ -109,13 +134,22 @@ class LookupSender implements Runnable {
         sendLookups(lookups);
     }
 
-    private void sendLookups(List<AbstractLookupQuery<?>> lookups) {
+    private void sendLookups(List<AbstractLookupQuery<?>> lookups) throws Exception {
         if (lookups.isEmpty()) {
             return;
         }
         // group by <leader, lookup type> to lookup batches
         Map<Tuple2<Integer, LookupType>, List<AbstractLookupQuery<?>>> lookupBatches =
                 groupByLeaderAndType(lookups);
+
+        // if no lookup batches, sleep a bit to avoid busy loop. This case will happen when there is
+        // no leader for all the lookup request in queue.
+        if (lookupBatches.isEmpty() && !lookupQueue.hasUnDrained()) {
+            // TODO: may use wait/notify mechanism to avoid active sleep, and use a dynamic sleep
+            // time based on the request waited time.
+            Thread.sleep(100);
+        }
+
         // now, send the batches
         lookupBatches.forEach(
                 (destAndType, batch) -> sendLookups(destAndType.f0, destAndType.f1, batch));
@@ -131,11 +165,14 @@ class LookupSender implements Runnable {
             // lookup the leader node
             TableBucket tb = lookup.tableBucket();
             try {
-                // TODO this can be a re-triable operation. We should retry here instead of
-                // throwing exception.
-                leader = metadataUpdater.leaderFor(tb);
+                // TODO Metadata requests are being sent too frequently here. consider first
+                // collecting the tables that need to be updated and then sending them together in
+                // one request.
+                leader = metadataUpdater.leaderFor(lookup.tablePath(), tb);
             } catch (Exception e) {
-                lookup.future().completeExceptionally(e);
+                // if leader is not found, re-enqueue the lookup to send again.
+                LOG.warn("Failed to lookup the leader for {} when lookup", tb, e);
+                reEnqueueLookup(lookup);
                 continue;
             }
             lookupBatchesByLeader
@@ -145,26 +182,22 @@ class LookupSender implements Runnable {
         return lookupBatchesByLeader;
     }
 
-    private void sendLookups(
+    @VisibleForTesting
+    void sendLookups(
             int destination, LookupType lookupType, List<AbstractLookupQuery<?>> lookupBatches) {
-        TabletServerGateway gateway = metadataUpdater.newTabletServerClientForNode(destination);
-        if (gateway == null) {
-            // TODO handle this exception, like retry.
-            throw new LeaderNotAvailableException(
-                    "Server " + destination + " is not found in metadata cache.");
-        }
-
         if (lookupType == LookupType.LOOKUP) {
-            sendLookupRequest(gateway, lookupBatches);
+            sendLookupRequest(destination, lookupBatches, false);
+        } else if (lookupType == LookupType.LOOKUP_WITH_INSERT_IF_NOT_EXISTS) {
+            sendLookupRequest(destination, lookupBatches, true);
         } else if (lookupType == LookupType.PREFIX_LOOKUP) {
-            sendPrefixLookupRequest(gateway, lookupBatches);
+            sendPrefixLookupRequest(destination, lookupBatches);
         } else {
             throw new IllegalArgumentException("Unsupported lookup type: " + lookupType);
         }
     }
 
     private void sendLookupRequest(
-            TabletServerGateway gateway, List<AbstractLookupQuery<?>> lookups) {
+            int destination, List<AbstractLookupQuery<?>> lookups, boolean insertIfNotExists) {
         // table id -> (bucket -> lookups)
         Map<Long, Map<TableBucket, LookupBatch>> lookupByTableId = new HashMap<>();
         for (AbstractLookupQuery<?> abstractLookupQuery : lookups) {
@@ -177,17 +210,37 @@ class LookupSender implements Runnable {
                     .addLookup(lookup);
         }
 
+        TabletServerGateway gateway = metadataUpdater.newTabletServerClientForNode(destination);
+        if (gateway == null) {
+            lookupByTableId.forEach(
+                    (tableId, lookupsByBucket) ->
+                            handleLookupRequestException(
+                                    new LeaderNotAvailableException(
+                                            "Server "
+                                                    + destination
+                                                    + " is not found in metadata cache."),
+                                    destination,
+                                    lookupsByBucket));
+            return;
+        }
+
         lookupByTableId.forEach(
                 (tableId, lookupsByBucket) ->
                         sendLookupRequestAndHandleResponse(
+                                destination,
                                 gateway,
-                                makeLookupRequest(tableId, lookupsByBucket.values()),
+                                makeLookupRequest(
+                                        tableId,
+                                        lookupsByBucket.values(),
+                                        insertIfNotExists,
+                                        acks,
+                                        maxRequestTimeoutMs),
                                 tableId,
                                 lookupsByBucket));
     }
 
     private void sendPrefixLookupRequest(
-            TabletServerGateway gateway, List<AbstractLookupQuery<?>> prefixLookups) {
+            int destination, List<AbstractLookupQuery<?>> prefixLookups) {
         // table id -> (bucket -> lookups)
         Map<Long, Map<TableBucket, PrefixLookupBatch>> lookupByTableId = new HashMap<>();
         for (AbstractLookupQuery<?> abstractLookupQuery : prefixLookups) {
@@ -200,9 +253,24 @@ class LookupSender implements Runnable {
                     .addLookup(prefixLookup);
         }
 
+        TabletServerGateway gateway = metadataUpdater.newTabletServerClientForNode(destination);
+        if (gateway == null) {
+            lookupByTableId.forEach(
+                    (tableId, lookupsByBucket) ->
+                            handlePrefixLookupException(
+                                    new LeaderNotAvailableException(
+                                            "Server "
+                                                    + destination
+                                                    + " is not found in metadata cache."),
+                                    destination,
+                                    lookupsByBucket));
+            return;
+        }
+
         lookupByTableId.forEach(
                 (tableId, prefixLookupBatch) ->
                         sendPrefixLookupRequestAndHandleResponse(
+                                destination,
                                 gateway,
                                 makePrefixLookupRequest(tableId, prefixLookupBatch.values()),
                                 tableId,
@@ -210,6 +278,7 @@ class LookupSender implements Runnable {
     }
 
     private void sendLookupRequestAndHandleResponse(
+            int destination,
             TabletServerGateway gateway,
             LookupRequest lookupRequest,
             long tableId,
@@ -224,7 +293,8 @@ class LookupSender implements Runnable {
                 .thenAccept(
                         lookupResponse -> {
                             try {
-                                handleLookupResponse(tableId, lookupResponse, lookupsByBucket);
+                                handleLookupResponse(
+                                        tableId, destination, lookupResponse, lookupsByBucket);
                             } finally {
                                 maxInFlightReuqestsSemaphore.release();
                             }
@@ -232,7 +302,7 @@ class LookupSender implements Runnable {
                 .exceptionally(
                         e -> {
                             try {
-                                handleLookupRequestException(e, lookupsByBucket);
+                                handleLookupRequestException(e, destination, lookupsByBucket);
                                 return null;
                             } finally {
                                 maxInFlightReuqestsSemaphore.release();
@@ -241,6 +311,7 @@ class LookupSender implements Runnable {
     }
 
     private void sendPrefixLookupRequestAndHandleResponse(
+            int destination,
             TabletServerGateway gateway,
             PrefixLookupRequest prefixLookupRequest,
             long tableId,
@@ -256,7 +327,10 @@ class LookupSender implements Runnable {
                         prefixLookupResponse -> {
                             try {
                                 handlePrefixLookupResponse(
-                                        tableId, prefixLookupResponse, lookupsByBucket);
+                                        tableId,
+                                        destination,
+                                        prefixLookupResponse,
+                                        lookupsByBucket);
                             } finally {
                                 maxInFlightReuqestsSemaphore.release();
                             }
@@ -264,7 +338,7 @@ class LookupSender implements Runnable {
                 .exceptionally(
                         e -> {
                             try {
-                                handlePrefixLookupException(e, lookupsByBucket);
+                                handlePrefixLookupException(e, destination, lookupsByBucket);
                                 return null;
                             } finally {
                                 maxInFlightReuqestsSemaphore.release();
@@ -274,6 +348,7 @@ class LookupSender implements Runnable {
 
     private void handleLookupResponse(
             long tableId,
+            int destination,
             LookupResponse lookupResponse,
             Map<TableBucket, LookupBatch> lookupsByBucket) {
         for (PbLookupRespForBucket pbLookupRespForBucket : lookupResponse.getBucketsRespsList()) {
@@ -286,13 +361,8 @@ class LookupSender implements Runnable {
                             pbLookupRespForBucket.getBucketId());
             LookupBatch lookupBatch = lookupsByBucket.get(tableBucket);
             if (pbLookupRespForBucket.hasErrorCode()) {
-                // TODO for re-triable error, we should retry here instead of throwing exception.
                 ApiError error = ApiError.fromErrorMessage(pbLookupRespForBucket);
-                LOG.warn(
-                        "Get error lookup response on table bucket {}, fail. Error: {}",
-                        tableBucket,
-                        error.formatErrMsg());
-                lookupBatch.completeExceptionally(error.exception());
+                handleLookupError(tableBucket, destination, error, lookupBatch.lookups(), "lookup");
             } else {
                 List<byte[]> byteValues =
                         pbLookupRespForBucket.getValuesList().stream()
@@ -312,6 +382,7 @@ class LookupSender implements Runnable {
 
     private void handlePrefixLookupResponse(
             long tableId,
+            int destination,
             PrefixLookupResponse prefixLookupResponse,
             Map<TableBucket, PrefixLookupBatch> prefixLookupsByBucket) {
         for (PbPrefixLookupRespForBucket pbRespForBucket :
@@ -326,13 +397,13 @@ class LookupSender implements Runnable {
 
             PrefixLookupBatch prefixLookupBatch = prefixLookupsByBucket.get(tableBucket);
             if (pbRespForBucket.hasErrorCode()) {
-                // TODO for re-triable error, we should retry here instead of throwing exception.
                 ApiError error = ApiError.fromErrorMessage(pbRespForBucket);
-                LOG.warn(
-                        "Get error prefix lookup response on table bucket {}, fail. Error: {}",
+                handleLookupError(
                         tableBucket,
-                        error.formatErrMsg());
-                prefixLookupBatch.completeExceptionally(error.exception());
+                        destination,
+                        error,
+                        prefixLookupBatch.lookups(),
+                        "prefix lookup");
             } else {
                 List<List<byte[]>> result = new ArrayList<>(pbRespForBucket.getValueListsCount());
                 for (int i = 0; i < pbRespForBucket.getValueListsCount(); i++) {
@@ -349,25 +420,96 @@ class LookupSender implements Runnable {
     }
 
     private void handleLookupRequestException(
-            Throwable t, Map<TableBucket, LookupBatch> lookupsByBucket) {
+            Throwable t, int destination, Map<TableBucket, LookupBatch> lookupsByBucket) {
         ApiError error = ApiError.fromThrowable(t);
         for (LookupBatch lookupBatch : lookupsByBucket.values()) {
-            // TODO for re-triable error, we should retry here instead of throwing exception.
-            LOG.warn(
-                    "Get error lookup response on table bucket {}, fail. Error: {}",
-                    lookupBatch.tableBucket(),
-                    error.formatErrMsg());
-            lookupBatch.completeExceptionally(error.exception());
+            handleLookupError(
+                    lookupBatch.tableBucket(), destination, error, lookupBatch.lookups(), "lookup");
         }
     }
 
     private void handlePrefixLookupException(
-            Throwable t, Map<TableBucket, PrefixLookupBatch> lookupsByBucket) {
+            Throwable t, int destination, Map<TableBucket, PrefixLookupBatch> lookupsByBucket) {
         ApiError error = ApiError.fromThrowable(t);
-        // TODO If error, we need to retry send the request instead of throw exception.
-        LOG.warn("Get error prefix lookup response. Error: {}", error.formatErrMsg());
         for (PrefixLookupBatch lookupBatch : lookupsByBucket.values()) {
-            lookupBatch.completeExceptionally(error.exception());
+            handleLookupError(
+                    lookupBatch.tableBucket(),
+                    destination,
+                    error,
+                    lookupBatch.lookups(),
+                    "prefix lookup");
+        }
+    }
+
+    private void reEnqueueLookup(AbstractLookupQuery<?> lookup) {
+        lookupQueue.reEnqueue(lookup);
+    }
+
+    private boolean canRetry(AbstractLookupQuery<?> lookup, Exception exception) {
+        return lookup.retries() < maxRetries
+                && !lookup.future().isDone()
+                && exception instanceof RetriableException;
+    }
+
+    /**
+     * Handle lookup error with retry logic. For each lookup in the list, check if it can be
+     * retried. If yes, re-enqueue it; otherwise, complete it exceptionally.
+     *
+     * @param tableBucket the table bucket
+     * @param error the error from server response
+     * @param lookups the list of lookups to handle
+     * @param lookupType the type of lookup ("" for regular lookup, "prefix " for prefix lookup)
+     */
+    private void handleLookupError(
+            TableBucket tableBucket,
+            int destination,
+            ApiError error,
+            List<? extends AbstractLookupQuery<?>> lookups,
+            String lookupType) {
+        ApiException exception = error.error().exception();
+        LOG.error(
+                "Failed to {} from node {} for bucket {}",
+                lookupType,
+                destination,
+                tableBucket,
+                exception);
+        if (exception instanceof InvalidMetadataException) {
+            LOG.warn(
+                    "Invalid metadata error in {} request. Going to request metadata update.",
+                    lookupType,
+                    exception);
+            long tableId = tableBucket.getTableId();
+            TableOrPartitions tableOrPartitions;
+            if (tableBucket.getPartitionId() == null) {
+                tableOrPartitions = new TableOrPartitions(Collections.singleton(tableId), null);
+            } else {
+                tableOrPartitions =
+                        new TableOrPartitions(
+                                null,
+                                Collections.singleton(
+                                        new TablePartition(tableId, tableBucket.getPartitionId())));
+            }
+            invalidTableOrPartitions(tableOrPartitions);
+        }
+
+        for (AbstractLookupQuery<?> lookup : lookups) {
+            if (canRetry(lookup, error.exception())) {
+                LOG.warn(
+                        "Get error {} response on table bucket {}, retrying ({} attempts left). Error: {}",
+                        lookupType,
+                        tableBucket,
+                        maxRetries - lookup.retries(),
+                        error.formatErrMsg());
+                lookup.incrementRetries();
+                reEnqueueLookup(lookup);
+            } else {
+                LOG.warn(
+                        "Get error {} response on table bucket {}, fail. Error: {}",
+                        lookupType,
+                        tableBucket,
+                        error.formatErrMsg());
+                lookup.future().completeExceptionally(error.exception());
+            }
         }
     }
 
@@ -382,4 +524,23 @@ class LookupSender implements Runnable {
         lookupQueue.close();
         running = false;
     }
+
+    /** A helper class to hold table ids or table partitions. */
+    private static class TableOrPartitions {
+        private final @Nullable Set<Long> tableIds;
+        private final @Nullable Set<TablePartition> tablePartitions;
+
+        TableOrPartitions(
+                @Nullable Set<Long> tableIds, @Nullable Set<TablePartition> tablePartitions) {
+            this.tableIds = tableIds;
+            this.tablePartitions = tablePartitions;
+        }
+    }
+
+    private void invalidTableOrPartitions(TableOrPartitions tableOrPartitions) {
+        Set<PhysicalTablePath> physicalTablePaths =
+                metadataUpdater.getPhysicalTablePathByIds(
+                        tableOrPartitions.tableIds, tableOrPartitions.tablePartitions);
+        metadataUpdater.invalidPhysicalTableBucketMeta(physicalTablePaths);
+    }
 }
