@@ -26,10 +26,13 @@ import org.apache.fluss.client.table.scanner.ScanRecord;
 import org.apache.fluss.cluster.BucketLocation;
 import org.apache.fluss.config.ConfigOptions;
 import org.apache.fluss.config.Configuration;
+import org.apache.fluss.exception.ApiException;
 import org.apache.fluss.exception.InvalidMetadataException;
 import org.apache.fluss.exception.LeaderNotAvailableException;
+import org.apache.fluss.exception.PartitionNotExistException;
 import org.apache.fluss.fs.FsPath;
 import org.apache.fluss.metadata.PhysicalTablePath;
+import org.apache.fluss.metadata.SchemaGetter;
 import org.apache.fluss.metadata.TableBucket;
 import org.apache.fluss.metadata.TableInfo;
 import org.apache.fluss.metadata.TablePartition;
@@ -47,6 +50,7 @@ import org.apache.fluss.rpc.messages.PbFetchLogReqForBucket;
 import org.apache.fluss.rpc.messages.PbFetchLogReqForTable;
 import org.apache.fluss.rpc.messages.PbFetchLogRespForBucket;
 import org.apache.fluss.rpc.messages.PbFetchLogRespForTable;
+import org.apache.fluss.rpc.protocol.ApiError;
 import org.apache.fluss.rpc.protocol.Errors;
 import org.apache.fluss.utils.IOUtils;
 import org.apache.fluss.utils.Projection;
@@ -65,6 +69,7 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
 import java.util.stream.Collectors;
 
@@ -114,12 +119,14 @@ public class LogFetcher implements Closeable {
             Configuration conf,
             MetadataUpdater metadataUpdater,
             ScannerMetricGroup scannerMetricGroup,
-            RemoteFileDownloader remoteFileDownloader) {
+            RemoteFileDownloader remoteFileDownloader,
+            SchemaGetter schemaGetter) {
         this.tablePath = tableInfo.getTablePath();
         this.isPartitioned = tableInfo.isPartitioned();
-        this.readContext = LogRecordReadContext.createReadContext(tableInfo, false, projection);
+        this.readContext =
+                LogRecordReadContext.createReadContext(tableInfo, false, projection, schemaGetter);
         this.remoteReadContext =
-                LogRecordReadContext.createReadContext(tableInfo, true, projection);
+                LogRecordReadContext.createReadContext(tableInfo, true, projection, schemaGetter);
         this.projection = projection;
         this.logScannerStatus = logScannerStatus;
         this.maxFetchBytes =
@@ -213,14 +220,27 @@ public class LogFetcher implements Closeable {
             }
         }
 
-        if (isPartitioned && !partitionIds.isEmpty()) {
-            metadataUpdater.updateMetadata(Collections.singleton(tablePath), null, partitionIds);
-        } else if (needUpdate) {
-            metadataUpdater.updateTableOrPartitionMetadata(tablePath, null);
+        try {
+            if (isPartitioned && !partitionIds.isEmpty()) {
+                metadataUpdater.updateMetadata(
+                        Collections.singleton(tablePath), null, partitionIds);
+            } else if (needUpdate) {
+                metadataUpdater.updateTableOrPartitionMetadata(tablePath, null);
+            }
+        } catch (Exception e) {
+            if (e instanceof PartitionNotExistException) {
+                // ignore this exception, this is probably happen because the partition is deleted.
+                // The fetcher can also work fine. The caller like flink can remove the partition
+                // from fetch list when receive exception.
+                LOG.warn("Receive PartitionNotExistException when update metadata, ignore it", e);
+            } else {
+                throw e;
+            }
         }
     }
 
-    private void sendFetchRequest(int destination, FetchLogRequest fetchLogRequest) {
+    @VisibleForTesting
+    void sendFetchRequest(int destination, FetchLogRequest fetchLogRequest) {
         TableOrPartitions tableOrPartitionsInFetchRequest =
                 getTableOrPartitionsInFetchRequest(fetchLogRequest);
         // TODO cache the tablet server gateway.
@@ -341,6 +361,14 @@ public class LogFetcher implements Closeable {
                                     respForBucket.getBucketId());
                     FetchLogResultForBucket fetchResultForBucket =
                             getFetchLogResultForBucket(tb, tablePath, respForBucket);
+
+                    // if error code is not NONE, it means the fetch log request failed, we need to
+                    // clear table bucket meta for InvalidMetadataException.
+                    if (fetchResultForBucket.getErrorCode() != Errors.NONE.code()) {
+                        ApiError error = ApiError.fromErrorMessage(respForBucket);
+                        handleFetchLogExceptionForBucket(tb, destination, error);
+                    }
+
                     Long fetchOffset = logScannerStatus.getBucketOffset(tb);
                     // if the offset is null, it means the bucket has been unsubscribed,
                     // we just set a Long.MAX_VALUE as the next fetch offset
@@ -359,8 +387,8 @@ public class LogFetcher implements Closeable {
                             LogRecords logRecords = fetchResultForBucket.recordsOrEmpty();
                             if (!MemoryLogRecords.EMPTY.equals(logRecords)
                                     || fetchResultForBucket.getErrorCode() != Errors.NONE.code()) {
-                                // In oder to not signal notEmptyCondition, add completed fetch to
-                                // buffer until log records is not empty.
+                                // In oder to not signal notEmptyCondition, add completed
+                                // fetch to buffer until log records is not empty.
                                 DefaultCompletedFetch completedFetch =
                                         new DefaultCompletedFetch(
                                                 tb,
@@ -383,6 +411,29 @@ public class LogFetcher implements Closeable {
         }
     }
 
+    private void handleFetchLogExceptionForBucket(TableBucket tb, int destination, ApiError error) {
+        ApiException exception = error.error().exception();
+        LOG.error("Failed to fetch log from node {} for bucket {}", destination, tb, exception);
+        if (exception instanceof InvalidMetadataException) {
+            LOG.warn(
+                    "Invalid metadata error in fetch log request. "
+                            + "Going to request metadata update.",
+                    exception);
+            long tableId = tb.getTableId();
+            TableOrPartitions tableOrPartitions;
+            if (tb.getPartitionId() == null) {
+                tableOrPartitions = new TableOrPartitions(Collections.singleton(tableId), null);
+            } else {
+                tableOrPartitions =
+                        new TableOrPartitions(
+                                null,
+                                Collections.singleton(
+                                        new TablePartition(tableId, tb.getPartitionId())));
+            }
+            invalidTableOrPartitions(tableOrPartitions);
+        }
+    }
+
     private void pendRemoteFetches(
             RemoteLogFetchInfo remoteLogFetchInfo, long firstFetchOffset, long highWatermark) {
         checkNotNull(remoteLogFetchInfo);
@@ -413,7 +464,8 @@ public class LogFetcher implements Closeable {
         }
     }
 
-    private Map<Integer, FetchLogRequest> prepareFetchLogRequests() {
+    @VisibleForTesting
+    Map<Integer, FetchLogRequest> prepareFetchLogRequests() {
         Map<Integer, List<PbFetchLogReqForBucket>> fetchLogReqForBuckets = new HashMap<>();
         int readyForFetchCount = 0;
         Long tableId = null;
@@ -502,8 +554,9 @@ public class LogFetcher implements Closeable {
     }
 
     private Integer getTableBucketLeader(TableBucket tableBucket) {
-        if (metadataUpdater.getBucketLocation(tableBucket).isPresent()) {
-            BucketLocation bucketLocation = metadataUpdater.getBucketLocation(tableBucket).get();
+        Optional<BucketLocation> bucketLocationOpt = metadataUpdater.getBucketLocation(tableBucket);
+        if (bucketLocationOpt.isPresent()) {
+            BucketLocation bucketLocation = bucketLocationOpt.get();
             if (bucketLocation.getLeader() != null) {
                 return bucketLocation.getLeader();
             }
