@@ -20,6 +20,7 @@ package org.apache.fluss.record;
 import org.apache.fluss.annotation.VisibleForTesting;
 import org.apache.fluss.compression.ArrowCompressionInfo;
 import org.apache.fluss.exception.InvalidColumnProjectionException;
+import org.apache.fluss.metadata.SchemaGetter;
 import org.apache.fluss.record.bytesview.MultiBytesView;
 import org.apache.fluss.shaded.arrow.com.google.flatbuffers.FlatBufferBuilder;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.flatbuf.Buffer;
@@ -50,9 +51,7 @@ import java.nio.channels.FileChannel;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.BitSet;
-import java.util.HashMap;
 import java.util.List;
-import java.util.Map;
 
 import static org.apache.fluss.record.DefaultLogRecordBatch.APPEND_ONLY_FLAG_MASK;
 import static org.apache.fluss.record.LogRecordBatchFormat.LENGTH_OFFSET;
@@ -66,9 +65,9 @@ import static org.apache.fluss.record.LogRecordBatchFormat.arrowChangeTypeOffset
 import static org.apache.fluss.record.LogRecordBatchFormat.attributeOffset;
 import static org.apache.fluss.record.LogRecordBatchFormat.recordBatchHeaderSize;
 import static org.apache.fluss.record.LogRecordBatchFormat.recordsCountOffset;
+import static org.apache.fluss.record.LogRecordBatchFormat.schemaIdOffset;
 import static org.apache.fluss.utils.FileUtils.readFully;
 import static org.apache.fluss.utils.FileUtils.readFullyOrFail;
-import static org.apache.fluss.utils.Preconditions.checkNotNull;
 import static org.apache.fluss.utils.Preconditions.checkState;
 
 /** Column projection util on Arrow format {@link FileLogRecords}. */
@@ -82,8 +81,8 @@ public class FileLogProjection {
     private static final int ARROW_HEADER_SIZE =
             ARROW_IPC_CONTINUATION_LENGTH + ARROW_IPC_METADATA_SIZE_LENGTH;
 
-    final Map<Long, ProjectionInfo> projectionsCache = new HashMap<>();
-    ProjectionInfo currentProjection;
+    // the projection cache shared in the TabletServer
+    private final ProjectionPushdownCache projectionsCache;
 
     // shared resources for multiple projections
     private final ByteArrayOutputStream outputStream;
@@ -97,8 +96,13 @@ public class FileLogProjection {
 
     private final ByteBuffer arrowHeaderBuffer = ByteBuffer.allocate(ARROW_HEADER_SIZE);
     private ByteBuffer arrowMetadataBuffer;
+    private SchemaGetter schemaGetter;
+    private long tableId;
+    private ArrowCompressionInfo compressionInfo;
+    private int[] selectedFieldPositions;
 
-    public FileLogProjection() {
+    public FileLogProjection(ProjectionPushdownCache projectionsCache) {
+        this.projectionsCache = projectionsCache;
         this.outputStream = new ByteArrayOutputStream();
         this.writeChannel = new WriteChannel(Channels.newChannel(outputStream));
         // fluss use little endian for encoding log records batch
@@ -109,60 +113,13 @@ public class FileLogProjection {
 
     public void setCurrentProjection(
             long tableId,
-            RowType schema,
+            SchemaGetter schemaGetter,
             ArrowCompressionInfo compressionInfo,
-            int[] selectedFields) {
-        if (projectionsCache.containsKey(tableId)) {
-            // the schema and projection should identical for the same table id.
-            currentProjection = projectionsCache.get(tableId);
-            if (!Arrays.equals(currentProjection.selectedFields, selectedFields)
-                    || !currentProjection.schema.equals(schema)) {
-                throw new InvalidColumnProjectionException(
-                        "The schema and projection should be identical for the same table id.");
-            }
-            return;
-        }
-
-        // initialize the projection util information
-        Schema arrowSchema = ArrowUtils.toArrowSchema(schema);
-        BitSet selection = toBitSet(arrowSchema.getFields().size(), selectedFields);
-        List<Tuple2<Field, Boolean>> flattenedFields = new ArrayList<>();
-        flattenFields(arrowSchema.getFields(), selection, flattenedFields);
-        int totalFieldNodes = flattenedFields.size();
-        int[] bufferLayoutCount = new int[totalFieldNodes];
-        BitSet nodesProjection = new BitSet(totalFieldNodes);
-        int totalBuffers = 0;
-        for (int i = 0; i < totalFieldNodes; i++) {
-            Field fieldNode = flattenedFields.get(i).f0;
-            boolean selected = flattenedFields.get(i).f1;
-            nodesProjection.set(i, selected);
-            bufferLayoutCount[i] = TypeLayout.getTypeBufferCount(fieldNode.getType());
-            totalBuffers += bufferLayoutCount[i];
-        }
-        BitSet buffersProjection = new BitSet(totalBuffers);
-        int bufferIndex = 0;
-        for (int i = 0; i < totalFieldNodes; i++) {
-            if (nodesProjection.get(i)) {
-                buffersProjection.set(bufferIndex, bufferIndex + bufferLayoutCount[i]);
-            }
-            bufferIndex += bufferLayoutCount[i];
-        }
-
-        Schema projectedArrowSchema = ArrowUtils.toArrowSchema(schema.project(selectedFields));
-        ArrowBodyCompression bodyCompression =
-                CompressionUtil.createBodyCompression(compressionInfo.createCompressionCodec());
-        int metadataLength =
-                ArrowUtils.estimateArrowMetadataLength(projectedArrowSchema, bodyCompression);
-        currentProjection =
-                new ProjectionInfo(
-                        nodesProjection,
-                        buffersProjection,
-                        bufferIndex,
-                        schema,
-                        metadataLength,
-                        bodyCompression,
-                        selectedFields);
-        projectionsCache.put(tableId, currentProjection);
+            int[] selectedFieldPositions) {
+        this.tableId = tableId;
+        this.schemaGetter = schemaGetter;
+        this.compressionInfo = compressionInfo;
+        this.selectedFieldPositions = selectedFieldPositions;
     }
 
     /**
@@ -173,15 +130,17 @@ public class FileLogProjection {
      */
     public BytesViewLogRecords project(FileChannel channel, int start, int end, int maxBytes)
             throws IOException {
-        checkNotNull(currentProjection, "There is no projection registered yet.");
+
         MultiBytesView.Builder builder = MultiBytesView.builder();
         int position = start;
 
+        ProjectionInfo currentProjection = null;
+        short prevSchemaId = -1;
         // The condition is an optimization to avoid read log header when there is no enough bytes,
         // So we use V0 header size here for a conservative judgment. In the end, the condition
         // of (position >= end - recordBatchHeaderSize) will ensure the final correctness.
         while (maxBytes > V0_RECORD_BATCH_HEADER_SIZE) {
-            if (position >= end - V0_RECORD_BATCH_HEADER_SIZE) {
+            if (position > end - V0_RECORD_BATCH_HEADER_SIZE) {
                 // the remaining bytes in the file are not enough to read a batch header up to
                 // magic.
                 return new BytesViewLogRecords(builder.build());
@@ -194,15 +153,25 @@ public class FileLogProjection {
             byte magic = logHeaderBuffer.get(MAGIC_OFFSET);
             int recordBatchHeaderSize = recordBatchHeaderSize(magic);
             int batchSizeInBytes = LOG_OVERHEAD + logHeaderBuffer.getInt(LENGTH_OFFSET);
+            short schemaId = logHeaderBuffer.getShort(schemaIdOffset(magic));
+
+            // reuse projection in the current log file
+            if (currentProjection == null || prevSchemaId != schemaId) {
+                prevSchemaId = schemaId;
+                currentProjection = getOrCreateProjectionInfo(schemaId);
+            }
+
             if (position > end - batchSizeInBytes) {
                 // the remaining bytes in the file are not enough to read a full batch
                 return new BytesViewLogRecords(builder.build());
             }
 
-            // Skip empty batch. The empty batch was generated when build cdc log batch when there
+            // Return empty batch to push forward log offset. The empty batch was generated when
+            // build cdc log batch when there
             // is no cdc log generated for this kv batch. See the comments about the field
             // 'lastOffsetDelta' in DefaultLogRecordBatch.
             if (batchSizeInBytes == recordBatchHeaderSize) {
+                builder.addBytes(channel, position, batchSizeInBytes);
                 position += batchSizeInBytes;
                 continue;
             }
@@ -434,35 +403,88 @@ public class FileLogProjection {
         return logHeaderBuffer;
     }
 
-    static final class ProjectionInfo {
+    private ProjectionInfo getOrCreateProjectionInfo(short schemaId) {
+        ProjectionInfo cachedProjection =
+                projectionsCache.getProjectionInfo(tableId, schemaId, selectedFieldPositions);
+        if (cachedProjection == null) {
+            cachedProjection = createProjectionInfo(schemaId, selectedFieldPositions);
+            projectionsCache.setProjectionInfo(
+                    tableId, schemaId, selectedFieldPositions, cachedProjection);
+        }
+        return cachedProjection;
+    }
+
+    private ProjectionInfo createProjectionInfo(short schemaId, int[] selectedFieldPositions) {
+        org.apache.fluss.metadata.Schema schema = schemaGetter.getSchema(schemaId);
+        RowType rowType = schema.getRowType();
+
+        // initialize the projection util information
+        Schema arrowSchema = ArrowUtils.toArrowSchema(rowType);
+        BitSet selection = toBitSet(arrowSchema.getFields().size(), selectedFieldPositions);
+        List<Tuple2<Field, Boolean>> flattenedFields = new ArrayList<>();
+        flattenFields(arrowSchema.getFields(), selection, flattenedFields);
+        int totalFieldNodes = flattenedFields.size();
+        int[] bufferLayoutCount = new int[totalFieldNodes];
+        BitSet nodesProjection = new BitSet(totalFieldNodes);
+        int totalBuffers = 0;
+        for (int i = 0; i < totalFieldNodes; i++) {
+            Field fieldNode = flattenedFields.get(i).f0;
+            boolean selected = flattenedFields.get(i).f1;
+            nodesProjection.set(i, selected);
+            bufferLayoutCount[i] = TypeLayout.getTypeBufferCount(fieldNode.getType());
+            totalBuffers += bufferLayoutCount[i];
+        }
+        BitSet buffersProjection = new BitSet(totalBuffers);
+        int bufferIndex = 0;
+        for (int i = 0; i < totalFieldNodes; i++) {
+            if (nodesProjection.get(i)) {
+                buffersProjection.set(bufferIndex, bufferIndex + bufferLayoutCount[i]);
+            }
+            bufferIndex += bufferLayoutCount[i];
+        }
+
+        Schema projectedArrowSchema =
+                ArrowUtils.toArrowSchema(rowType.project(selectedFieldPositions));
+        ArrowBodyCompression bodyCompression =
+                CompressionUtil.createBodyCompression(compressionInfo.createCompressionCodec());
+        int metadataLength =
+                ArrowUtils.estimateArrowMetadataLength(projectedArrowSchema, bodyCompression);
+        return new ProjectionInfo(
+                nodesProjection,
+                buffersProjection,
+                bufferIndex,
+                metadataLength,
+                bodyCompression,
+                selectedFieldPositions);
+    }
+
+    /** Projection pushdown information for a specific schema and selected fields. */
+    public static final class ProjectionInfo {
         final BitSet nodesProjection;
         final BitSet buffersProjection;
         final int bufferCount;
-        final RowType schema;
         final int arrowMetadataLength;
         final ArrowBodyCompression bodyCompression;
-        final int[] selectedFields;
+        final int[] selectedFieldPositions;
 
         private ProjectionInfo(
                 BitSet nodesProjection,
                 BitSet buffersProjection,
                 int bufferCount,
-                RowType schema,
                 int arrowMetadataLength,
                 ArrowBodyCompression bodyCompression,
-                int[] selectedFields) {
+                int[] selectedFieldPositions) {
             this.nodesProjection = nodesProjection;
             this.buffersProjection = buffersProjection;
             this.bufferCount = bufferCount;
-            this.schema = schema;
             this.arrowMetadataLength = arrowMetadataLength;
             this.bodyCompression = bodyCompression;
-            this.selectedFields = selectedFields;
+            this.selectedFieldPositions = selectedFieldPositions;
         }
     }
 
     /** Metadata of a projected arrow record batch. */
-    public static final class ProjectedArrowBatch {
+    static final class ProjectedArrowBatch {
         /** Number of records. */
         final long numRecords;
 
