@@ -23,12 +23,16 @@ import org.apache.fluss.row.BinaryRow;
 import org.apache.fluss.row.BinarySection;
 import org.apache.fluss.row.BinarySegmentUtils;
 import org.apache.fluss.row.BinaryString;
+import org.apache.fluss.row.DataSetters;
 import org.apache.fluss.row.Decimal;
+import org.apache.fluss.row.InternalArray;
+import org.apache.fluss.row.InternalMap;
 import org.apache.fluss.row.InternalRow;
 import org.apache.fluss.row.NullAwareGetters;
 import org.apache.fluss.row.TimestampLtz;
 import org.apache.fluss.row.TimestampNtz;
-import org.apache.fluss.row.TypedSetters;
+import org.apache.fluss.row.array.AlignedArray;
+import org.apache.fluss.row.map.AlignedMap;
 import org.apache.fluss.types.DataType;
 import org.apache.fluss.types.DecimalType;
 import org.apache.fluss.types.LocalZonedTimestampType;
@@ -66,7 +70,7 @@ import static org.apache.fluss.utils.Preconditions.checkArgument;
  */
 @Internal
 public final class AlignedRow extends BinarySection
-        implements BinaryRow, NullAwareGetters, TypedSetters {
+        implements BinaryRow, NullAwareGetters, DataSetters {
 
     private static final long serialVersionUID = 1L;
 
@@ -343,8 +347,7 @@ public final class AlignedRow extends BinarySection
 
         int fieldOffset = getFieldOffset(pos);
         final long offsetAndSize = segments[0].getLong(fieldOffset);
-        return BinarySegmentUtils.readDecimalData(
-                segments, offset, offsetAndSize, precision, scale);
+        return BinarySegmentUtils.readDecimal(segments, offset, offsetAndSize, precision, scale);
     }
 
     @Override
@@ -355,7 +358,7 @@ public final class AlignedRow extends BinarySection
             return TimestampLtz.fromEpochMillis(segments[0].getLong(getFieldOffset(pos)));
         }
         final long offsetAndNanoOfMilli = segments[0].getLong(fieldOffset);
-        return BinarySegmentUtils.readTimestampLtzData(segments, offset, offsetAndNanoOfMilli);
+        return BinarySegmentUtils.readTimestampLtz(segments, offset, offsetAndNanoOfMilli);
     }
 
     @Override
@@ -366,7 +369,7 @@ public final class AlignedRow extends BinarySection
             return TimestampNtz.fromMillis(segments[0].getLong(fieldOffset));
         }
         final long offsetAndNanoOfMilli = segments[0].getLong(fieldOffset);
-        return BinarySegmentUtils.readTimestampNtzData(segments, offset, offsetAndNanoOfMilli);
+        return BinarySegmentUtils.readTimestampNtz(segments, offset, offsetAndNanoOfMilli);
     }
 
     @Override
@@ -385,6 +388,31 @@ public final class AlignedRow extends BinarySection
         return BinarySegmentUtils.readBinary(segments, offset, fieldOffset, offsetAndLen);
     }
 
+    @Override
+    public InternalArray getArray(int pos) {
+        assertIndexIsValid(pos);
+        int fieldOffset = getFieldOffset(pos);
+        final long offsetAndSize = segments[0].getLong(fieldOffset);
+        return BinarySegmentUtils.readBinaryArray(
+                segments, offset, offsetAndSize, new AlignedArray());
+    }
+
+    @Override
+    public InternalMap getMap(int pos) {
+        assertIndexIsValid(pos);
+        int fieldOffset = getFieldOffset(pos);
+        final long offsetAndSize = segments[0].getLong(fieldOffset);
+        return BinarySegmentUtils.readBinaryMap(segments, offset, offsetAndSize, new AlignedMap());
+    }
+
+    @Override
+    public InternalRow getRow(int pos, int numFields) {
+        assertIndexIsValid(pos);
+        int fieldOffset = getFieldOffset(pos);
+        final long offsetAndSize = segments[0].getLong(fieldOffset);
+        return BinarySegmentUtils.readAlignedRow(segments, offset, offsetAndSize, numFields);
+    }
+
     /** The bit is 1 when the field is null. Default is 0. */
     public boolean anyNull() {
         // Skip the header.
