@@ -21,8 +21,8 @@ import org.apache.fluss.annotation.Internal;
 import org.apache.fluss.compression.ArrowCompressionFactory;
 import org.apache.fluss.exception.FlussRuntimeException;
 import org.apache.fluss.memory.MemorySegment;
-import org.apache.fluss.row.InternalRow;
 import org.apache.fluss.row.arrow.ArrowReader;
+import org.apache.fluss.row.arrow.vectors.ArrowArrayColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowBigIntColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowBinaryColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowBooleanColumnVector;
@@ -31,6 +31,8 @@ import org.apache.fluss.row.arrow.vectors.ArrowDecimalColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowDoubleColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowFloatColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowIntColumnVector;
+import org.apache.fluss.row.arrow.vectors.ArrowMapColumnVector;
+import org.apache.fluss.row.arrow.vectors.ArrowRowColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowSmallIntColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowTimeColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowTimestampLtzColumnVector;
@@ -38,6 +40,7 @@ import org.apache.fluss.row.arrow.vectors.ArrowTimestampNtzColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowTinyIntColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowVarBinaryColumnVector;
 import org.apache.fluss.row.arrow.vectors.ArrowVarCharColumnVector;
+import org.apache.fluss.row.arrow.writers.ArrowArrayWriter;
 import org.apache.fluss.row.arrow.writers.ArrowBigIntWriter;
 import org.apache.fluss.row.arrow.writers.ArrowBinaryWriter;
 import org.apache.fluss.row.arrow.writers.ArrowBooleanWriter;
@@ -47,6 +50,8 @@ import org.apache.fluss.row.arrow.writers.ArrowDoubleWriter;
 import org.apache.fluss.row.arrow.writers.ArrowFieldWriter;
 import org.apache.fluss.row.arrow.writers.ArrowFloatWriter;
 import org.apache.fluss.row.arrow.writers.ArrowIntWriter;
+import org.apache.fluss.row.arrow.writers.ArrowMapWriter;
+import org.apache.fluss.row.arrow.writers.ArrowRowWriter;
 import org.apache.fluss.row.arrow.writers.ArrowSmallIntWriter;
 import org.apache.fluss.row.arrow.writers.ArrowTimeWriter;
 import org.apache.fluss.row.arrow.writers.ArrowTimestampLtzWriter;
@@ -55,6 +60,7 @@ import org.apache.fluss.row.arrow.writers.ArrowTinyIntWriter;
 import org.apache.fluss.row.arrow.writers.ArrowVarBinaryWriter;
 import org.apache.fluss.row.arrow.writers.ArrowVarCharWriter;
 import org.apache.fluss.row.columnar.ColumnVector;
+import org.apache.fluss.row.columnar.VectorizedColumnBatch;
 import org.apache.fluss.shaded.arrow.com.google.flatbuffers.FlatBufferBuilder;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.flatbuf.MessageHeader;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.flatbuf.RecordBatch;
@@ -82,6 +88,9 @@ import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.VarBinaryVector;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.VarCharVector;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.VectorLoader;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.VectorSchemaRoot;
+import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.complex.ListVector;
+import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.complex.MapVector;
+import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.complex.StructVector;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.compression.NoCompressionCodec;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.ipc.ReadChannel;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.ipc.WriteChannel;
@@ -95,16 +104,19 @@ import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.ipc.message.Message
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.types.DateUnit;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.types.FloatingPointPrecision;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.types.TimeUnit;
+import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.types.Types;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.types.pojo.ArrowType;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.types.pojo.Field;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.types.pojo.FieldType;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.types.pojo.Schema;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.vector.util.DataSizeRoundingUtil;
+import org.apache.fluss.types.ArrayType;
 import org.apache.fluss.types.BigIntType;
 import org.apache.fluss.types.BinaryType;
 import org.apache.fluss.types.BooleanType;
 import org.apache.fluss.types.BytesType;
 import org.apache.fluss.types.CharType;
+import org.apache.fluss.types.DataField;
 import org.apache.fluss.types.DataType;
 import org.apache.fluss.types.DataTypeDefaultVisitor;
 import org.apache.fluss.types.DateType;
@@ -113,6 +125,7 @@ import org.apache.fluss.types.DoubleType;
 import org.apache.fluss.types.FloatType;
 import org.apache.fluss.types.IntType;
 import org.apache.fluss.types.LocalZonedTimestampType;
+import org.apache.fluss.types.MapType;
 import org.apache.fluss.types.RowType;
 import org.apache.fluss.types.SmallIntType;
 import org.apache.fluss.types.StringType;
@@ -125,6 +138,7 @@ import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.channels.Channels;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
 import java.util.stream.Collectors;
 
@@ -167,7 +181,8 @@ public class ArrowUtils {
                 columnVectors.add(
                         createArrowColumnVector(fieldVectors.get(i), rowType.getTypeAt(i)));
             }
-            return new ArrowReader(schemaRoot, columnVectors.toArray(new ColumnVector[0]));
+            return new ArrowReader(
+                    columnVectors.toArray(new ColumnVector[0]), schemaRoot.getRowCount());
         } catch (IOException e) {
             throw new RuntimeException("Failed to deserialize ArrowRecordBatch.", e);
         }
@@ -280,56 +295,81 @@ public class ArrowUtils {
         return buffers;
     }
 
-    public static ArrowFieldWriter<InternalRow> createArrowFieldWriter(
-            ValueVector vector, DataType dataType) {
+    public static ArrowFieldWriter createArrowFieldWriter(FieldVector vector, DataType dataType) {
         if (vector instanceof TinyIntVector) {
-            return ArrowTinyIntWriter.forField((TinyIntVector) vector);
+            return new ArrowTinyIntWriter((TinyIntVector) vector);
         } else if (vector instanceof SmallIntVector) {
-            return ArrowSmallIntWriter.forField((SmallIntVector) vector);
+            return new ArrowSmallIntWriter((SmallIntVector) vector);
         } else if (vector instanceof IntVector) {
-            return ArrowIntWriter.forField((IntVector) vector);
+            return new ArrowIntWriter((IntVector) vector);
         } else if (vector instanceof BigIntVector) {
-            return ArrowBigIntWriter.forField((BigIntVector) vector);
+            return new ArrowBigIntWriter((BigIntVector) vector);
         } else if (vector instanceof BitVector) {
-            return ArrowBooleanWriter.forField((BitVector) vector);
+            return new ArrowBooleanWriter((BitVector) vector);
         } else if (vector instanceof Float4Vector) {
-            return ArrowFloatWriter.forField((Float4Vector) vector);
+            return new ArrowFloatWriter((Float4Vector) vector);
         } else if (vector instanceof Float8Vector) {
-            return ArrowDoubleWriter.forField((Float8Vector) vector);
+            return new ArrowDoubleWriter((Float8Vector) vector);
         } else if (vector instanceof VarCharVector) {
-            return ArrowVarCharWriter.forField((VarCharVector) vector);
+            return new ArrowVarCharWriter((VarCharVector) vector);
         } else if (vector instanceof FixedSizeBinaryVector) {
-            return ArrowBinaryWriter.forField((FixedSizeBinaryVector) vector);
+            return new ArrowBinaryWriter((FixedSizeBinaryVector) vector);
         } else if (vector instanceof VarBinaryVector) {
-            return ArrowVarBinaryWriter.forField((VarBinaryVector) vector);
+            return new ArrowVarBinaryWriter((VarBinaryVector) vector);
         } else if (vector instanceof DecimalVector) {
             DecimalVector decimalVector = (DecimalVector) vector;
-            return ArrowDecimalWriter.forField(
+            return new ArrowDecimalWriter(
                     decimalVector, getPrecision(decimalVector), decimalVector.getScale());
         } else if (vector instanceof DateDayVector) {
-            return ArrowDateWriter.forField((DateDayVector) vector);
+            return new ArrowDateWriter((DateDayVector) vector);
         } else if (vector instanceof TimeSecVector
                 || vector instanceof TimeMilliVector
                 || vector instanceof TimeMicroVector
                 || vector instanceof TimeNanoVector) {
-            return ArrowTimeWriter.forField(vector);
+            return new ArrowTimeWriter(vector);
         } else if (vector instanceof TimeStampVector
                 && ((ArrowType.Timestamp) vector.getField().getType()).getTimezone() == null) {
             int precision;
             if (dataType instanceof LocalZonedTimestampType) {
                 precision = ((LocalZonedTimestampType) dataType).getPrecision();
-                return ArrowTimestampLtzWriter.forField(vector, precision);
+                return new ArrowTimestampLtzWriter(vector, precision);
             } else {
                 precision = ((TimestampType) dataType).getPrecision();
-                return ArrowTimestampNtzWriter.forField(vector, precision);
+                return new ArrowTimestampNtzWriter(vector, precision);
             }
+        } else if (vector instanceof ListVector && dataType instanceof ArrayType) {
+            DataType elementType = ((ArrayType) dataType).getElementType();
+            FieldVector elementFieldVector = ((ListVector) vector).getDataVector();
+            return new ArrowArrayWriter(
+                    vector, ArrowUtils.createArrowFieldWriter(elementFieldVector, elementType));
+        } else if (vector instanceof MapVector && dataType instanceof MapType) {
+            MapType mapType = (MapType) dataType;
+            MapVector mapVector = (MapVector) vector;
+            StructVector structVector = (StructVector) mapVector.getDataVector();
+            FieldVector keyVector = structVector.getChild(MapVector.KEY_NAME);
+            FieldVector valueVector = structVector.getChild(MapVector.VALUE_NAME);
+            return new ArrowMapWriter(
+                    vector,
+                    ArrowUtils.createArrowFieldWriter(keyVector, mapType.getKeyType()),
+                    ArrowUtils.createArrowFieldWriter(valueVector, mapType.getValueType()));
+        } else if (vector instanceof StructVector && dataType instanceof RowType) {
+            RowType rowType = (RowType) dataType;
+            StructVector structVector = (StructVector) vector;
+            List<FieldVector> fieldVectors = structVector.getChildrenFromFields();
+            ArrowFieldWriter[] fieldWriters = new ArrowFieldWriter[fieldVectors.size()];
+            for (int i = 0; i < fieldVectors.size(); i++) {
+                fieldWriters[i] =
+                        ArrowUtils.createArrowFieldWriter(
+                                fieldVectors.get(i), rowType.getTypeAt(i));
+            }
+            return new ArrowRowWriter(vector, fieldWriters);
         } else {
             throw new UnsupportedOperationException(
                     String.format("Unsupported type %s.", dataType));
         }
     }
 
-    private static ColumnVector createArrowColumnVector(ValueVector vector, DataType dataType) {
+    public static ColumnVector createArrowColumnVector(ValueVector vector, DataType dataType) {
         if (vector instanceof TinyIntVector) {
             return new ArrowTinyIntColumnVector((TinyIntVector) vector);
         } else if (vector instanceof SmallIntVector) {
@@ -366,6 +406,27 @@ public class ArrowUtils {
             } else {
                 return new ArrowTimestampNtzColumnVector(vector);
             }
+        } else if (vector instanceof ListVector && dataType instanceof ArrayType) {
+            DataType elementType = ((ArrayType) dataType).getElementType();
+            ListVector listVector = (ListVector) vector;
+            return new ArrowArrayColumnVector(
+                    listVector,
+                    ArrowUtils.createArrowColumnVector(listVector.getDataVector(), elementType));
+        } else if (vector instanceof MapVector && dataType instanceof MapType) {
+            MapType mapType = (MapType) dataType;
+            return new ArrowMapColumnVector(
+                    (FieldVector) vector, mapType.getKeyType(), mapType.getValueType());
+        } else if (vector instanceof StructVector && dataType instanceof RowType) {
+            RowType rowType = (RowType) dataType;
+            StructVector structVector = (StructVector) vector;
+            List<FieldVector> fieldVectors = structVector.getChildrenFromFields();
+            ColumnVector[] columnVectors = new ColumnVector[fieldVectors.size()];
+            for (int i = 0; i < fieldVectors.size(); i++) {
+                columnVectors[i] =
+                        ArrowUtils.createArrowColumnVector(
+                                fieldVectors.get(i), rowType.getTypeAt(i));
+            }
+            return new ArrowRowColumnVector(structVector, new VectorizedColumnBatch(columnVectors));
         } else {
             throw new UnsupportedOperationException(
                     String.format("Unsupported type %s.", dataType));
@@ -378,7 +439,31 @@ public class ArrowUtils {
                         logicalType.isNullable(),
                         logicalType.accept(DataTypeToArrowTypeConverter.INSTANCE),
                         null);
-        return new Field(fieldName, fieldType, null);
+        List<Field> children = null;
+        if (logicalType instanceof ArrayType) {
+            children =
+                    Collections.singletonList(
+                            toArrowField("element", ((ArrayType) logicalType).getElementType()));
+        } else if (logicalType instanceof RowType) {
+            RowType rowType = (RowType) logicalType;
+            children = new ArrayList<>(rowType.getFieldCount());
+            for (DataField field : rowType.getFields()) {
+                children.add(toArrowField(field.getName(), field.getType()));
+            }
+        } else if (logicalType instanceof MapType) {
+            MapType mapType = (MapType) logicalType;
+            // Map keys are always non-nullable (enforced by MapType constructor)
+            Field keyField = toArrowField(MapVector.KEY_NAME, mapType.getKeyType());
+            Field valueField = toArrowField(MapVector.VALUE_NAME, mapType.getValueType());
+            FieldType structFieldType = new FieldType(false, ArrowType.Struct.INSTANCE, null);
+            List<Field> structChildren = new ArrayList<>();
+            structChildren.add(keyField);
+            structChildren.add(valueField);
+            Field structField =
+                    new Field(MapVector.DATA_VECTOR_NAME, structFieldType, structChildren);
+            children = Collections.singletonList(structField);
+        }
+        return new Field(fieldName, fieldType, children);
     }
 
     private static class DataTypeToArrowTypeConverter extends DataTypeDefaultVisitor<ArrowType> {
@@ -492,6 +577,21 @@ public class ArrowUtils {
             }
         }
 
+        @Override
+        public ArrowType visit(ArrayType arrayType) {
+            return Types.MinorType.LIST.getType();
+        }
+
+        @Override
+        public ArrowType visit(MapType mapType) {
+            return new ArrowType.Map(false);
+        }
+
+        @Override
+        public ArrowType visit(RowType rowType) {
+            return ArrowType.Struct.INSTANCE;
+        }
+
         @Override
         protected ArrowType defaultMethod(DataType dataType) {
             throw new UnsupportedOperationException(
