@@ -20,17 +20,22 @@ package org.apache.fluss.flink.source.enumerator;
 import org.apache.fluss.client.Connection;
 import org.apache.fluss.client.ConnectionFactory;
 import org.apache.fluss.client.admin.Admin;
+import org.apache.fluss.client.initializer.BucketOffsetsRetrieverImpl;
+import org.apache.fluss.client.initializer.NoStoppingOffsetsInitializer;
+import org.apache.fluss.client.initializer.OffsetsInitializer;
+import org.apache.fluss.client.initializer.OffsetsInitializer.BucketOffsetsRetriever;
+import org.apache.fluss.client.initializer.SnapshotOffsetsInitializer;
 import org.apache.fluss.client.metadata.KvSnapshots;
 import org.apache.fluss.config.ConfigOptions;
 import org.apache.fluss.config.Configuration;
 import org.apache.fluss.flink.lake.LakeSplitGenerator;
-import org.apache.fluss.flink.source.enumerator.initializer.BucketOffsetsRetrieverImpl;
-import org.apache.fluss.flink.source.enumerator.initializer.NoStoppingOffsetsInitializer;
-import org.apache.fluss.flink.source.enumerator.initializer.OffsetsInitializer;
-import org.apache.fluss.flink.source.enumerator.initializer.OffsetsInitializer.BucketOffsetsRetriever;
-import org.apache.fluss.flink.source.enumerator.initializer.SnapshotOffsetsInitializer;
+import org.apache.fluss.flink.lake.split.LakeSnapshotAndFlussLogSplit;
+import org.apache.fluss.flink.lake.split.LakeSnapshotSplit;
+import org.apache.fluss.flink.source.FlinkSource;
+import org.apache.fluss.flink.source.event.FinishedKvSnapshotConsumeEvent;
 import org.apache.fluss.flink.source.event.PartitionBucketsUnsubscribedEvent;
 import org.apache.fluss.flink.source.event.PartitionsRemovedEvent;
+import org.apache.fluss.flink.source.reader.LeaseContext;
 import org.apache.fluss.flink.source.split.HybridSnapshotLogSplit;
 import org.apache.fluss.flink.source.split.LogSplit;
 import org.apache.fluss.flink.source.split.SourceSplitBase;
@@ -38,6 +43,7 @@ import org.apache.fluss.flink.source.state.SourceEnumeratorState;
 import org.apache.fluss.lake.source.LakeSource;
 import org.apache.fluss.lake.source.LakeSplit;
 import org.apache.fluss.metadata.PartitionInfo;
+import org.apache.fluss.metadata.PhysicalTablePath;
 import org.apache.fluss.metadata.TableBucket;
 import org.apache.fluss.metadata.TableInfo;
 import org.apache.fluss.metadata.TablePath;
@@ -69,9 +75,11 @@ import java.util.LinkedHashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.NavigableMap;
 import java.util.Objects;
 import java.util.OptionalLong;
 import java.util.Set;
+import java.util.TreeMap;
 import java.util.stream.Collectors;
 
 import static org.apache.fluss.utils.Preconditions.checkNotNull;
@@ -83,7 +91,8 @@ import static org.apache.fluss.utils.Preconditions.checkState;
  * <p>The enumerator is responsible for:
  *
  * <ul>
- *   <li>Get the all splits(snapshot split + log split) for a table of Fluss to be read.
+ *   <li>Get the all splits(lake split + kv snapshot split + log split) for a table of Fluss to be
+ *       read.
  *   <li>Assign the splits to readers with the guarantee that the splits belong to the same bucket
  *       will be assigned to same reader.
  * </ul>
@@ -110,10 +119,15 @@ public class FlinkSourceEnumerator
      *
      * <p>It's mainly used to help enumerator to broadcast the partition removed event to the
      * readers when partitions is dropped.
+     *
+     * <p>If an assigned partition exists only in the lake and has already expired in Fluss, it will
+     * remain here indefinitely and will not be removed. However, considering that only a small
+     * number of such lake-only partitions might exist during the initial startup, and they consume
+     * minimal memory, this issue is being ignored for now.
      */
     private final Map<Long, String> assignedPartitions;
 
-    /** buckets that have been assigned to readers. */
+    /** Buckets that have been assigned to readers. */
     private final Set<TableBucket> assignedTableBuckets;
 
     @Nullable private List<SourceSplitBase> pendingHybridLakeFlussSplits;
@@ -124,6 +138,11 @@ public class FlinkSourceEnumerator
     private final OffsetsInitializer startingOffsetsInitializer;
     private final OffsetsInitializer stoppingOffsetsInitializer;
 
+    private final LeaseContext leaseContext;
+
+    /** checkpointId -> tableBuckets who finished consume kv snapshots. */
+    private final TreeMap<Long, Set<TableBucket>> consumedKvSnapshotMap = new TreeMap<>();
+
     // Lazily instantiated or mutable fields.
     private Connection connection;
     private Admin flussAdmin;
@@ -138,6 +157,27 @@ public class FlinkSourceEnumerator
 
     private volatile boolean closed = false;
 
+    /**
+     * Whether a checkpoint has been successfully completed before.
+     *
+     * <p>This flag is used in {@link #close()} to decide whether the kv snapshot lease should be
+     * dropped:
+     *
+     * <ul>
+     *   <li>If {@code false} (no checkpoint completed), the lease ID has not been persisted to
+     *       checkpoint state, so it is safe to drop the lease on close — no future restore will
+     *       reference it.
+     *   <li>If {@code true} (at least one checkpoint completed), the lease ID has been persisted
+     *       and may be restored via {@link FlinkSource#restoreEnumerator}. Dropping the lease on
+     *       close would invalidate the restored lease, so it must be kept.
+     * </ul>
+     *
+     * <p>This field is initialized to {@code true} when restoring from a checkpoint (i.e., {@code
+     * assignedTableBuckets} is non-empty), and set to {@code true} in {@link
+     * #notifyCheckpointComplete(long)} upon the first successful checkpoint.
+     */
+    private volatile boolean checkpointTriggeredBefore;
+
     @Nullable private final Predicate partitionFilters;
 
     @Nullable private final LakeSource<LakeSplit> lakeSource;
@@ -152,7 +192,9 @@ public class FlinkSourceEnumerator
             long scanPartitionDiscoveryIntervalMs,
             boolean streaming,
             @Nullable Predicate partitionFilters,
-            @Nullable LakeSource<LakeSplit> lakeSource) {
+            @Nullable LakeSource<LakeSplit> lakeSource,
+            LeaseContext leaseContext,
+            boolean checkpointTriggeredBefore) {
         this(
                 tablePath,
                 flussConf,
@@ -166,7 +208,9 @@ public class FlinkSourceEnumerator
                 scanPartitionDiscoveryIntervalMs,
                 streaming,
                 partitionFilters,
-                lakeSource);
+                lakeSource,
+                leaseContext,
+                checkpointTriggeredBefore);
     }
 
     public FlinkSourceEnumerator(
@@ -182,7 +226,9 @@ public class FlinkSourceEnumerator
             long scanPartitionDiscoveryIntervalMs,
             boolean streaming,
             @Nullable Predicate partitionFilters,
-            @Nullable LakeSource<LakeSplit> lakeSource) {
+            @Nullable LakeSource<LakeSplit> lakeSource,
+            LeaseContext leaseContext,
+            boolean checkpointTriggeredBefore) {
         this(
                 tablePath,
                 flussConf,
@@ -197,7 +243,9 @@ public class FlinkSourceEnumerator
                 streaming,
                 partitionFilters,
                 lakeSource,
-                new WorkerExecutor(context));
+                new WorkerExecutor(context),
+                leaseContext,
+                checkpointTriggeredBefore);
     }
 
     FlinkSourceEnumerator(
@@ -214,7 +262,9 @@ public class FlinkSourceEnumerator
             boolean streaming,
             @Nullable Predicate partitionFilters,
             @Nullable LakeSource<LakeSplit> lakeSource,
-            WorkerExecutor workerExecutor) {
+            WorkerExecutor workerExecutor,
+            LeaseContext leaseContext,
+            boolean checkpointTriggeredBefore) {
         this.tablePath = checkNotNull(tablePath);
         this.flussConf = checkNotNull(flussConf);
         this.hasPrimaryKey = hasPrimaryKey;
@@ -222,12 +272,12 @@ public class FlinkSourceEnumerator
         this.context = checkNotNull(context);
         this.pendingSplitAssignment = new HashMap<>();
         this.assignedTableBuckets = new HashSet<>(assignedTableBuckets);
-        this.startingOffsetsInitializer = startingOffsetsInitializer;
         this.assignedPartitions = new HashMap<>(assignedPartitions);
         this.pendingHybridLakeFlussSplits =
                 pendingHybridLakeFlussSplits == null
                         ? null
                         : new LinkedList<>(pendingHybridLakeFlussSplits);
+        this.startingOffsetsInitializer = startingOffsetsInitializer;
         this.scanPartitionDiscoveryIntervalMs = scanPartitionDiscoveryIntervalMs;
         this.streaming = streaming;
         this.partitionFilters = partitionFilters;
@@ -235,6 +285,8 @@ public class FlinkSourceEnumerator
                 streaming ? new NoStoppingOffsetsInitializer() : OffsetsInitializer.latest();
         this.lakeSource = lakeSource;
         this.workerExecutor = workerExecutor;
+        this.leaseContext = leaseContext;
+        this.checkpointTriggeredBefore = checkpointTriggeredBefore;
     }
 
     @Override
@@ -258,6 +310,10 @@ public class FlinkSourceEnumerator
                     // we'll need to consider lake splits
                     List<SourceSplitBase> hybridLakeFlussSplits = generateHybridLakeFlussSplits();
                     if (hybridLakeFlussSplits != null) {
+                        LOG.info(
+                                "Generated {} hybrid lake splits for table {}.",
+                                hybridLakeFlussSplits.size(),
+                                tablePath);
                         // handle hybrid lake fluss splits firstly
                         handleSplitsAdd(hybridLakeFlussSplits, null);
                     }
@@ -337,16 +393,7 @@ public class FlinkSourceEnumerator
 
     private List<SourceSplitBase> initNonPartitionedSplits() {
         if (hasPrimaryKey && startingOffsetsInitializer instanceof SnapshotOffsetsInitializer) {
-            // get the table snapshot info
-            final KvSnapshots kvSnapshots;
-            try {
-                kvSnapshots = flussAdmin.getLatestKvSnapshots(tablePath).get();
-            } catch (Exception e) {
-                throw new FlinkRuntimeException(
-                        String.format("Failed to get table snapshot for %s", tablePath),
-                        ExceptionUtils.stripCompletionException(e));
-            }
-            return getSnapshotAndLogSplits(kvSnapshots, null);
+            return getSnapshotAndLogSplits(getLatestKvSnapshotsAndRegister(null), null);
         } else {
             return getLogSplit(null, null);
         }
@@ -519,22 +566,77 @@ public class FlinkSourceEnumerator
         List<SourceSplitBase> splits = new ArrayList<>();
         for (Partition partition : newPartitions) {
             String partitionName = partition.getPartitionName();
-            // get the table snapshot info
-            final KvSnapshots kvSnapshots;
-            try {
-                kvSnapshots = flussAdmin.getLatestKvSnapshots(tablePath, partitionName).get();
-            } catch (Exception e) {
-                throw new FlinkRuntimeException(
-                        String.format(
-                                "Failed to get table snapshot for table %s and partition %s",
-                                tablePath, partitionName),
-                        ExceptionUtils.stripCompletionException(e));
-            }
-            splits.addAll(getSnapshotAndLogSplits(kvSnapshots, partitionName));
+            splits.addAll(
+                    getSnapshotAndLogSplits(
+                            getLatestKvSnapshotsAndRegister(partitionName), partitionName));
         }
         return splits;
     }
 
+    private KvSnapshots getLatestKvSnapshotsAndRegister(@Nullable String partitionName) {
+        long tableId;
+        Long partitionId;
+        Map<Integer, Long> snapshotIds = new HashMap<>();
+        Map<Integer, Long> logOffsets = new HashMap<>();
+
+        // Get the latest kv snapshots and acquire kvSnapshot lease.
+        try {
+            KvSnapshots kvSnapshots = getLatestKvSnapshots(partitionName);
+
+            tableId = kvSnapshots.getTableId();
+            partitionId = kvSnapshots.getPartitionId();
+
+            Map<TableBucket, Long> bucketsToLease = new HashMap<>();
+            for (TableBucket tb : kvSnapshots.getTableBuckets()) {
+                int bucket = tb.getBucket();
+                OptionalLong snapshotIdOpt = kvSnapshots.getSnapshotId(bucket);
+                OptionalLong logOffsetOpt = kvSnapshots.getLogOffset(bucket);
+                if (snapshotIdOpt.isPresent() && !ignoreTableBucket(tb)) {
+                    bucketsToLease.put(tb, snapshotIdOpt.getAsLong());
+                }
+
+                snapshotIds.put(
+                        bucket, snapshotIdOpt.isPresent() ? snapshotIdOpt.getAsLong() : null);
+                logOffsets.put(bucket, logOffsetOpt.isPresent() ? logOffsetOpt.getAsLong() : null);
+            }
+
+            if (!bucketsToLease.isEmpty()) {
+                String kvSnapshotLeaseId = leaseContext.getKvSnapshotLeaseId();
+                LOG.info(
+                        "Try to acquire kv snapshot lease {} for table {}",
+                        kvSnapshotLeaseId,
+                        PhysicalTablePath.of(tablePath, partitionName));
+                long kvSnapshotLeaseDurationMs = leaseContext.getKvSnapshotLeaseDurationMs();
+                Set<TableBucket> unavailableTableBucketSet =
+                        flussAdmin
+                                .createKvSnapshotLease(kvSnapshotLeaseId, kvSnapshotLeaseDurationMs)
+                                .acquireSnapshots(bucketsToLease)
+                                .get()
+                                .getUnavailableTableBucketSet();
+                if (!unavailableTableBucketSet.isEmpty()) {
+                    LOG.error(
+                            "Failed to acquire kv snapshot lease for table {}: {}.",
+                            tablePath,
+                            unavailableTableBucketSet);
+                }
+            }
+        } catch (Exception e) {
+            throw new FlinkRuntimeException(
+                    String.format("Failed to get table snapshot for %s", tablePath),
+                    ExceptionUtils.stripCompletionException(e));
+        }
+
+        return new KvSnapshots(tableId, partitionId, snapshotIds, logOffsets);
+    }
+
+    private KvSnapshots getLatestKvSnapshots(@Nullable String partitionName) throws Exception {
+        if (partitionName == null) {
+            return flussAdmin.getLatestKvSnapshots(tablePath).get();
+        } else {
+            return flussAdmin.getLatestKvSnapshots(tablePath, partitionName).get();
+        }
+    }
+
     private List<SourceSplitBase> getSnapshotAndLogSplits(
             KvSnapshots snapshots, @Nullable String partitionName) {
         long tableId = snapshots.getTableId();
@@ -554,7 +656,8 @@ public class FlinkSourceEnumerator
                 // hybrid snapshot log split;
                 OptionalLong logOffset = snapshots.getLogOffset(bucketId);
                 checkState(
-                        logOffset.isPresent(), "Log offset should be present if snapshot id is.");
+                        logOffset.isPresent(),
+                        "Log offset should be present if snapshot id is present.");
                 splits.add(
                         new HybridSnapshotLogSplit(
                                 tb, partitionName, snapshotId.getAsLong(), logOffset.getAsLong()));
@@ -616,6 +719,7 @@ public class FlinkSourceEnumerator
         // should be restored from checkpoint, shouldn't
         // list splits again
         if (pendingHybridLakeFlussSplits != null) {
+            LOG.info("Still have pending lake fluss splits, shouldn't list splits again.");
             return pendingHybridLakeFlussSplits;
         }
         try {
@@ -628,8 +732,16 @@ public class FlinkSourceEnumerator
                             stoppingOffsetsInitializer,
                             tableInfo.getNumBuckets(),
                             this::listPartitions);
-            pendingHybridLakeFlussSplits = lakeSplitGenerator.generateHybridLakeFlussSplits();
-            return pendingHybridLakeFlussSplits;
+            List<SourceSplitBase> generatedSplits =
+                    lakeSplitGenerator.generateHybridLakeFlussSplits();
+            if (generatedSplits == null) {
+                // no hybrid lake splits, set the pending splits to empty list
+                pendingHybridLakeFlussSplits = Collections.emptyList();
+                return null;
+            } else {
+                pendingHybridLakeFlussSplits = generatedSplits;
+                return generatedSplits;
+            }
         } catch (Exception e) {
             throw new FlinkRuntimeException("Failed to generate hybrid lake fluss splits", e);
         }
@@ -656,9 +768,28 @@ public class FlinkSourceEnumerator
         pendingSplitAssignment.forEach(
                 (reader, splits) ->
                         splits.removeIf(
-                                split ->
-                                        removedPartitionsMap.containsKey(
-                                                split.getTableBucket().getPartitionId())));
+                                split -> {
+                                    // Never remove LakeSnapshotSplit, because during union reads,
+                                    // data from the lake must still be read even if the partition
+                                    // has already expired in Fluss.
+                                    if (split instanceof LakeSnapshotSplit) {
+                                        return false;
+                                    }
+
+                                    // Similar to LakeSnapshotSplit, if it contains any lake split,
+                                    // never remove it; otherwise, it can be removed when the Fluss
+                                    // partition expires.
+                                    if (split instanceof LakeSnapshotAndFlussLogSplit) {
+                                        LakeSnapshotAndFlussLogSplit hybridSplit =
+                                                (LakeSnapshotAndFlussLogSplit) split;
+                                        if (!hybridSplit.isLakeSplitFinished()) {
+                                            return false;
+                                        }
+                                    }
+
+                                    return removedPartitionsMap.containsKey(
+                                            split.getTableBucket().getPartitionId());
+                                }));
 
         // send partition removed event to all readers
         PartitionsRemovedEvent event = new PartitionsRemovedEvent(removedPartitionsMap);
@@ -729,16 +860,6 @@ public class FlinkSourceEnumerator
                             TableBucket tableBucket = split.getTableBucket();
                             assignedTableBuckets.add(tableBucket);
 
-                            if (pendingHybridLakeFlussSplits != null) {
-                                // removed from the pendingHybridLakeFlussSplits
-                                // since this split already be assigned
-                                pendingHybridLakeFlussSplits.removeIf(
-                                        hybridLakeFlussSplit ->
-                                                hybridLakeFlussSplit
-                                                        .splitId()
-                                                        .equals(split.splitId()));
-                            }
-
                             if (isPartitioned) {
                                 long partitionId =
                                         checkNotNull(
@@ -751,6 +872,17 @@ public class FlinkSourceEnumerator
                                 assignedPartitions.put(partitionId, partitionName);
                             }
                         });
+
+                if (pendingHybridLakeFlussSplits != null) {
+                    Set<String> splitIdsToRemove =
+                            pendingAssignmentForReader.stream()
+                                    .map(SourceSplitBase::splitId)
+                                    .collect(Collectors.toSet());
+                    // removed from the pendingHybridLakeFlussSplits
+                    // since this split already be assigned
+                    pendingHybridLakeFlussSplits.removeIf(
+                            split -> splitIdsToRemove.contains(split.splitId()));
+                }
             }
         }
 
@@ -847,6 +979,18 @@ public class FlinkSourceEnumerator
             for (Long partitionToRemove : partitionsPendingRemove) {
                 assignedPartitions.remove(partitionToRemove);
             }
+        } else if (sourceEvent instanceof FinishedKvSnapshotConsumeEvent) {
+            FinishedKvSnapshotConsumeEvent event = (FinishedKvSnapshotConsumeEvent) sourceEvent;
+            long checkpointId = event.getCheckpointId();
+            Set<TableBucket> tableBuckets = event.getTableBuckets();
+            if (!tableBuckets.isEmpty()) {
+                LOG.info(
+                        "Received finished kv snapshot consumer event for buckets: {}, checkpoint id: {}",
+                        tableBuckets,
+                        checkpointId);
+            }
+
+            tableBuckets.forEach(tableBucket -> addConsumedBucket(checkpointId, tableBucket));
         }
     }
 
@@ -855,6 +999,11 @@ public class FlinkSourceEnumerator
         return assignedPartitions;
     }
 
+    @VisibleForTesting
+    Map<Integer, List<SourceSplitBase>> getPendingSplitAssignment() {
+        return pendingSplitAssignment;
+    }
+
     @Override
     public void addSplitsBack(List<SourceSplitBase> splits, int subtaskId) {
         LOG.debug("Flink Source Enumerator adds splits back: {}", splits);
@@ -876,21 +1025,74 @@ public class FlinkSourceEnumerator
     public SourceEnumeratorState snapshotState(long checkpointId) {
         final SourceEnumeratorState enumeratorState =
                 new SourceEnumeratorState(
-                        assignedTableBuckets, assignedPartitions, pendingHybridLakeFlussSplits);
+                        assignedTableBuckets,
+                        assignedPartitions,
+                        pendingHybridLakeFlussSplits,
+                        leaseContext.getKvSnapshotLeaseId());
         LOG.debug("Source Checkpoint is {}", enumeratorState);
         return enumeratorState;
     }
 
+    @Override
+    public void notifyCheckpointComplete(long checkpointId) throws Exception {
+        checkpointTriggeredBefore = true;
+
+        // lower than this checkpoint id.
+        Set<TableBucket> consumedKvSnapshots = getAndRemoveConsumedBucketsUpTo(checkpointId);
+
+        LOG.info(
+                "kv snapshot has already consumed and try to release kv snapshot lease for: {}, checkpoint id: {}",
+                consumedKvSnapshots,
+                checkpointId);
+
+        // send request to fluss to unregister the kv snapshot lease.
+        try {
+            flussAdmin
+                    .createKvSnapshotLease(
+                            leaseContext.getKvSnapshotLeaseId(),
+                            leaseContext.getKvSnapshotLeaseDurationMs())
+                    .releaseSnapshots(consumedKvSnapshots)
+                    .get();
+        } catch (Exception e) {
+            LOG.error("Failed to release kv snapshot lease. These snapshot need to re-enqueue", e);
+            // use the current checkpoint id to re-enqueue the buckets
+            consumedKvSnapshots.forEach(
+                    tableBucket -> addConsumedBucket(checkpointId, tableBucket));
+        }
+    }
+
+    /** Add bucket who has been consumed kv snapshot to the consumedKvSnapshotMap. */
+    public void addConsumedBucket(long checkpointId, TableBucket tableBucket) {
+        consumedKvSnapshotMap.computeIfAbsent(checkpointId, k -> new HashSet<>()).add(tableBucket);
+    }
+
+    /** Get and remove the buckets who have been consumed kv snapshot up to the checkpoint id. */
+    public Set<TableBucket> getAndRemoveConsumedBucketsUpTo(long checkpointId) {
+        NavigableMap<Long, Set<TableBucket>> toRemove =
+                consumedKvSnapshotMap.headMap(checkpointId, false);
+        Set<TableBucket> result = new HashSet<>();
+        for (Set<TableBucket> snapshots : toRemove.values()) {
+            result.addAll(snapshots);
+        }
+        toRemove.clear();
+        return result;
+    }
+
     @Override
     public void close() throws IOException {
         try {
+            maybeDropKvSnapshotLease();
+
             closed = true;
+
             if (workerExecutor != null) {
                 workerExecutor.close();
             }
+
             if (flussAdmin != null) {
                 flussAdmin.close();
             }
+
             if (connection != null) {
                 connection.close();
             }
@@ -899,6 +1101,28 @@ public class FlinkSourceEnumerator
         }
     }
 
+    private void maybeDropKvSnapshotLease() throws Exception {
+        if (flussAdmin != null
+                && hasPrimaryKey
+                && startingOffsetsInitializer instanceof SnapshotOffsetsInitializer
+                && !checkpointTriggeredBefore) {
+            // 1. Drop the kv snapshot lease for the batch mode.
+            // 2. For streaming mode, if no checkpoint was triggered, the lease ID
+            // has not been persisted to state. It won't be restored on restart,
+            // so it's safe to drop it now.
+            LOG.info(
+                    "Dropping kv snapshot lease {} when source enumerator close. isStreaming {}",
+                    leaseContext.getKvSnapshotLeaseId(),
+                    streaming);
+            flussAdmin
+                    .createKvSnapshotLease(
+                            leaseContext.getKvSnapshotLeaseId(),
+                            leaseContext.getKvSnapshotLeaseDurationMs())
+                    .dropLease()
+                    .get();
+        }
+    }
+
     // --------------- private class ---------------
     /** A container class to hold the newly added partitions and removed partitions. */
     private static class PartitionChange {
