@@ -22,8 +22,6 @@ import org.apache.fluss.client.ConnectionFactory;
 import org.apache.fluss.client.admin.Admin;
 import org.apache.fluss.client.admin.ListOffsetsResult;
 import org.apache.fluss.client.admin.OffsetSpec;
-import org.apache.fluss.client.admin.OffsetSpec.EarliestSpec;
-import org.apache.fluss.client.admin.OffsetSpec.LatestSpec;
 import org.apache.fluss.client.table.Table;
 import org.apache.fluss.client.table.scanner.Scan;
 import org.apache.fluss.client.table.scanner.batch.BatchScanUtils;
@@ -31,12 +29,14 @@ import org.apache.fluss.client.table.scanner.batch.BatchScanner;
 import org.apache.fluss.client.table.writer.UpsertWriter;
 import org.apache.fluss.config.Configuration;
 import org.apache.fluss.exception.FlussRuntimeException;
+import org.apache.fluss.exception.UnsupportedVersionException;
 import org.apache.fluss.flink.source.lookup.FlinkLookupFunction;
 import org.apache.fluss.flink.source.lookup.LookupNormalizer;
 import org.apache.fluss.metadata.PartitionInfo;
 import org.apache.fluss.metadata.TableBucket;
 import org.apache.fluss.metadata.TableInfo;
 import org.apache.fluss.metadata.TablePath;
+import org.apache.fluss.metadata.TableStats;
 import org.apache.fluss.row.BinaryString;
 import org.apache.fluss.row.Decimal;
 import org.apache.fluss.row.GenericRow;
@@ -76,6 +76,7 @@ import java.util.stream.Collectors;
 import java.util.stream.IntStream;
 
 import static org.apache.fluss.flink.source.lookup.LookupNormalizer.createPrimaryKeyLookupNormalizer;
+import static org.apache.fluss.utils.ExceptionUtils.findThrowable;
 
 /** Utilities for pushdown abilities. */
 public class PushdownUtils {
@@ -262,7 +263,6 @@ public class PushdownUtils {
             Configuration flussConfig,
             RowType sourceOutputType,
             int[] primaryKeyIndexes,
-            int lookupMaxRetryTimes,
             @Nullable int[] projectedFields) {
         LookupNormalizer lookupNormalizer =
                 createPrimaryKeyLookupNormalizer(primaryKeyIndexes, sourceOutputType);
@@ -271,9 +271,9 @@ public class PushdownUtils {
                         flussConfig,
                         tablePath,
                         sourceOutputType,
-                        lookupMaxRetryTimes,
                         lookupNormalizer,
-                        projectedFields);
+                        projectedFields,
+                        false);
         try {
             // it's fine to pass null here, as we don't use it in it
             lookupFunction.open(null);
@@ -366,34 +366,57 @@ public class PushdownUtils {
         }
     }
 
-    public static long countLogTable(TablePath tablePath, Configuration flussConfig) {
+    public static long countTable(TablePath tablePath, Configuration flussConfig) {
         try (Connection connection = ConnectionFactory.createConnection(flussConfig);
                 Admin flussAdmin = connection.getAdmin()) {
-            TableInfo tableInfo = flussAdmin.getTableInfo(tablePath).get();
-            int bucketCount = tableInfo.getNumBuckets();
-            Collection<Integer> buckets =
-                    IntStream.range(0, bucketCount).boxed().collect(Collectors.toList());
-            List<PartitionInfo> partitionInfos;
-            if (tableInfo.isPartitioned()) {
-                partitionInfos = flussAdmin.listPartitionInfos(tablePath).get();
-            } else {
-                partitionInfos = Collections.singletonList(null);
+            try {
+                TableStats tableStats = flussAdmin.getTableStats(tablePath).get();
+                return tableStats.getRowCount();
+            } catch (Exception e) {
+                if (findThrowable(e, UnsupportedVersionException.class).isPresent()) {
+                    // if the server doesn't support getTableStats, we fallback to countLogTable,
+                    // which is less efficient.
+                    return countLogTable(flussAdmin, tablePath);
+                } else {
+                    throw e;
+                }
             }
-
-            List<CompletableFuture<Long>> countFutureList =
-                    offsetLengthes(flussAdmin, tablePath, partitionInfos, buckets);
-            // wait for all the response
-            CompletableFuture.allOf(countFutureList.toArray(new CompletableFuture[0])).join();
-            long count = 0;
-            for (CompletableFuture<Long> countFuture : countFutureList) {
-                count += countFuture.get();
-            }
-            return count;
         } catch (Exception e) {
             throw new FlussRuntimeException(e);
         }
     }
 
+    /**
+     * We keep this method for back compatibility for old clusters (< 0.9), but it's not efficient.
+     * We should use countTable instead in the future.
+     */
+    private static long countLogTable(Admin flussAdmin, TablePath tablePath) throws Exception {
+        TableInfo tableInfo = flussAdmin.getTableInfo(tablePath).get();
+        if (tableInfo.hasPrimaryKey()) {
+            throw new IllegalArgumentException(
+                    "The Fluss cluster doesn't support count(*) on primary key table yet. Please upgrade to newer version (≥ 0.9).");
+        }
+        int bucketCount = tableInfo.getNumBuckets();
+        Collection<Integer> buckets =
+                IntStream.range(0, bucketCount).boxed().collect(Collectors.toList());
+        List<PartitionInfo> partitionInfos;
+        if (tableInfo.isPartitioned()) {
+            partitionInfos = flussAdmin.listPartitionInfos(tablePath).get();
+        } else {
+            partitionInfos = Collections.singletonList(null);
+        }
+
+        List<CompletableFuture<Long>> countFutureList =
+                offsetLengthes(flussAdmin, tablePath, partitionInfos, buckets);
+        // wait for all the response
+        CompletableFuture.allOf(countFutureList.toArray(new CompletableFuture[0])).join();
+        long count = 0;
+        for (CompletableFuture<Long> countFuture : countFutureList) {
+            count += countFuture.get();
+        }
+        return count;
+    }
+
     private static List<CompletableFuture<Long>> offsetLengthes(
             Admin flussAdmin,
             TablePath tablePath,
@@ -403,9 +426,19 @@ public class PushdownUtils {
         for (@Nullable PartitionInfo info : partitionInfos) {
             String partitionName = info != null ? info.getPartitionName() : null;
             ListOffsetsResult earliestOffsets =
-                    listOffsets(flussAdmin, tablePath, buckets, new EarliestSpec(), partitionName);
+                    listOffsets(
+                            flussAdmin,
+                            tablePath,
+                            buckets,
+                            new OffsetSpec.EarliestSpec(),
+                            partitionName);
             ListOffsetsResult latestOffsets =
-                    listOffsets(flussAdmin, tablePath, buckets, new LatestSpec(), partitionName);
+                    listOffsets(
+                            flussAdmin,
+                            tablePath,
+                            buckets,
+                            new OffsetSpec.LatestSpec(),
+                            partitionName);
             CompletableFuture<Long> apply =
                     earliestOffsets
                             .all()
