@@ -22,6 +22,8 @@ import org.apache.fluss.config.Configuration;
 import org.apache.fluss.exception.IllegalConfigurationException;
 import org.apache.fluss.exception.InvalidPartitionException;
 import org.apache.fluss.exception.InvalidTableException;
+import org.apache.fluss.flink.adapter.ResolvedCatalogMaterializedTableAdapter;
+import org.apache.fluss.flink.lake.LakeFlinkCatalog;
 import org.apache.fluss.flink.utils.FlinkConversionsTest;
 import org.apache.fluss.server.testutils.FlussClusterExtension;
 import org.apache.fluss.utils.ExceptionUtils;
@@ -36,9 +38,9 @@ import org.apache.flink.table.catalog.CatalogMaterializedTable;
 import org.apache.flink.table.catalog.CatalogPartitionSpec;
 import org.apache.flink.table.catalog.CatalogTable;
 import org.apache.flink.table.catalog.Column;
+import org.apache.flink.table.catalog.GenericInMemoryCatalog;
 import org.apache.flink.table.catalog.IntervalFreshness;
 import org.apache.flink.table.catalog.ObjectPath;
-import org.apache.flink.table.catalog.ResolvedCatalogMaterializedTable;
 import org.apache.flink.table.catalog.ResolvedCatalogTable;
 import org.apache.flink.table.catalog.ResolvedSchema;
 import org.apache.flink.table.catalog.TableChange;
@@ -58,8 +60,7 @@ import org.apache.flink.table.expressions.ResolvedExpression;
 import org.apache.flink.table.expressions.utils.ResolvedExpressionMock;
 import org.apache.flink.table.factories.Factory;
 import org.apache.flink.table.factories.FactoryUtil;
-import org.junit.jupiter.api.AfterAll;
-import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.extension.RegisterExtension;
@@ -78,6 +79,7 @@ import static org.apache.fluss.config.ConfigOptions.TABLE_DATALAKE_FORMAT;
 import static org.apache.fluss.flink.FlinkConnectorOptions.BUCKET_KEY;
 import static org.apache.fluss.flink.FlinkConnectorOptions.BUCKET_NUMBER;
 import static org.apache.fluss.flink.FlinkConnectorOptions.SCAN_STARTUP_MODE;
+import static org.apache.fluss.flink.adapter.CatalogTableAdapter.toCatalogTable;
 import static org.apache.fluss.flink.utils.CatalogTableTestUtils.addOptions;
 import static org.apache.fluss.flink.utils.CatalogTableTestUtils.checkEqualsIgnoreSchema;
 import static org.apache.fluss.flink.utils.CatalogTableTestUtils.checkEqualsRespectSchema;
@@ -101,7 +103,8 @@ class FlinkCatalogTest {
     private static final FlinkConversionsTest.TestRefreshHandler REFRESH_HANDLER =
             new FlinkConversionsTest.TestRefreshHandler("jobID: xxx, clusterId: yyy");
 
-    static Catalog catalog;
+    private Catalog catalog;
+    private MockLakeFlinkCatalog mockLakeCatalog;
     private final ObjectPath tableInDefaultDb = new ObjectPath(DEFAULT_DB, "t1");
 
     private static Configuration initConfig() {
@@ -110,7 +113,7 @@ class FlinkCatalogTest {
         return configuration;
     }
 
-    private ResolvedSchema createSchema() {
+    protected ResolvedSchema createSchema() {
         return new ResolvedSchema(
                 Arrays.asList(
                         Column.physical("first", DataTypes.STRING().notNull()),
@@ -128,7 +131,7 @@ class FlinkCatalogTest {
     private CatalogTable newCatalogTable(
             ResolvedSchema resolvedSchema, Map<String, String> options) {
         CatalogTable origin =
-                CatalogTable.of(
+                toCatalogTable(
                         Schema.newBuilder().fromResolvedSchema(resolvedSchema).build(),
                         "test comment",
                         Collections.emptyList(),
@@ -155,32 +158,44 @@ class FlinkCatalogTest {
                         .refreshMode(refreshMode)
                         .refreshStatus(CatalogMaterializedTable.RefreshStatus.INITIALIZING)
                         .build();
-        return new ResolvedCatalogMaterializedTable(origin, resolvedSchema);
+        return ResolvedCatalogMaterializedTableAdapter.create(
+                origin,
+                resolvedSchema,
+                refreshMode,
+                IntervalFreshness.of("5", IntervalFreshness.TimeUnit.SECOND));
     }
 
-    @BeforeAll
-    static void beforeAll() {
-        // set fluss conf
-        Configuration flussConf = FLUSS_CLUSTER_EXTENSION.getClientConfig();
-        catalog =
-                new FlinkCatalog(
-                        CATALOG_NAME,
-                        DEFAULT_DB,
-                        String.join(",", flussConf.get(BOOTSTRAP_SERVERS)),
-                        Thread.currentThread().getContextClassLoader(),
-                        Collections.emptyMap());
-        catalog.open();
-    }
-
-    @AfterAll
-    static void afterAll() {
-        if (catalog != null) {
-            catalog.close();
-        }
+    protected FlinkCatalog initCatalog(
+            String catalogName,
+            String databaseName,
+            String bootstrapServers,
+            LakeFlinkCatalog lakeFlinkCatalog) {
+        return new FlinkCatalog(
+                catalogName,
+                databaseName,
+                bootstrapServers,
+                Thread.currentThread().getContextClassLoader(),
+                Collections.emptyMap(),
+                Collections::emptyMap,
+                lakeFlinkCatalog);
     }
 
     @BeforeEach
     void beforeEach() throws Exception {
+        // set fluss conf
+        Configuration flussConf = FLUSS_CLUSTER_EXTENSION.getClientConfig();
+
+        mockLakeCatalog =
+                new MockLakeFlinkCatalog(
+                        CATALOG_NAME, Thread.currentThread().getContextClassLoader());
+        catalog =
+                initCatalog(
+                        CATALOG_NAME,
+                        DEFAULT_DB,
+                        String.join(",", flussConf.get(BOOTSTRAP_SERVERS)),
+                        mockLakeCatalog);
+        catalog.open();
+
         // First check if database exists, and drop it if it does
         if (catalog.databaseExists(DEFAULT_DB)) {
             catalog.dropDatabase(DEFAULT_DB, true, true);
@@ -198,6 +213,13 @@ class FlinkCatalogTest {
         }
     }
 
+    @AfterEach
+    void afterEach() {
+        if (catalog != null) {
+            catalog.close();
+        }
+    }
+
     @Test
     void testCreateTable() throws Exception {
         Map<String, String> options = new HashMap<>();
@@ -270,7 +292,7 @@ class FlinkCatalogTest {
         ResolvedSchema resolvedSchema = this.createSchema();
         CatalogTable table2 =
                 new ResolvedCatalogTable(
-                        CatalogTable.of(
+                        toCatalogTable(
                                 Schema.newBuilder().fromResolvedSchema(resolvedSchema).build(),
                                 "test comment",
                                 Collections.singletonList("first"),
@@ -306,15 +328,16 @@ class FlinkCatalogTest {
         CatalogTable table = this.newCatalogTable(options);
         catalog.createTable(lakeTablePath, table, false);
         assertThat(catalog.tableExists(lakeTablePath)).isTrue();
+        // get the lake table from lake catalog.
+        mockLakeCatalog.registerLakeTable(lakeTablePath, table);
+        assertThat((CatalogTable) catalog.getTable(new ObjectPath(DEFAULT_DB, "lake_table$lake")))
+                .isEqualTo(table);
+
         // drop fluss table
         catalog.dropTable(lakeTablePath, false);
-        // create the table again, should throw exception with ignore if exist = false
-        assertThatThrownBy(() -> catalog.createTable(lakeTablePath, table, false))
-                .isInstanceOf(CatalogException.class)
-                .hasMessage(
-                        String.format(
-                                "The table %s already exists in %s catalog, please first drop the table in %s catalog or use a new table name.",
-                                lakeTablePath, "paimon", "paimon"));
+        assertThat(catalog.tableExists(lakeTablePath)).isFalse();
+        // create the table again should be ok, because the existing lake table is matched
+        catalog.createTable(lakeTablePath, table, false);
     }
 
     @Test
@@ -367,7 +390,7 @@ class FlinkCatalogTest {
                         UniqueConstraint.primaryKey(
                                 "PK_first", Collections.singletonList("first")));
         CatalogTable origin =
-                CatalogTable.of(
+                toCatalogTable(
                         Schema.newBuilder().fromResolvedSchema(resolvedSchema).build(),
                         "test comment",
                         Collections.emptyList(),
@@ -627,7 +650,8 @@ class FlinkCatalogTest {
                                                 ",",
                                                 flussConf.get(ConfigOptions.BOOTSTRAP_SERVERS)),
                                         Thread.currentThread().getContextClassLoader(),
-                                        Collections.emptyMap()))
+                                        Collections.emptyMap(),
+                                        Collections::emptyMap))
                 .hasMessageContaining("defaultDatabase cannot be null or empty");
     }
 
@@ -652,7 +676,7 @@ class FlinkCatalogTest {
         ResolvedSchema resolvedSchema = this.createSchema();
         CatalogTable table2 =
                 new ResolvedCatalogTable(
-                        CatalogTable.of(
+                        toCatalogTable(
                                 Schema.newBuilder().fromResolvedSchema(resolvedSchema).build(),
                                 "test comment",
                                 Collections.singletonList("first"),
@@ -762,7 +786,7 @@ class FlinkCatalogTest {
         ObjectPath partitionedPath = new ObjectPath(DEFAULT_DB, "partitioned_table1");
         CatalogTable partitionedTable =
                 new ResolvedCatalogTable(
-                        CatalogTable.of(
+                        toCatalogTable(
                                 Schema.newBuilder().fromResolvedSchema(resolvedSchema).build(),
                                 "test comment",
                                 Collections.singletonList("first"),
@@ -815,7 +839,8 @@ class FlinkCatalogTest {
                         "default",
                         "invalid-bootstrap-server:9092",
                         Thread.currentThread().getContextClassLoader(),
-                        Collections.emptyMap());
+                        Collections.emptyMap(),
+                        Collections::emptyMap);
 
         // Test open() throws proper exception
         assertThatThrownBy(() -> badCatalog.open())
@@ -849,7 +874,7 @@ class FlinkCatalogTest {
         ResolvedSchema schema = createSchema();
         CatalogTable partTable =
                 new ResolvedCatalogTable(
-                        CatalogTable.of(
+                        toCatalogTable(
                                 Schema.newBuilder().fromResolvedSchema(schema).build(),
                                 "partitioned table for stats",
                                 Collections.singletonList("first"),
@@ -945,7 +970,8 @@ class FlinkCatalogTest {
                         DEFAULT_DB,
                         String.join(",", flussConf.get(BOOTSTRAP_SERVERS)),
                         Thread.currentThread().getContextClassLoader(),
-                        securityConfigs);
+                        securityConfigs,
+                        Collections::emptyMap);
         securedCatalog.open();
 
         try {
@@ -978,4 +1004,24 @@ class FlinkCatalogTest {
         checkEqualsRespectSchema((CatalogTable) tableCreated, table);
         catalog.dropTable(tablePath, false);
     }
+
+    private static class MockLakeFlinkCatalog extends LakeFlinkCatalog {
+        private final GenericInMemoryCatalog catalog;
+
+        public MockLakeFlinkCatalog(String catalogName, ClassLoader classLoader) {
+            super(catalogName, classLoader);
+            catalog = new GenericInMemoryCatalog(catalogName, DEFAULT_DB);
+        }
+
+        @Override
+        public Catalog getLakeCatalog(
+                Configuration tableOptions, Map<String, String> lakeCatalogProperties) {
+            return catalog;
+        }
+
+        void registerLakeTable(ObjectPath tablePath, CatalogTable table)
+                throws TableAlreadyExistException, DatabaseNotExistException {
+            catalog.createTable(tablePath, table, false);
+        }
+    }
 }
