@@ -20,6 +20,7 @@ package org.apache.fluss.flink.source;
 import org.apache.fluss.client.table.Table;
 import org.apache.fluss.client.table.writer.AppendWriter;
 import org.apache.fluss.client.table.writer.UpsertWriter;
+import org.apache.fluss.exception.InvalidTableException;
 import org.apache.fluss.flink.utils.FlinkTestBase;
 import org.apache.fluss.metadata.TablePath;
 
@@ -91,8 +92,8 @@ abstract class FlinkTableSourceBatchITCase extends FlinkTestBase {
                 .contains(
                         String.format(
                                 "TableSourceScan(table=[[testcatalog, defaultdb, %s, "
-                                        + "filter=[and(=(id, 1), =(name, _UTF-16LE'name1':VARCHAR(2147483647) CHARACTER SET \"UTF-16LE\"))], "
-                                        + "project=[address]]], fields=[address])",
+                                        + "filter=[and(=(id, 1), =(name, _UTF-16LE'name1':VARCHAR(2147483647) CHARACTER SET \"UTF-16LE\"))]]], "
+                                        + "fields=[id, address, name])",
                                 tableName));
         CloseableIterator<Row> collected = tEnv.executeSql(query).collect();
         List<String> expected = Collections.singletonList("+I[1, address1, name1]");
@@ -108,8 +109,8 @@ abstract class FlinkTableSourceBatchITCase extends FlinkTestBase {
                 .contains(
                         String.format(
                                 "TableSourceScan(table=[[testcatalog, defaultdb, %s, "
-                                        + "filter=[and(=(id, 1), =(name, _UTF-16LE'name1':VARCHAR(2147483647) CHARACTER SET \"UTF-16LE\"))], "
-                                        + "project=[address]]], fields=[address])",
+                                        + "filter=[and(=(id, 1), =(name, _UTF-16LE'name1':VARCHAR(2147483647) CHARACTER SET \"UTF-16LE\"))]]], "
+                                        + "fields=[id, address, name])",
                                 tableName));
         CloseableIterator<Row> collected = tEnv.executeSql(query).collect();
         List<String> expected = Collections.singletonList("+I[1, address1, name1]");
@@ -126,7 +127,7 @@ abstract class FlinkTableSourceBatchITCase extends FlinkTestBase {
                         String.format(
                                 "TableSourceScan(table=[[testcatalog, defaultdb, %s, "
                                         + "filter=[=(id, 1)], "
-                                        + "project=[name]]], fields=[name])",
+                                        + "project=[id, name]]], fields=[id, name])",
                                 tableName));
         CloseableIterator<Row> collected = tEnv.executeSql(query).collect();
         List<String> expected = Collections.singletonList("+I[1, name1]");
@@ -149,8 +150,8 @@ abstract class FlinkTableSourceBatchITCase extends FlinkTestBase {
                 .contains(
                         String.format(
                                 "TableSourceScan(table=[[testcatalog, defaultdb, %s, "
-                                        + "filter=[and(=(id, 1), =(dt, _UTF-16LE'%s':VARCHAR(2147483647) CHARACTER SET \"UTF-16LE\"))], "
-                                        + "project=[address, name]]], fields=[address, name])\n",
+                                        + "filter=[and(=(id, 1), =(dt, _UTF-16LE'%s':VARCHAR(2147483647) CHARACTER SET \"UTF-16LE\"))]]], "
+                                        + "fields=[id, address, name, dt])\n",
                                 tableName, partition1));
 
         CloseableIterator<Row> collected = tEnv.executeSql(query).collect();
@@ -159,6 +160,84 @@ abstract class FlinkTableSourceBatchITCase extends FlinkTestBase {
         assertResultsIgnoreOrder(collected, expected, true);
     }
 
+    @Test
+    void testFilterOnLookupSource() throws Exception {
+        String srcTableName = String.format("test_src_table_%s", RandomUtils.nextInt());
+        tEnv.executeSql(
+                String.format(
+                        "create table %s ("
+                                + "  id int not null,"
+                                + "  name varchar,"
+                                + "  dt varchar,"
+                                + "  dim_dt varchar,"
+                                + "  primary key (id, dt) NOT ENFORCED) partitioned by (dt)"
+                                + " with ("
+                                + "  'bucket.num' = '4', "
+                                + "  'table.auto-partition.enabled' = 'true',"
+                                + "  'table.auto-partition.time-unit' = 'year')",
+                        srcTableName));
+
+        String dimTableName = String.format("test_dim_table_%s", RandomUtils.nextInt());
+        tEnv.executeSql(
+                String.format(
+                        "create table %s ("
+                                + "  id int not null,"
+                                + "  address varchar,"
+                                + "  dt varchar,"
+                                + "  primary key (id, dt) NOT ENFORCED) partitioned by (dt)"
+                                + " with ("
+                                + "  'bucket.num' = '4', "
+                                + "  'table.auto-partition.enabled' = 'true',"
+                                + "  'table.auto-partition.time-unit' = 'year')",
+                        dimTableName));
+
+        TablePath srcTablePath = TablePath.of(DEFAULT_DB, srcTableName);
+        Map<Long, String> partitionNameById =
+                waitUntilPartitions(FLUSS_CLUSTER_EXTENSION.getZooKeeperClient(), srcTablePath);
+        // just pick first partition to insert data
+        Iterator<String> partitionIterator =
+                partitionNameById.values().stream().sorted().iterator();
+        String partition1 = partitionIterator.next();
+
+        // prepare src table data
+        try (Table srcTable = conn.getTable(srcTablePath)) {
+            UpsertWriter upsertWriter = srcTable.newUpsert().createWriter();
+            for (int i = 1; i <= 2; i++) {
+                Object[] values = new Object[] {i, "name" + i, partition1, partition1};
+                upsertWriter.upsert(row(values));
+            }
+            upsertWriter.flush();
+        }
+
+        TablePath dimTablePath = TablePath.of(DEFAULT_DB, dimTableName);
+        // prepare dim table data
+        try (Table dimTable = conn.getTable(dimTablePath)) {
+            UpsertWriter upsertWriter = dimTable.newUpsert().createWriter();
+            for (int i = 1; i <= 2; i++) {
+                Object[] values = new Object[] {i, "address" + i, partition1};
+                upsertWriter.upsert(row(values));
+            }
+            upsertWriter.flush();
+        }
+
+        tEnv.executeSql(
+                String.format(
+                        "CREATE TEMPORARY VIEW my_view AS "
+                                + "SELECT *, proctime() as proc from %s WHERE id = 1 AND dt = '%s'",
+                        srcTableName, partition1));
+
+        CloseableIterator<Row> collected =
+                tEnv.executeSql(
+                                String.format(
+                                        "SELECT src.id, src.name, h.id, h.address FROM my_view src "
+                                                + " LEFT JOIN %s FOR SYSTEM_TIME AS OF src.proc as h "
+                                                + " ON src.id = h.id and src.dim_dt = h.dt and h.dt <> '%s'",
+                                        dimTableName, partition1))
+                        .collect();
+        List<String> expected = Collections.singletonList("+I[1, name1, null, null]");
+        assertResultsIgnoreOrder(collected, expected, true);
+    }
+
     @Test
     void testScanSingleRowFilterException() throws Exception {
         String tableName = prepareSourceTable(new String[] {"id", "name"}, null);
@@ -274,21 +353,19 @@ abstract class FlinkTableSourceBatchITCase extends FlinkTestBase {
 
     @ParameterizedTest
     @ValueSource(booleans = {true, false})
-    void testCountPushDown(boolean partitionTable) throws Exception {
-        String tableName = partitionTable ? preparePartitionedLogTable() : prepareLogTable();
-        int expectedRows = partitionTable ? 10 : 5;
+    void testCountPushDownForPkTable(boolean partitionTable) throws Exception {
+        String tableName =
+                partitionTable
+                        ? prepareSourceTable(new String[] {"id", "dt"}, "dt")
+                        : prepareSourceTable(new String[] {"id"}, null);
         // normal scan
         String query = String.format("SELECT COUNT(*) FROM %s", tableName);
         assertThat(tEnv.explainSql(query))
                 .contains(
-                        String.format(
-                                "TableSourceScan(table=[[testcatalog, defaultdb, %s, project=[id], "
-                                        + "aggregates=[grouping=[], aggFunctions=[Count1AggFunction()]]]], "
-                                        + "fields=[count1$0])",
-                                tableName));
+                        "aggregates=[grouping=[], aggFunctions=[Count1AggFunction()]]]], fields=[count1$0]");
         CloseableIterator<Row> iterRows = tEnv.executeSql(query).collect();
         List<String> collected = collectRowsWithTimeout(iterRows, 1);
-        List<String> expected = Collections.singletonList(String.format("+I[%s]", expectedRows));
+        List<String> expected = Collections.singletonList("+I[5]");
         assertThat(collected).isEqualTo(expected);
 
         // test not push down grouping count.
@@ -301,15 +378,51 @@ abstract class FlinkTableSourceBatchITCase extends FlinkTestBase {
                                         .wait())
                 .hasMessageContaining(
                         "Currently, Fluss only support queries on table with datalake enabled or point queries on primary key when it's in batch execution mode.");
+    }
 
-        // test not support primary key now
-        String primaryTableName = prepareSourceTable(new String[] {"id"}, null);
+    @Test
+    void testCountPushDownWithWALMode() throws Exception {
+        String tableName = "test_count_table_with_wal";
+        tEnv.executeSql(
+                        String.format(
+                                "create table %s ("
+                                        + "  id int not null,"
+                                        + "  address varchar,"
+                                        + "  name varchar,"
+                                        + "  primary key (id) NOT ENFORCED)"
+                                        + " with ('bucket.num' = '4', 'table.changelog.image' = 'wal')",
+                                tableName))
+                .await();
+        // normal scan
+        String query = String.format("SELECT COUNT(*) FROM %s", tableName);
+        assertThatThrownBy(() -> tEnv.executeSql(query))
+                .hasRootCauseInstanceOf(InvalidTableException.class)
+                .hasMessageContaining(
+                        "Row count is disabled for this table 'defaultdb.test_count_table_with_wal'.");
+    }
+
+    @ParameterizedTest
+    @ValueSource(booleans = {true, false})
+    void testCountPushDownForLogTable(boolean partitionTable) throws Exception {
+        String tableName = partitionTable ? preparePartitionedLogTable() : prepareLogTable();
+        int expectedRows = partitionTable ? 10 : 5;
+        // normal scan
+        String query = String.format("SELECT COUNT(*) FROM %s", tableName);
+        assertThat(tEnv.explainSql(query))
+                .contains(
+                        "aggregates=[grouping=[], aggFunctions=[Count1AggFunction()]]]], fields=[count1$0]");
+        CloseableIterator<Row> iterRows = tEnv.executeSql(query).collect();
+        List<String> collected = collectRowsWithTimeout(iterRows, 1);
+        List<String> expected = Collections.singletonList(String.format("+I[%s]", expectedRows));
+        assertThat(collected).isEqualTo(expected);
+
+        // test not push down grouping count.
         assertThatThrownBy(
                         () ->
                                 tEnv.explainSql(
                                                 String.format(
-                                                        "SELECT COUNT(*) FROM %s ",
-                                                        primaryTableName))
+                                                        "SELECT COUNT(*) FROM %s group by id",
+                                                        tableName))
                                         .wait())
                 .hasMessageContaining(
                         "Currently, Fluss only support queries on table with datalake enabled or point queries on primary key when it's in batch execution mode.");
@@ -356,8 +469,8 @@ abstract class FlinkTableSourceBatchITCase extends FlinkTestBase {
         }
 
         // prepare table data
-        try (Table dimTable = conn.getTable(tablePath)) {
-            UpsertWriter upsertWriter = dimTable.newUpsert().createWriter();
+        try (Table table = conn.getTable(tablePath)) {
+            UpsertWriter upsertWriter = table.newUpsert().createWriter();
             for (int i = 1; i <= 5; i++) {
                 Object[] values =
                         partition1 == null
@@ -401,7 +514,7 @@ abstract class FlinkTableSourceBatchITCase extends FlinkTestBase {
         return tableName;
     }
 
-    private String preparePartitionedLogTable() throws Exception {
+    protected String preparePartitionedLogTable() throws Exception {
         String tableName = String.format("test_partitioned_log_table_%s", RandomUtils.nextInt());
         tEnv.executeSql(
                 String.format(
