@@ -79,24 +79,6 @@ class FlinkSourceSplitReaderTest extends FlinkTestBase {
         TableDescriptor descriptor1 = TableDescriptor.builder().schema(schema1).build();
         createTable(tablePath1, descriptor1);
 
-        assertThatThrownBy(
-                        () ->
-                                new FlinkSourceSplitReader(
-                                        clientConf,
-                                        tablePath1,
-                                        DataTypes.ROW(
-                                                DataTypes.FIELD("id", DataTypes.BIGINT()),
-                                                DataTypes.FIELD("name", DataTypes.STRING()),
-                                                DataTypes.FIELD("age", DataTypes.INT())),
-                                        null,
-                                        createMockSourceReaderMetrics(),
-                                        null))
-                .isInstanceOf(ValidationException.class)
-                .hasMessage(
-                        "The Flink query schema is not matched to Fluss table schema. \n"
-                                + "Flink query schema: ROW<`id` BIGINT, `name` STRING, `age` INT>\n"
-                                + "Fluss table schema: ROW<`id` BIGINT NOT NULL, `name` STRING, `age` INT>");
-
         assertThatThrownBy(
                         () ->
                                 new FlinkSourceSplitReader(
@@ -114,6 +96,32 @@ class FlinkSourceSplitReaderTest extends FlinkTestBase {
                         "The Flink query schema is not matched to Fluss table schema. \n"
                                 + "Flink query schema: ROW<`id` BIGINT NOT NULL, `name` STRING>\n"
                                 + "Fluss table schema: ROW<`name` STRING, `id` BIGINT NOT NULL> (projection [1, 0])");
+
+        assertThatThrownBy(
+                        () ->
+                                new FlinkSourceSplitReader(
+                                        clientConf,
+                                        tablePath1,
+                                        DataTypes.ROW(
+                                                DataTypes.FIELD("name2", DataTypes.STRING()),
+                                                DataTypes.FIELD("id", DataTypes.BIGINT())),
+                                        null,
+                                        createMockSourceReaderMetrics(),
+                                        null))
+                .isInstanceOf(IllegalArgumentException.class)
+                .hasMessage("Field name2 does not exist in the row type.");
+
+        FlinkSourceSplitReader flinkSourceSplitReader =
+                new FlinkSourceSplitReader(
+                        clientConf,
+                        tablePath1,
+                        DataTypes.ROW(
+                                DataTypes.FIELD("name", DataTypes.STRING()),
+                                DataTypes.FIELD("id", DataTypes.BIGINT().copy(false))),
+                        null,
+                        createMockSourceReaderMetrics(),
+                        null);
+        assertThat(flinkSourceSplitReader.getProjectedFields()).isNull();
     }
 
     @Test
@@ -136,7 +144,7 @@ class FlinkSourceSplitReaderTest extends FlinkTestBase {
             Map<TableBucket, List<InternalRow>> rows = putRows(tableId, tablePath, 10);
 
             // check the expected records
-            waitUntilSnapshot(tableId, 0);
+            FLUSS_CLUSTER_EXTENSION.triggerAndWaitSnapshot(tablePath);
 
             hybridSnapshotLogSplits = getHybridSnapshotLogSplits(tablePath);
 
@@ -150,6 +158,29 @@ class FlinkSourceSplitReaderTest extends FlinkTestBase {
         }
     }
 
+    @Test
+    void testTableIdChange() throws Exception {
+        TablePath tablePath = TablePath.of(DEFAULT_DB, "test-only-snapshot-table");
+        long tableId = createTable(tablePath, DEFAULT_PK_TABLE_DESCRIPTOR);
+        try (FlinkSourceSplitReader splitReader =
+                createSplitReader(tablePath, DEFAULT_PK_TABLE_SCHEMA.getRowType())) {
+            assertThatThrownBy(
+                            () ->
+                                    splitReader.handleSplitsChanges(
+                                            new SplitsAddition<>(
+                                                    Collections.singletonList(
+                                                            new LogSplit(
+                                                                    new TableBucket(tableId + 1, 0),
+                                                                    null,
+                                                                    0)))))
+                    .hasMessageContaining(
+                            "Table ID mismatch: expected 0, but split contains 1 for table 'test-flink-db.test-only-snapshot-table'. "
+                                    + "This usually happens when a table with the same name was dropped and recreated between job runs, "
+                                    + "causing metadata inconsistency. To resolve this, please restart the job **without** using "
+                                    + "the previous savepoint or checkpoint.");
+        }
+    }
+
     private Map<String, List<RecordAndPos>> constructRecords(
             Map<TableBucket, List<InternalRow>> rows) {
         Map<String, List<RecordAndPos>> expectedRecords = new HashMap<>();
@@ -221,7 +252,7 @@ class FlinkSourceSplitReaderTest extends FlinkTestBase {
             Map<TableBucket, List<InternalRow>> rows = putRows(tableId, tablePath, 10);
 
             // check the expected records
-            waitUntilSnapshot(tableId, 0);
+            FLUSS_CLUSTER_EXTENSION.triggerAndWaitSnapshot(tablePath);
 
             List<SourceSplitBase> totalSplits =
                     new ArrayList<>(getHybridSnapshotLogSplits(tablePath));
