@@ -23,7 +23,9 @@ import org.apache.fluss.config.ConfigOptions;
 import org.apache.fluss.metadata.Schema;
 import org.apache.fluss.metadata.TableDescriptor;
 import org.apache.fluss.metadata.TablePath;
+import org.apache.fluss.row.BinaryString;
 import org.apache.fluss.row.Decimal;
+import org.apache.fluss.row.GenericMap;
 import org.apache.fluss.row.InternalRow;
 import org.apache.fluss.row.TimestampLtz;
 import org.apache.fluss.row.TimestampNtz;
@@ -49,6 +51,7 @@ import java.time.Instant;
 import java.time.LocalDateTime;
 import java.time.ZoneId;
 import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
@@ -126,12 +129,12 @@ public class FlinkUnionReadLogTableITCase extends FlinkUnionReadTestBase {
             // check filter push down
             assertThat(plan)
                     .contains("TableSourceScan(")
-                    .contains("LogicalFilter(condition=[=($15, _UTF-16LE'" + partition + "'")
+                    .contains("LogicalFilter(condition=[=($17, _UTF-16LE'" + partition + "'")
                     .contains("filter=[=(p, _UTF-16LE'" + partition + "'");
 
             List<Row> expectedFiltered =
                     writtenRows.stream()
-                            .filter(r -> partition.equals(r.getField(15)))
+                            .filter(r -> partition.equals(r.getField(17)))
                             .collect(Collectors.toList());
 
             List<Row> actualFiltered =
@@ -295,7 +298,13 @@ public class FlinkUnionReadLogTableITCase extends FlinkUnionReadTestBase {
                         .column("f_timestamp_ltz2", DataTypes.TIMESTAMP_LTZ(6))
                         .column("f_timestamp_ntz1", DataTypes.TIMESTAMP(3))
                         .column("f_timestamp_ntz2", DataTypes.TIMESTAMP(6))
-                        .column("f_binary", DataTypes.BINARY(4));
+                        .column("f_binary", DataTypes.BINARY(4))
+                        .column(
+                                "f_row",
+                                DataTypes.ROW(
+                                        DataTypes.FIELD("f_nested_int", DataTypes.INT()),
+                                        DataTypes.FIELD("f_nested_string", DataTypes.STRING())))
+                        .column("f_map", DataTypes.MAP(DataTypes.STRING(), DataTypes.INT()));
 
         TableDescriptor.Builder tableBuilder =
                 TableDescriptor.builder()
@@ -325,6 +334,17 @@ public class FlinkUnionReadLogTableITCase extends FlinkUnionReadTestBase {
         List<InternalRow> rows = new ArrayList<>();
         List<Row> flinkRows = new ArrayList<>();
         for (int i = 0; i < rowCount; i++) {
+            // Map for Fluss InternalRow
+            Map<Object, Object> mapData = new HashMap<>();
+            mapData.put(BinaryString.fromString("key1"), 100 + i);
+            mapData.put(BinaryString.fromString("key2"), 200 + i);
+            GenericMap map = new GenericMap(mapData);
+
+            // Map for FlinkRow
+            Map<String, Integer> flinkMap = new HashMap<>();
+            flinkMap.put("key1", 100 + i);
+            flinkMap.put("key2", 200 + i);
+
             if (partition == null) {
                 rows.add(
                         row(
@@ -342,7 +362,9 @@ public class FlinkUnionReadLogTableITCase extends FlinkUnionReadTestBase {
                                 TimestampLtz.fromEpochMillis(1698235273400L, 7000),
                                 TimestampNtz.fromMillis(1698235273501L),
                                 TimestampNtz.fromMillis(1698235273501L, 8000),
-                                new byte[] {5, 6, 7, 8}));
+                                new byte[] {5, 6, 7, 8},
+                                row(10, "nested_string"),
+                                map));
 
                 flinkRows.add(
                         Row.of(
@@ -364,7 +386,9 @@ public class FlinkUnionReadLogTableITCase extends FlinkUnionReadTestBase {
                                                 Instant.ofEpochMilli(1698235273501L),
                                                 ZoneId.of("UTC"))
                                         .plusNanos(8000),
-                                new byte[] {5, 6, 7, 8}));
+                                new byte[] {5, 6, 7, 8},
+                                Row.of(10, "nested_string"),
+                                flinkMap));
             } else {
                 rows.add(
                         row(
@@ -383,6 +407,8 @@ public class FlinkUnionReadLogTableITCase extends FlinkUnionReadTestBase {
                                 TimestampNtz.fromMillis(1698235273501L),
                                 TimestampNtz.fromMillis(1698235273501L, 8000),
                                 new byte[] {5, 6, 7, 8},
+                                row(10, "nested_string"),
+                                map,
                                 partition));
 
                 flinkRows.add(
@@ -406,6 +432,8 @@ public class FlinkUnionReadLogTableITCase extends FlinkUnionReadTestBase {
                                                 ZoneId.of("UTC"))
                                         .plusNanos(8000),
                                 new byte[] {5, 6, 7, 8},
+                                Row.of(10, "nested_string"),
+                                flinkMap,
                                 partition));
             }
         }
