@@ -24,7 +24,10 @@ import org.apache.fluss.metadata.Schema;
 import org.apache.fluss.metadata.TableBucket;
 import org.apache.fluss.metadata.TableDescriptor;
 import org.apache.fluss.metadata.TablePath;
+import org.apache.fluss.row.BinaryString;
 import org.apache.fluss.row.Decimal;
+import org.apache.fluss.row.GenericArray;
+import org.apache.fluss.row.GenericRow;
 import org.apache.fluss.row.InternalRow;
 import org.apache.fluss.row.TimestampLtz;
 import org.apache.fluss.row.TimestampNtz;
@@ -98,7 +101,9 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                                 TimestampNtz.fromMillis(1698235273183L),
                                 TimestampNtz.fromMillis(1698235273183L, 6000),
                                 new byte[] {1, 2, 3, 4},
-                                partition));
+                                new float[] {1.1f, 1.2f, 1.3f},
+                                partition,
+                                Row.of(1, "nested_row1")));
                 expectedRows.add(
                         Row.of(
                                 true,
@@ -116,7 +121,9 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                                 TimestampNtz.fromMillis(1698235273201L),
                                 TimestampNtz.fromMillis(1698235273201L, 6000),
                                 new byte[] {1, 2, 3, 4},
-                                partition));
+                                new float[] {1.1f, 1.2f, 1.3f},
+                                partition,
+                                Row.of(2, "nested_row2")));
             }
         } else {
             expectedRows =
@@ -137,7 +144,9 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                                     TimestampNtz.fromMillis(1698235273183L),
                                     TimestampNtz.fromMillis(1698235273183L, 6000),
                                     new byte[] {1, 2, 3, 4},
-                                    null),
+                                    new float[] {1.1f, 1.2f, 1.3f},
+                                    null,
+                                    Row.of(1, "nested_row1")),
                             Row.of(
                                     true,
                                     (byte) 10,
@@ -154,7 +163,9 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                                     TimestampNtz.fromMillis(1698235273201L),
                                     TimestampNtz.fromMillis(1698235273201L, 6000),
                                     new byte[] {1, 2, 3, 4},
-                                    null));
+                                    new float[] {1.1f, 1.2f, 1.3f},
+                                    null,
+                                    Row.of(2, "nested_row2")));
         }
 
         String query = "select * from " + tableName;
@@ -196,7 +207,9 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                                 TimestampNtz.fromMillis(1698235273201L),
                                 TimestampNtz.fromMillis(1698235273201L, 6000),
                                 new byte[] {1, 2, 3, 4},
-                                partition));
+                                new float[] {1.1f, 1.2f, 1.3f},
+                                partition,
+                                Row.of(2, "nested_row2")));
                 expectedRows2.add(
                         Row.ofKind(
                                 RowKind.UPDATE_AFTER,
@@ -215,7 +228,9 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                                 TimestampNtz.fromMillis(1698235273501L),
                                 TimestampNtz.fromMillis(1698235273501L, 8000),
                                 new byte[] {5, 6, 7, 8},
-                                partition));
+                                new float[] {2.1f, 2.2f, 2.3f},
+                                partition,
+                                Row.of(3, "nested_update")));
             }
         } else {
             expectedRows2.add(
@@ -236,7 +251,9 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                             TimestampNtz.fromMillis(1698235273201L),
                             TimestampNtz.fromMillis(1698235273201L, 6000),
                             new byte[] {1, 2, 3, 4},
-                            null));
+                            new float[] {1.1f, 1.2f, 1.3f},
+                            null,
+                            Row.of(2, "nested_row2")));
             expectedRows2.add(
                     Row.ofKind(
                             RowKind.UPDATE_AFTER,
@@ -255,7 +272,9 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                             TimestampNtz.fromMillis(1698235273501L),
                             TimestampNtz.fromMillis(1698235273501L, 8000),
                             new byte[] {5, 6, 7, 8},
-                            null));
+                            new float[] {2.1f, 2.2f, 2.3f},
+                            null,
+                            Row.of(3, "nested_update")));
         }
 
         if (isPartitioned) {
@@ -340,6 +359,10 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
     }
 
     private void writeFullTypeRow(TablePath tablePath, String partition) throws Exception {
+        GenericRow nestedRow = new GenericRow(2);
+        nestedRow.setField(0, 3);
+        nestedRow.setField(1, BinaryString.fromString("nested_update"));
+
         List<InternalRow> rows =
                 Collections.singletonList(
                         row(
@@ -358,7 +381,9 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                                 TimestampNtz.fromMillis(1698235273501L),
                                 TimestampNtz.fromMillis(1698235273501L, 8000),
                                 new byte[] {5, 6, 7, 8},
-                                partition));
+                                new GenericArray(new float[] {2.1f, 2.2f, 2.3f}),
+                                partition,
+                                nestedRow));
         writeRows(tablePath, rows, false);
     }
 
@@ -411,7 +436,13 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                         .column("c13", DataTypes.TIMESTAMP(3))
                         .column("c14", DataTypes.TIMESTAMP(6))
                         .column("c15", DataTypes.BINARY(4))
-                        .column("c16", DataTypes.STRING());
+                        .column("c16", DataTypes.ARRAY(DataTypes.FLOAT()))
+                        .column("c17", DataTypes.STRING())
+                        .column(
+                                "c18",
+                                DataTypes.ROW(
+                                        DataTypes.FIELD("a", DataTypes.INT()),
+                                        DataTypes.FIELD("b", DataTypes.STRING())));
 
         TableDescriptor.Builder tableBuilder =
                 TableDescriptor.builder()
@@ -421,8 +452,8 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
 
         if (isPartitioned) {
             tableBuilder.property(ConfigOptions.TABLE_AUTO_PARTITION_ENABLED, true);
-            tableBuilder.partitionedBy("c16");
-            schemaBuilder.primaryKey("c4", "c16");
+            tableBuilder.partitionedBy("c17");
+            schemaBuilder.primaryKey("c4", "c17");
             tableBuilder.property(
                     ConfigOptions.TABLE_AUTO_PARTITION_TIME_UNIT, AutoPartitionTimeUnit.YEAR);
         } else {
@@ -433,6 +464,14 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
     }
 
     private List<InternalRow> generateKvRowsFullType(@Nullable String partition) {
+        GenericRow nestedRow1 = new GenericRow(2);
+        nestedRow1.setField(0, 1);
+        nestedRow1.setField(1, BinaryString.fromString("nested_row1"));
+
+        GenericRow nestedRow2 = new GenericRow(2);
+        nestedRow2.setField(0, 2);
+        nestedRow2.setField(1, BinaryString.fromString("nested_row2"));
+
         return Arrays.asList(
                 row(
                         false,
@@ -450,7 +489,9 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                         TimestampNtz.fromMillis(1698235273183L),
                         TimestampNtz.fromMillis(1698235273183L, 6000),
                         new byte[] {1, 2, 3, 4},
-                        partition),
+                        new GenericArray(new float[] {1.1f, 1.2f, 1.3f}),
+                        partition,
+                        nestedRow1),
                 row(
                         true,
                         (byte) 10,
@@ -467,7 +508,9 @@ public class FlinkUnionReadPrimaryKeyTableITCase extends FlinkUnionReadTestBase
                         TimestampNtz.fromMillis(1698235273201L),
                         TimestampNtz.fromMillis(1698235273201L, 6000),
                         new byte[] {1, 2, 3, 4},
-                        partition));
+                        new GenericArray(new float[] {1.1f, 1.2f, 1.3f}),
+                        partition,
+                        nestedRow2));
     }
 
     private Map<TableBucket, Long> getBucketLogEndOffset(
