@@ -28,6 +28,7 @@ import org.apache.fluss.config.ConfigOptions;
 import org.apache.fluss.config.Configuration;
 import org.apache.fluss.exception.FlussRuntimeException;
 import org.apache.fluss.flink.tiering.LakeTieringJobBuilder;
+import org.apache.fluss.fs.FsPath;
 import org.apache.fluss.metadata.DataLakeFormat;
 import org.apache.fluss.metadata.Schema;
 import org.apache.fluss.metadata.TableBucket;
@@ -37,6 +38,7 @@ import org.apache.fluss.row.InternalRow;
 import org.apache.fluss.server.replica.Replica;
 import org.apache.fluss.server.testutils.FlussClusterExtension;
 import org.apache.fluss.server.zk.ZooKeeperClient;
+import org.apache.fluss.server.zk.data.lake.LakeTable;
 
 import org.apache.flink.api.common.RuntimeExecutionMode;
 import org.apache.flink.core.execution.JobClient;
@@ -76,6 +78,7 @@ import java.util.SortedSet;
 import java.util.TreeSet;
 
 import static org.apache.fluss.flink.tiering.source.TieringSourceOptions.POLL_TIERING_TABLE_INTERVAL;
+import static org.apache.fluss.lake.committer.LakeCommitter.FLUSS_LAKE_SNAP_BUCKET_OFFSET_PROPERTY;
 import static org.apache.fluss.lake.iceberg.utils.IcebergConversions.toIceberg;
 import static org.apache.fluss.metadata.TableDescriptor.OFFSET_COLUMN_NAME;
 import static org.apache.fluss.testutils.DataTestUtils.row;
@@ -105,8 +108,7 @@ public class FlinkIcebergTieringTestBase {
 
     private static Configuration initConfig() {
         Configuration conf = new Configuration();
-        conf.set(ConfigOptions.KV_SNAPSHOT_INTERVAL, Duration.ofSeconds(1))
-                .set(ConfigOptions.KV_MAX_RETAINED_SNAPSHOTS, Integer.MAX_VALUE);
+        conf.set(ConfigOptions.KV_MAX_RETAINED_SNAPSHOTS, Integer.MAX_VALUE);
 
         // Configure the tiering sink to be Iceberg
         conf.set(ConfigOptions.DATALAKE_FORMAT, DataLakeFormat.ICEBERG);
@@ -161,6 +163,7 @@ public class FlinkIcebergTieringTestBase {
                         execEnv,
                         flussConfig,
                         Configuration.fromMap(getIcebergCatalogConf()),
+                        new Configuration(),
                         DataLakeFormat.ICEBERG.toString())
                 .build();
     }
@@ -315,13 +318,6 @@ public class FlinkIcebergTieringTestBase {
         }
     }
 
-    protected void waitUntilSnapshot(long tableId, int bucketNum, long snapshotId) {
-        for (int i = 0; i < bucketNum; i++) {
-            TableBucket tableBucket = new TableBucket(tableId, i);
-            FLUSS_CLUSTER_EXTENSION.waitUntilSnapshotFinished(tableBucket, snapshotId);
-        }
-    }
-
     protected List<Record> getIcebergRecords(TablePath tablePath) throws IOException {
         List<Record> icebergRecords = new ArrayList<>();
         try (CloseableIterator<Record> records = getIcebergRows(tablePath)) {
@@ -437,7 +433,7 @@ public class FlinkIcebergTieringTestBase {
 
             for (DataFile file : files) {
                 Iterable<Record> iterable =
-                        Parquet.read(table.io().newInputFile(file.path().toString()))
+                        Parquet.read(table.io().newInputFile(file.location()))
                                 .project(table.schema())
                                 .createReaderFunc(
                                         fileSchema ->
@@ -470,11 +466,20 @@ public class FlinkIcebergTieringTestBase {
         return tableScan;
     }
 
-    protected void checkSnapshotPropertyInIceberg(
-            TablePath tablePath, Map<String, String> expectedProperties) throws Exception {
+    protected void checkFlussOffsetsInSnapshot(
+            TablePath tablePath, Map<TableBucket, Long> expectedOffsets) throws Exception {
         org.apache.iceberg.Table table = icebergCatalog.loadTable(toIceberg(tablePath));
         Snapshot snapshot = table.currentSnapshot();
-        assertThat(snapshot.summary()).containsAllEntriesOf(expectedProperties);
+
+        String offsetFile = snapshot.summary().get(FLUSS_LAKE_SNAP_BUCKET_OFFSET_PROPERTY);
+        Map<TableBucket, Long> recordedOffsets =
+                new LakeTable(
+                                new LakeTable.LakeSnapshotMetadata(
+                                        // don't care about snapshot id
+                                        -1, new FsPath(offsetFile), null))
+                        .getOrReadLatestTableSnapshot()
+                        .getBucketLogEndOffset();
+        assertThat(recordedOffsets).isEqualTo(expectedOffsets);
     }
 
     protected Map<String, List<InternalRow>> writeRowsIntoPartitionedTable(
