@@ -19,8 +19,10 @@ package org.apache.fluss.rpc.netty.client;
 
 import org.apache.fluss.annotation.VisibleForTesting;
 import org.apache.fluss.cluster.ServerNode;
+import org.apache.fluss.cluster.ServerType;
 import org.apache.fluss.exception.DisconnectException;
 import org.apache.fluss.exception.FlussRuntimeException;
+import org.apache.fluss.exception.InvalidServerTypeException;
 import org.apache.fluss.exception.NetworkException;
 import org.apache.fluss.exception.RetriableAuthenticationException;
 import org.apache.fluss.rpc.messages.ApiMessage;
@@ -29,7 +31,7 @@ import org.apache.fluss.rpc.messages.ApiVersionsResponse;
 import org.apache.fluss.rpc.messages.AuthenticateRequest;
 import org.apache.fluss.rpc.messages.AuthenticateResponse;
 import org.apache.fluss.rpc.metrics.ClientMetricGroup;
-import org.apache.fluss.rpc.metrics.ConnectionMetricGroup;
+import org.apache.fluss.rpc.metrics.ConnectionMetrics;
 import org.apache.fluss.rpc.protocol.ApiKeys;
 import org.apache.fluss.rpc.protocol.ApiManager;
 import org.apache.fluss.rpc.protocol.ApiMethod;
@@ -72,7 +74,7 @@ final class ServerConnection {
     // TODO: add max inflight requests limit like Kafka's "max.in.flight.requests.per.connection"
     private final Map<Integer, InflightRequest> inflightRequests = MapUtils.newConcurrentHashMap();
     private final CompletableFuture<Void> closeFuture = new CompletableFuture<>();
-    private final ConnectionMetricGroup connectionMetricGroup;
+    private final ConnectionMetrics connectionMetrics;
     private final ClientAuthenticator authenticator;
     private final ExponentialBackoff backoff;
 
@@ -108,7 +110,7 @@ final class ServerConnection {
             boolean isInnerClient) {
         this.node = node;
         this.state = ConnectionState.CONNECTING;
-        this.connectionMetricGroup = clientMetricGroup.createConnectionMetricGroup(node.uid());
+        this.connectionMetrics = clientMetricGroup.createConnectionMetricGroup(node.uid());
         this.authenticator = authenticator;
         this.backoff = new ExponentialBackoff(100L, 2, 5000L, 0.2);
         whenClose(closeCallback);
@@ -166,7 +168,9 @@ final class ServerConnection {
             // notify all the inflight requests
             for (int requestId : inflightRequests.keySet()) {
                 InflightRequest request = inflightRequests.remove(requestId);
-                request.responseFuture.completeExceptionally(requestCause);
+                if (request != null) {
+                    request.responseFuture.completeExceptionally(requestCause);
+                }
             }
 
             // notify all the pending requests
@@ -190,7 +194,7 @@ final class ServerConnection {
                 closeFuture.completeExceptionally(cause);
             }
 
-            connectionMetricGroup.close();
+            connectionMetrics.close();
         }
 
         closeQuietly(authenticator);
@@ -220,7 +224,7 @@ final class ServerConnection {
         public void onRequestResult(int requestId, ApiMessage response) {
             InflightRequest request = inflightRequests.remove(requestId);
             if (request != null && !request.responseFuture.isDone()) {
-                connectionMetricGroup.updateMetricsAfterGetResponse(
+                connectionMetrics.updateMetricsAfterGetResponse(
                         ApiKeys.forId(request.apiKey),
                         request.requestStartTime,
                         response.totalSize());
@@ -232,7 +236,7 @@ final class ServerConnection {
         public void onRequestFailure(int requestId, Throwable cause) {
             InflightRequest request = inflightRequests.remove(requestId);
             if (request != null && !request.responseFuture.isDone()) {
-                connectionMetricGroup.updateMetricsAfterGetResponse(
+                connectionMetrics.updateMetricsAfterGetResponse(
                         ApiKeys.forId(request.apiKey), request.requestStartTime, 0);
                 request.responseFuture.completeExceptionally(cause);
             }
@@ -249,6 +253,12 @@ final class ServerConnection {
     private void establishConnection(ChannelFuture future, boolean isInnerClient) {
         synchronized (lock) {
             if (future.isSuccess()) {
+                if (state.isDisconnected()) {
+                    LOG.debug(
+                            "Connection established to {} but connection is already closed.", node);
+                    future.channel().close();
+                    return;
+                }
                 LOG.debug("Established connection to server {}.", node);
                 channel = future.channel();
                 channel.pipeline()
@@ -305,6 +315,7 @@ final class ServerConnection {
                     version = serverApiVersions.highestAvailableVersion(apiKey);
                 } catch (Exception e) {
                     responseFuture.completeExceptionally(e);
+                    return responseFuture;
                 }
             }
 
@@ -329,14 +340,14 @@ final class ServerConnection {
                 return responseFuture;
             }
 
-            connectionMetricGroup.updateMetricsBeforeSendRequest(apiKey, rawRequest.totalSize());
+            connectionMetrics.updateMetricsBeforeSendRequest(apiKey, rawRequest.totalSize());
 
             channel.writeAndFlush(byteBuf)
                     .addListener(
                             (ChannelFutureListener)
                                     future -> {
                                         if (!future.isSuccess()) {
-                                            connectionMetricGroup.updateMetricsAfterGetResponse(
+                                            connectionMetrics.updateMetricsAfterGetResponse(
                                                     apiKey, inflight.requestStartTime, 0);
                                             Throwable cause = future.cause();
                                             if (cause instanceof IOException) {
@@ -364,6 +375,27 @@ final class ServerConnection {
             return;
         }
 
+        ApiVersionsResponse apiVersion = (ApiVersionsResponse) response;
+        if (apiVersion.hasServerType()) {
+            ServerType serverType = ServerType.fromTypeId(apiVersion.getServerType());
+            // bootstrap servers are set to unknown type, because they may coordinator or tablet.
+            if (node.serverType() != ServerType.UNKNOWN && serverType != node.serverType()) {
+                LOG.warn(
+                        "Server type mismatch, expected: {}, actual: {}",
+                        node.serverType(),
+                        serverType);
+                close(
+                        new InvalidServerTypeException(
+                                "Server type mismatch, expected: "
+                                        + node.serverType()
+                                        + ", actual: "
+                                        + serverType
+                                        + ", node: "
+                                        + node));
+                return;
+            }
+        }
+
         synchronized (lock) {
             serverApiVersions =
                     new ServerApiVersions(((ApiVersionsResponse) response).getApiVersionsList());
@@ -449,8 +481,10 @@ final class ServerConnection {
     }
 
     private void switchState(ConnectionState targetState) {
-        LOG.debug("switch state form {} to {}", state, targetState);
-        state = targetState;
+        if (state != ConnectionState.DISCONNECTED) {
+            LOG.debug("switch state form {} to {}", state, targetState);
+            state = targetState;
+        }
         if (targetState == ConnectionState.READY) {
             // process pending requests
             PendingRequest pending;
