@@ -52,10 +52,10 @@ import org.apache.fluss.rpc.messages.GetFileSystemSecurityTokenRequest;
 import org.apache.fluss.rpc.messages.GetFileSystemSecurityTokenResponse;
 import org.apache.fluss.rpc.messages.GetKvSnapshotMetadataRequest;
 import org.apache.fluss.rpc.messages.GetKvSnapshotMetadataResponse;
+import org.apache.fluss.rpc.messages.GetLakeSnapshotRequest;
+import org.apache.fluss.rpc.messages.GetLakeSnapshotResponse;
 import org.apache.fluss.rpc.messages.GetLatestKvSnapshotsRequest;
 import org.apache.fluss.rpc.messages.GetLatestKvSnapshotsResponse;
-import org.apache.fluss.rpc.messages.GetLatestLakeSnapshotRequest;
-import org.apache.fluss.rpc.messages.GetLatestLakeSnapshotResponse;
 import org.apache.fluss.rpc.messages.GetTableInfoRequest;
 import org.apache.fluss.rpc.messages.GetTableInfoResponse;
 import org.apache.fluss.rpc.messages.GetTableSchemaRequest;
@@ -93,7 +93,7 @@ import org.apache.fluss.server.tablet.TabletService;
 import org.apache.fluss.server.utils.ServerRpcMessageUtils;
 import org.apache.fluss.server.zk.ZooKeeperClient;
 import org.apache.fluss.server.zk.data.BucketSnapshot;
-import org.apache.fluss.server.zk.data.LakeTableSnapshot;
+import org.apache.fluss.server.zk.data.lake.LakeTableSnapshot;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -108,19 +108,21 @@ import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.ExecutorService;
 import java.util.stream.Collectors;
 
 import static org.apache.fluss.rpc.util.CommonRpcMessageUtils.toAclFilter;
 import static org.apache.fluss.rpc.util.CommonRpcMessageUtils.toResolvedPartitionSpec;
 import static org.apache.fluss.security.acl.Resource.TABLE_SPLITTER;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.buildMetadataResponse;
+import static org.apache.fluss.server.utils.ServerRpcMessageUtils.makeGetLakeSnapshotResponse;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.makeGetLatestKvSnapshotsResponse;
-import static org.apache.fluss.server.utils.ServerRpcMessageUtils.makeGetLatestLakeSnapshotResponse;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.makeKvSnapshotMetadataResponse;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.makeListAclsResponse;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.toGetFileSystemSecurityTokenResponse;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.toListPartitionInfosResponse;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.toPbConfigEntries;
+import static org.apache.fluss.server.utils.ServerRpcMessageUtils.toPbDatabaseSummary;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.toTablePath;
 import static org.apache.fluss.utils.Preconditions.checkState;
 
@@ -144,13 +146,16 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
     private long tokenLastUpdateTimeMs = 0;
     private ObtainedSecurityToken securityToken = null;
 
+    private final ExecutorService ioExecutor;
+
     public RpcServiceBase(
             FileSystem remoteFileSystem,
             ServerType provider,
             ZooKeeperClient zkClient,
             MetadataManager metadataManager,
             @Nullable Authorizer authorizer,
-            DynamicConfigManager dynamicConfigManager) {
+            DynamicConfigManager dynamicConfigManager,
+            ExecutorService ioExecutor) {
         this.remoteFileSystem = remoteFileSystem;
         this.provider = provider;
         this.apiManager = new ApiManager(provider);
@@ -158,6 +163,7 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
         this.metadataManager = metadataManager;
         this.authorizer = authorizer;
         this.dynamicConfigManager = dynamicConfigManager;
+        this.ioExecutor = ioExecutor;
     }
 
     @Override
@@ -165,6 +171,20 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
         return provider;
     }
 
+    public abstract void authorizeTable(OperationType operationType, long tableId);
+
+    public void authorizeDatabase(OperationType operationType, String databaseName) {
+        if (authorizer != null) {
+            authorizer.authorize(currentSession(), operationType, Resource.database(databaseName));
+        }
+    }
+
+    public void authorizeTable(OperationType operationType, TablePath tablePath) {
+        if (authorizer != null) {
+            authorizer.authorize(currentSession(), operationType, Resource.table(tablePath));
+        }
+    }
+
     @Override
     public CompletableFuture<ApiVersionsResponse> apiVersions(ApiVersionsRequest request) {
         Set<ApiKeys> apiKeys = apiManager.enabledApis();
@@ -178,6 +198,7 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
         }
         ApiVersionsResponse response = new ApiVersionsResponse();
         response.addAllApiVersions(apiVersions);
+        response.setServerType(provider.toTypeId());
         return CompletableFuture.completedFuture(response);
     }
 
@@ -198,22 +219,24 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
                     authorizedDatabase.stream().map(Resource::getName).collect(Collectors.toList());
         }
 
-        response.addAllDatabaseNames(databaseNames);
+        if (request.hasIncludeSummary() && request.isIncludeSummary()) {
+            response.addAllDatabaseSummaries(
+                    toPbDatabaseSummary(metadataManager.listDatabaseSummaries(databaseNames)));
+        } else {
+            response.addAllDatabaseNames(databaseNames);
+        }
+
         return CompletableFuture.completedFuture(response);
     }
 
     @Override
     public CompletableFuture<GetDatabaseInfoResponse> getDatabaseInfo(
             GetDatabaseInfoRequest request) {
-        if (authorizer != null) {
-            authorizer.authorize(
-                    currentSession(),
-                    OperationType.DESCRIBE,
-                    Resource.database(request.getDatabaseName()));
-        }
+        String databaseName = request.getDatabaseName();
+        authorizeDatabase(OperationType.DESCRIBE, databaseName);
 
         GetDatabaseInfoResponse response = new GetDatabaseInfoResponse();
-        DatabaseInfo databaseInfo = metadataManager.getDatabase(request.getDatabaseName());
+        DatabaseInfo databaseInfo = metadataManager.getDatabase(databaseName);
         response.setDatabaseJson(databaseInfo.getDatabaseDescriptor().toJsonBytes())
                 .setCreatedTime(databaseInfo.getCreatedTime())
                 .setModifiedTime(databaseInfo.getModifiedTime());
@@ -222,6 +245,7 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
 
     @Override
     public CompletableFuture<DatabaseExistsResponse> databaseExists(DatabaseExistsRequest request) {
+        // By design: database exists not need to check database authorization.
         DatabaseExistsResponse response = new DatabaseExistsResponse();
         boolean exists = metadataManager.databaseExists(request.getDatabaseName());
         response.setExists(exists);
@@ -253,12 +277,7 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
     @Override
     public CompletableFuture<GetTableInfoResponse> getTableInfo(GetTableInfoRequest request) {
         TablePath tablePath = toTablePath(request.getTablePath());
-        if (authorizer != null) {
-            authorizer.authorize(
-                    currentSession(),
-                    OperationType.DESCRIBE,
-                    Resource.table(tablePath.getDatabaseName(), tablePath.getTableName()));
-        }
+        authorizeTable(OperationType.DESCRIBE, tablePath);
 
         GetTableInfoResponse response = new GetTableInfoResponse();
         TableInfo tableInfo = metadataManager.getTable(tablePath);
@@ -273,6 +292,8 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
     @Override
     public CompletableFuture<GetTableSchemaResponse> getTableSchema(GetTableSchemaRequest request) {
         TablePath tablePath = toTablePath(request.getTablePath());
+        authorizeTable(OperationType.DESCRIBE, tablePath);
+
         final SchemaInfo schemaInfo;
         if (request.hasSchemaId()) {
             schemaInfo = metadataManager.getSchemaById(tablePath, request.getSchemaId());
@@ -287,6 +308,7 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
 
     @Override
     public CompletableFuture<TableExistsResponse> tableExists(TableExistsRequest request) {
+        // By design: table exists not need to check table authorization.
         TableExistsResponse response = new TableExistsResponse();
         boolean exists = metadataManager.tableExists(toTablePath(request.getTablePath()));
         response.setExists(exists);
@@ -297,6 +319,8 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
     public CompletableFuture<GetLatestKvSnapshotsResponse> getLatestKvSnapshots(
             GetLatestKvSnapshotsRequest request) {
         TablePath tablePath = toTablePath(request.getTablePath());
+        authorizeTable(OperationType.DESCRIBE, tablePath);
+
         // get table info
         TableInfo tableInfo = metadataManager.getTable(tablePath);
 
@@ -358,9 +382,12 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
     @Override
     public CompletableFuture<GetKvSnapshotMetadataResponse> getKvSnapshotMetadata(
             GetKvSnapshotMetadataRequest request) {
+        long tableId = request.getTableId();
+        authorizeTable(OperationType.DESCRIBE, tableId);
+
         TableBucket tableBucket =
                 new TableBucket(
-                        request.getTableId(),
+                        tableId,
                         request.hasPartitionId() ? request.getPartitionId() : null,
                         request.getBucketId());
         long snapshotId = request.getSnapshotId();
@@ -407,6 +434,8 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
     public CompletableFuture<ListPartitionInfosResponse> listPartitionInfos(
             ListPartitionInfosRequest request) {
         TablePath tablePath = toTablePath(request.getTablePath());
+        authorizeTable(OperationType.DESCRIBE, tablePath);
+
         Map<String, Long> partitionNameAndIds;
         if (request.hasPartialPartitionSpec()) {
             ResolvedPartitionSpec partitionSpecFromRequest =
@@ -423,35 +452,68 @@ public abstract class RpcServiceBase extends RpcGatewayService implements AdminR
     }
 
     @Override
-    public CompletableFuture<GetLatestLakeSnapshotResponse> getLatestLakeSnapshot(
-            GetLatestLakeSnapshotRequest request) {
+    public CompletableFuture<GetLakeSnapshotResponse> getLakeSnapshot(
+            GetLakeSnapshotRequest request) {
         // get table info
         TablePath tablePath = toTablePath(request.getTablePath());
+        authorizeTable(OperationType.DESCRIBE, tablePath);
+
+        boolean requestReadableSnapshot = request.hasReadable() && request.isReadable();
+        if (requestReadableSnapshot && request.hasSnapshotId()) {
+            CompletableFuture<GetLakeSnapshotResponse> failed = new CompletableFuture<>();
+            failed.completeExceptionally(
+                    new IllegalArgumentException(
+                            "GetLakeSnapshotRequest cannot set both snapshot_id and readable=true; "
+                                    + "use one or the other to specify either a snapshot by id or the latest readable snapshot."));
+            return failed;
+        }
+
+        // get table info
         TableInfo tableInfo = metadataManager.getTable(tablePath);
         // get table id
         long tableId = tableInfo.getTableId();
-
-        Optional<LakeTableSnapshot> optLakeTableSnapshot;
-        try {
-            optLakeTableSnapshot = zkClient.getLakeTableSnapshot(tableId);
-        } catch (Exception e) {
-            throw new FlussRuntimeException(
-                    String.format(
-                            "Failed to get lake table snapshot for table: %s, table id: %d",
-                            tablePath, tableId),
-                    e);
-        }
-
-        if (!optLakeTableSnapshot.isPresent()) {
-            throw new LakeTableSnapshotNotExistException(
-                    String.format(
-                            "Lake table snapshot not exist for table: %s, table id: %d",
-                            tablePath, tableId));
-        }
-
-        LakeTableSnapshot lakeTableSnapshot = optLakeTableSnapshot.get();
-        return CompletableFuture.completedFuture(
-                makeGetLatestLakeSnapshotResponse(tableId, lakeTableSnapshot));
+        CompletableFuture<GetLakeSnapshotResponse> resultFuture = new CompletableFuture<>();
+        ioExecutor.execute(
+                () -> {
+                    try {
+                        Optional<LakeTableSnapshot> optSnapshot =
+                                requestReadableSnapshot
+                                        ? zkClient.getLatestReadableLakeTableSnapshot(tableId)
+                                        : zkClient.getLakeTableSnapshot(
+                                                tableId,
+                                                request.hasSnapshotId()
+                                                        ? request.getSnapshotId()
+                                                        : null);
+                        if (optSnapshot.isPresent()) {
+                            resultFuture.complete(
+                                    makeGetLakeSnapshotResponse(tableId, optSnapshot.get()));
+                        } else {
+                            String snapshotType =
+                                    requestReadableSnapshot ? "readable snapshot" : "snapshot";
+                            StringBuilder errorMsg =
+                                    new StringBuilder()
+                                            .append("Lake table ")
+                                            .append(snapshotType)
+                                            .append(" doesn't exist for table: ")
+                                            .append(tablePath)
+                                            .append(", table id: ")
+                                            .append(tableId);
+                            if (!requestReadableSnapshot && request.hasSnapshotId()) {
+                                errorMsg.append(", snapshot id: ").append(request.getSnapshotId());
+                            }
+                            resultFuture.completeExceptionally(
+                                    new LakeTableSnapshotNotExistException(errorMsg.toString()));
+                        }
+                    } catch (Exception e) {
+                        resultFuture.completeExceptionally(
+                                new FlussRuntimeException(
+                                        String.format(
+                                                "Failed to get lake table snapshot for table: %s, table id: %d",
+                                                tablePath, tableId),
+                                        e));
+                    }
+                });
+        return resultFuture;
     }
 
     @Override
