@@ -18,6 +18,7 @@
 package org.apache.fluss.server.coordinator;
 
 import org.apache.fluss.annotation.VisibleForTesting;
+import org.apache.fluss.cluster.rebalance.ServerTag;
 import org.apache.fluss.metadata.PhysicalTablePath;
 import org.apache.fluss.metadata.TableBucket;
 import org.apache.fluss.metadata.TableBucketReplica;
@@ -102,6 +103,9 @@ public class CoordinatorContext {
      */
     private final Map<Integer, Set<TableBucket>> replicasOnOffline = new HashMap<>();
 
+    /** A mapping from tabletServers to server tag. */
+    private final Map<Integer, ServerTag> serverTags = new HashMap<>();
+
     private ServerInfo coordinatorServerInfo = null;
     private int coordinatorEpoch = INITIAL_COORDINATOR_EPOCH;
 
@@ -180,7 +184,22 @@ public class CoordinatorContext {
         return tablePathById;
     }
 
-    public Set<TableBucket> allBuckets() {
+    /**
+     * Returns the number of lake tables (tables with datalake enabled) in the cluster.
+     *
+     * @return the count of lake tables
+     */
+    public int getLakeTableCount() {
+        int count = 0;
+        for (TableInfo tableInfo : tableInfoById.values()) {
+            if (tableInfo.getTableConfig().isDataLakeEnabled()) {
+                count++;
+            }
+        }
+        return count;
+    }
+
+    public Set<TableBucket> getAllBuckets() {
         Set<TableBucket> allBuckets = new HashSet<>();
         for (Map.Entry<Long, Map<Integer, List<Integer>>> tableAssign :
                 tableAssignments.entrySet()) {
@@ -635,6 +654,26 @@ public class CoordinatorContext {
         }
     }
 
+    public void initSeverTags(Map<Integer, ServerTag> initialServerTags) {
+        serverTags.putAll(initialServerTags);
+    }
+
+    public void putServerTag(int serverId, ServerTag serverTag) {
+        serverTags.put(serverId, serverTag);
+    }
+
+    public Map<Integer, ServerTag> getServerTags() {
+        return new HashMap<>(serverTags);
+    }
+
+    public Optional<ServerTag> getServerTag(int serverId) {
+        return Optional.ofNullable(serverTags.get(serverId));
+    }
+
+    public void removeServerTag(int serverId) {
+        serverTags.remove(serverId);
+    }
+
     private void clearTablesState() {
         tableAssignments.clear();
         partitionAssignments.clear();
@@ -656,6 +695,7 @@ public class CoordinatorContext {
         // clear the live tablet servers
         liveTabletServers.clear();
         shuttingDownTabletServers.clear();
+        serverTags.clear();
     }
 
     public int getTotalPartitionCount() {
