@@ -82,20 +82,22 @@ import static org.apache.fluss.utils.concurrent.LockUtils.inLock;
  * └──┬──┘ └──┬───┘
  *    ▼       ▼
  *  ┌──────────┐ (lake freshness > tiering interval)
- *  │Scheduled ├──────┐
- *  └──────────┘      ▼
- *       ▲        ┌───────┐ (assign to tier service)  ┌───────┐
- *       |        |Pending├──────────────────────────►|Tiering├─┐
- *       |        └───────┘                           └───┬───┘ │
- *       |            ▲                 ┌─────────────────┘     │
- *       |            |                 | (timeout or failure)  | (finished)
- *       |            |                 ▼                       ▼
- *       |            |  (retry)   ┌─────────┐             ┌────────┐
- *       |            └────────────│ Failed  │             │ Tiered │
- *       |                         └─────────┘             └────┬───┘
- *       |                                                      |
- *       └──────────────────────────────────────────────────────┘
- *                   (ready for next round of tiering)
+ *  │Scheduled ├─────┐
+ *  └─────▲────┘     │
+ *        │      ┌───▼───┐ (assign to tier service)  ┌───────┐
+ *        │      |Pending├──────────────────────────►│Tiering├─┐
+ *        │      └───▲───┘                           └───┬───┘ │
+ *        │          │                 ┌─────────────────┘     │
+ *        │          │                 │ (timeout or failure)  │ (finished)
+ *        │          │                 ▼                       ▼
+ *        │          │  (retry)   ┌─────────┐             ┌────────┐
+ *        │          │◀───────────│ Failed  │             │ Tiered │
+ *        │          │            └─────────┘             └─┬───┬──┘
+ *        │          │                                      │   │
+ *        │          │           (force finished)           │   │
+ *        │          └─────────────────────────────────────-┘   │
+ *        │                     (ready for next round)          │
+ *        └─────────────────────────────────────────────────────┘
  * }</pre>
  */
 public class LakeTableTieringManager implements AutoCloseable {
@@ -130,6 +132,9 @@ public class LakeTableTieringManager implements AutoCloseable {
     // from table_id -> last heartbeat time by the tiering service
     private final Map<Long, Long> liveTieringTableIds;
 
+    // table_id -> delayed tiering task
+    private final Map<Long, DelayedTiering> delayedTieringByTableId;
+
     private final Lock lock = new ReentrantLock();
 
     public LakeTableTieringManager() {
@@ -159,6 +164,7 @@ public class LakeTableTieringManager implements AutoCloseable {
                 this::checkTieringServiceTimeout, 0, 15, TimeUnit.SECONDS);
         this.tableTierEpoch = new HashMap<>();
         this.tableLastTieredTime = new HashMap<>();
+        this.delayedTieringByTableId = new HashMap<>();
     }
 
     public void initWithLakeTables(List<Tuple2<TableInfo, Long>> tableInfoWithTieredTime) {
@@ -205,10 +211,17 @@ public class LakeTableTieringManager implements AutoCloseable {
             // the table has been dropped, return directly
             return;
         }
+        // Before reschedule, remove the existing DelayedTiering if present
+        DelayedTiering existingDelayedTiering = delayedTieringByTableId.remove(tableId);
+        if (existingDelayedTiering != null) {
+            existingDelayedTiering.cancel();
+        }
         long delayMs = freshnessInterval - (clock.milliseconds() - lastTieredTime);
         // if the delayMs is < 0, the DelayedTiering will be triggered at once without
         // adding into timing wheel.
-        lakeTieringScheduleTimer.add(new DelayedTiering(tableId, delayMs));
+        DelayedTiering delayedTiering = new DelayedTiering(tableId, delayMs);
+        delayedTieringByTableId.put(tableId, delayedTiering);
+        lakeTieringScheduleTimer.add(delayedTiering);
     }
 
     public void removeLakeTable(long tableId) {
@@ -221,6 +234,53 @@ public class LakeTableTieringManager implements AutoCloseable {
                     tieringStates.remove(tableId);
                     liveTieringTableIds.remove(tableId);
                     tableTierEpoch.remove(tableId);
+                    // Remove and cancel the delayed tiering task if present
+                    DelayedTiering delayedTiering = delayedTieringByTableId.remove(tableId);
+                    if (delayedTiering != null) {
+                        delayedTiering.cancel();
+                    }
+                });
+    }
+
+    /**
+     * Update the lake freshness for a table. This method should be called when the table's datalake
+     * freshness property is changed via ALTER TABLE.
+     *
+     * @param tableId the table id
+     * @param newFreshnessMs the new freshness interval in milliseconds
+     */
+    public void updateTableLakeFreshness(long tableId, long newFreshnessMs) {
+        inLock(
+                lock,
+                () -> {
+                    Long currentFreshness = tableLakeFreshness.get(tableId);
+                    if (currentFreshness == null) {
+                        // the table is not a lake table or has been dropped, skip update
+                        LOG.warn(
+                                "Cannot update lake freshness for table {} as it's not tracked by lake tiering manager.",
+                                tableId);
+                        return;
+                    }
+
+                    if (currentFreshness.equals(newFreshnessMs)) {
+                        // no change, skip update
+                        return;
+                    }
+
+                    tableLakeFreshness.put(tableId, newFreshnessMs);
+                    LOG.info(
+                            "Updated lake freshness for table {} from {} ms to {} ms.",
+                            tableId,
+                            currentFreshness,
+                            newFreshnessMs);
+
+                    // If the table is in Scheduled state, we need to reschedule it with the new
+                    // freshness
+                    TieringState currentState = tieringStates.get(tableId);
+                    if (currentState == TieringState.Scheduled) {
+                        // Reschedule the table tiering with the new freshness interval
+                        scheduleTableTiering(tableId);
+                    }
                 });
     }
 
@@ -272,15 +332,20 @@ public class LakeTableTieringManager implements AutoCloseable {
                 });
     }
 
-    public void finishTableTiering(long tableId, long tieredEpoch) {
+    public void finishTableTiering(long tableId, long tieredEpoch, boolean isForceFinished) {
         inLock(
                 lock,
                 () -> {
                     validateTieringServiceRequest(tableId, tieredEpoch);
                     // to tiered state firstly
                     doHandleStateChange(tableId, TieringState.Tiered);
-                    // then to scheduled state to enable other tiering service can pick it
-                    doHandleStateChange(tableId, TieringState.Scheduled);
+                    if (isForceFinished) {
+                        // add to pending again since it's forced to finish
+                        doHandleStateChange(tableId, TieringState.Pending);
+                    } else {
+                        // then to scheduled state to enable other tiering service can pick it
+                        doHandleStateChange(tableId, TieringState.Scheduled);
+                    }
                 });
     }
 
@@ -362,7 +427,17 @@ public class LakeTableTieringManager implements AutoCloseable {
      * <p>Tiering -> Failed
      *
      * <p>-- When the tiering service timeout to report heartbeat or report failure for the table,
-     * do: transmit to Tiered state
+     * do: transmit to Failed state
+     *
+     * <p>Tiered -> Pending
+     *
+     * <p>-- When the tiering is force finished due to exceeding the specified tiering duration, do:
+     * transmit to Pending state to enable immediate re-tiering
+     *
+     * <p>Tiered -> Scheduled
+     *
+     * <p>-- When the tiering is normally finished, do: transmit to Scheduled state to wait for the
+     * next round of tiering
      */
     private void doHandleStateChange(long tableId, TieringState targetState) {
         TieringState currentState = tieringStates.get(tableId);
@@ -458,9 +533,12 @@ public class LakeTableTieringManager implements AutoCloseable {
         public void run() {
             inLock(
                     lock,
-                    () ->
-                            // to pending state
-                            doHandleStateChange(tableId, TieringState.Pending));
+                    () -> {
+                        // to pending state
+                        doHandleStateChange(tableId, TieringState.Pending);
+                        // Remove from map after execution
+                        delayedTieringByTableId.remove(tableId);
+                    });
         }
     }
 
@@ -509,7 +587,7 @@ public class LakeTableTieringManager implements AutoCloseable {
         Pending {
             @Override
             public Set<TieringState> validPreviousStates() {
-                return EnumSet.of(Scheduled, Failed);
+                return EnumSet.of(Scheduled, Failed, Tiered);
             }
         },
         // When one tiering service is tiering the table, the state will be Tiering
