@@ -22,34 +22,48 @@ import org.apache.fluss.compression.ArrowCompressionInfo;
 import org.apache.fluss.config.ConfigOptions;
 import org.apache.fluss.config.Configuration;
 import org.apache.fluss.exception.DeletionDisabledException;
+import org.apache.fluss.exception.InvalidTableException;
 import org.apache.fluss.exception.KvStorageException;
+import org.apache.fluss.exception.SchemaNotExistException;
 import org.apache.fluss.memory.MemorySegmentPool;
+import org.apache.fluss.metadata.ChangelogImage;
 import org.apache.fluss.metadata.DeleteBehavior;
 import org.apache.fluss.metadata.KvFormat;
 import org.apache.fluss.metadata.LogFormat;
 import org.apache.fluss.metadata.PhysicalTablePath;
 import org.apache.fluss.metadata.Schema;
+import org.apache.fluss.metadata.SchemaGetter;
+import org.apache.fluss.metadata.SchemaInfo;
 import org.apache.fluss.metadata.TableBucket;
 import org.apache.fluss.metadata.TablePath;
+import org.apache.fluss.record.BinaryValue;
 import org.apache.fluss.record.ChangeType;
 import org.apache.fluss.record.KvRecord;
 import org.apache.fluss.record.KvRecordBatch;
 import org.apache.fluss.record.KvRecordReadContext;
 import org.apache.fluss.row.BinaryRow;
+import org.apache.fluss.row.PaddingRow;
 import org.apache.fluss.row.arrow.ArrowWriterPool;
 import org.apache.fluss.row.arrow.ArrowWriterProvider;
 import org.apache.fluss.row.encode.ValueDecoder;
-import org.apache.fluss.row.encode.ValueEncoder;
+import org.apache.fluss.rpc.protocol.MergeMode;
+import org.apache.fluss.server.kv.autoinc.AutoIncIDRange;
+import org.apache.fluss.server.kv.autoinc.AutoIncrementManager;
+import org.apache.fluss.server.kv.autoinc.AutoIncrementUpdater;
 import org.apache.fluss.server.kv.prewrite.KvPreWriteBuffer;
 import org.apache.fluss.server.kv.prewrite.KvPreWriteBuffer.TruncateReason;
 import org.apache.fluss.server.kv.rocksdb.RocksDBKv;
 import org.apache.fluss.server.kv.rocksdb.RocksDBKvBuilder;
 import org.apache.fluss.server.kv.rocksdb.RocksDBResourceContainer;
+import org.apache.fluss.server.kv.rocksdb.RocksDBStatistics;
+import org.apache.fluss.server.kv.rowmerger.DefaultRowMerger;
 import org.apache.fluss.server.kv.rowmerger.RowMerger;
 import org.apache.fluss.server.kv.snapshot.KvFileHandleAndLocalPath;
 import org.apache.fluss.server.kv.snapshot.KvSnapshotDataUploader;
 import org.apache.fluss.server.kv.snapshot.RocksIncrementalSnapshot;
+import org.apache.fluss.server.kv.snapshot.TabletState;
 import org.apache.fluss.server.kv.wal.ArrowWalBuilder;
+import org.apache.fluss.server.kv.wal.CompactedWalBuilder;
 import org.apache.fluss.server.kv.wal.IndexWalBuilder;
 import org.apache.fluss.server.kv.wal.WalBuilder;
 import org.apache.fluss.server.log.LogAppendInfo;
@@ -57,13 +71,11 @@ import org.apache.fluss.server.log.LogTablet;
 import org.apache.fluss.server.metrics.group.TabletServerMetricGroup;
 import org.apache.fluss.server.utils.FatalErrorHandler;
 import org.apache.fluss.shaded.arrow.org.apache.arrow.memory.BufferAllocator;
-import org.apache.fluss.types.DataType;
 import org.apache.fluss.types.RowType;
 import org.apache.fluss.utils.BytesUtils;
 import org.apache.fluss.utils.FileUtils;
-import org.apache.fluss.utils.FlussPaths;
-import org.apache.fluss.utils.types.Tuple2;
 
+import org.rocksdb.RateLimiter;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -87,6 +99,7 @@ import static org.apache.fluss.utils.concurrent.LockUtils.inWriteLock;
 @ThreadSafe
 public final class KvTablet {
     private static final Logger LOG = LoggerFactory.getLogger(KvTablet.class);
+    private static final long ROW_COUNT_DISABLED = -1;
 
     private final PhysicalTablePath physicalPath;
     private final TableBucket tableBucket;
@@ -99,15 +112,27 @@ public final class KvTablet {
     private final long writeBatchSize;
     private final RocksDBKv rocksDBKv;
     private final KvPreWriteBuffer kvPreWriteBuffer;
+    private final TabletServerMetricGroup serverMetricGroup;
 
     // A lock that guards all modifications to the kv.
     private final ReadWriteLock kvLock = new ReentrantReadWriteLock();
     private final LogFormat logFormat;
     private final KvFormat kvFormat;
-    private final Schema schema;
     // defines how to merge rows on the same primary key
     private final RowMerger rowMerger;
+    // Pre-created DefaultRowMerger for OVERWRITE mode (undo recovery scenarios)
+    // This avoids creating a new instance on every putAsLeader call
+    private final RowMerger overwriteRowMerger;
     private final ArrowCompressionInfo arrowCompressionInfo;
+    private final AutoIncrementManager autoIncrementManager;
+
+    private final SchemaGetter schemaGetter;
+
+    // the changelog image mode for this tablet
+    private final ChangelogImage changelogImage;
+
+    // RocksDB statistics accessor for this tablet
+    @Nullable private final RocksDBStatistics rocksDBStatistics;
 
     /**
      * The kv data in pre-write buffer whose log offset is less than the flushedLogOffset has been
@@ -115,6 +140,8 @@ public final class KvTablet {
      */
     private volatile long flushedLogOffset = 0;
 
+    private volatile long rowCount;
+
     @GuardedBy("kvLock")
     private volatile boolean isClosed = false;
 
@@ -130,52 +157,35 @@ public final class KvTablet {
             BufferAllocator arrowBufferAllocator,
             MemorySegmentPool memorySegmentPool,
             KvFormat kvFormat,
-            Schema schema,
             RowMerger rowMerger,
-            ArrowCompressionInfo arrowCompressionInfo) {
+            ArrowCompressionInfo arrowCompressionInfo,
+            SchemaGetter schemaGetter,
+            ChangelogImage changelogImage,
+            @Nullable RocksDBStatistics rocksDBStatistics,
+            AutoIncrementManager autoIncrementManager) {
         this.physicalPath = physicalPath;
         this.tableBucket = tableBucket;
         this.logTablet = logTablet;
         this.kvTabletDir = kvTabletDir;
         this.rocksDBKv = rocksDBKv;
         this.writeBatchSize = writeBatchSize;
+        this.serverMetricGroup = serverMetricGroup;
         this.kvPreWriteBuffer = new KvPreWriteBuffer(createKvBatchWriter(), serverMetricGroup);
         this.logFormat = logFormat;
         this.arrowWriterProvider = new ArrowWriterPool(arrowBufferAllocator);
         this.memorySegmentPool = memorySegmentPool;
         this.kvFormat = kvFormat;
-        this.schema = schema;
         this.rowMerger = rowMerger;
+        // Pre-create DefaultRowMerger for OVERWRITE mode to avoid creating new instances
+        // on every putAsLeader call. Used for undo recovery scenarios.
+        this.overwriteRowMerger = new DefaultRowMerger(kvFormat, DeleteBehavior.ALLOW);
         this.arrowCompressionInfo = arrowCompressionInfo;
-    }
-
-    public static KvTablet create(
-            LogTablet logTablet,
-            File kvTabletDir,
-            Configuration serverConf,
-            TabletServerMetricGroup serverMetricGroup,
-            BufferAllocator arrowBufferAllocator,
-            MemorySegmentPool memorySegmentPool,
-            KvFormat kvFormat,
-            Schema schema,
-            RowMerger rowMerger,
-            ArrowCompressionInfo arrowCompressionInfo)
-            throws IOException {
-        Tuple2<PhysicalTablePath, TableBucket> tablePathAndBucket =
-                FlussPaths.parseTabletDir(kvTabletDir);
-        return create(
-                tablePathAndBucket.f0,
-                tablePathAndBucket.f1,
-                logTablet,
-                kvTabletDir,
-                serverConf,
-                serverMetricGroup,
-                arrowBufferAllocator,
-                memorySegmentPool,
-                kvFormat,
-                schema,
-                rowMerger,
-                arrowCompressionInfo);
+        this.schemaGetter = schemaGetter;
+        this.changelogImage = changelogImage;
+        this.rocksDBStatistics = rocksDBStatistics;
+        this.autoIncrementManager = autoIncrementManager;
+        // disable row count for WAL image mode.
+        this.rowCount = changelogImage == ChangelogImage.WAL ? ROW_COUNT_DISABLED : 0L;
     }
 
     public static KvTablet create(
@@ -188,11 +198,27 @@ public final class KvTablet {
             BufferAllocator arrowBufferAllocator,
             MemorySegmentPool memorySegmentPool,
             KvFormat kvFormat,
-            Schema schema,
             RowMerger rowMerger,
-            ArrowCompressionInfo arrowCompressionInfo)
+            ArrowCompressionInfo arrowCompressionInfo,
+            SchemaGetter schemaGetter,
+            ChangelogImage changelogImage,
+            RateLimiter sharedRateLimiter,
+            AutoIncrementManager autoIncrementManager)
             throws IOException {
-        RocksDBKv kv = buildRocksDBKv(serverConf, kvTabletDir);
+        RocksDBKv kv = buildRocksDBKv(serverConf, kvTabletDir, sharedRateLimiter);
+
+        // Create RocksDB statistics accessor (will be registered to TableMetricGroup by Replica)
+        // Pass ResourceGuard to ensure thread-safe access during concurrent close operations
+        // Pass ColumnFamilyHandle for column family specific properties like num-files-at-level0
+        // Pass Cache for accurate block cache memory tracking
+        RocksDBStatistics rocksDBStatistics =
+                new RocksDBStatistics(
+                        kv.getDb(),
+                        kv.getStatistics(),
+                        kv.getResourceGuard(),
+                        kv.getDefaultColumnFamilyHandle(),
+                        kv.getBlockCache());
+
         return new KvTablet(
                 tablePath,
                 tableBucket,
@@ -205,15 +231,20 @@ public final class KvTablet {
                 arrowBufferAllocator,
                 memorySegmentPool,
                 kvFormat,
-                schema,
                 rowMerger,
-                arrowCompressionInfo);
+                arrowCompressionInfo,
+                schemaGetter,
+                changelogImage,
+                rocksDBStatistics,
+                autoIncrementManager);
     }
 
-    private static RocksDBKv buildRocksDBKv(Configuration configuration, File kvDir)
+    private static RocksDBKv buildRocksDBKv(
+            Configuration configuration, File kvDir, RateLimiter sharedRateLimiter)
             throws IOException {
+        // Enable statistics to support RocksDB statistics collection
         RocksDBResourceContainer rocksDBResourceContainer =
-                new RocksDBResourceContainer(configuration, kvDir);
+                new RocksDBResourceContainer(configuration, kvDir, true, sharedRateLimiter);
         RocksDBKvBuilder rocksDBKvBuilder =
                 new RocksDBKvBuilder(
                         kvDir,
@@ -230,6 +261,14 @@ public final class KvTablet {
         return physicalPath.getTablePath();
     }
 
+    public long getAutoIncrementCacheSize() {
+        return autoIncrementManager.getAutoIncrementCacheSize();
+    }
+
+    public void updateAutoIncrementIDRange(AutoIncIDRange newRange) {
+        autoIncrementManager.updateIDRange(newRange);
+    }
+
     @Nullable
     public String getPartitionName() {
         return physicalPath.getPartitionName();
@@ -239,115 +278,139 @@ public final class KvTablet {
         return kvTabletDir;
     }
 
+    /**
+     * Get RocksDB statistics accessor for this tablet.
+     *
+     * @return the RocksDB statistics accessor, or null if not available
+     */
+    @Nullable
+    public RocksDBStatistics getRocksDBStatistics() {
+        return rocksDBStatistics;
+    }
+
     void setFlushedLogOffset(long flushedLogOffset) {
         this.flushedLogOffset = flushedLogOffset;
     }
 
-    public long getFlushedLogOffset() {
-        return flushedLogOffset;
+    void setRowCount(long rowCount) {
+        this.rowCount = rowCount;
+    }
+
+    // row_count is volatile, so it's safe to read without lock
+    public long getRowCount() {
+        if (rowCount == ROW_COUNT_DISABLED) {
+            throw new InvalidTableException(
+                    String.format(
+                            "Row count is disabled for this table '%s'. This usually happens when the table is"
+                                    + "created before v0.9 or the changelog image is set to WAL, "
+                                    + "as maintaining row count in WAL mode is costly and not necessary for most use cases. "
+                                    + "If you want to enable row count, please set changelog image to FULL.",
+                            getTablePath()));
+        }
+        return rowCount;
     }
 
     /**
-     * Put the KvRecordBatch into the kv storage, and return the appended wal log info.
+     * Get the current state of the tablet, including the log offset, row count and auto-increment
+     * ID range. This is used for snapshot and recovery to capture the state of the tablet at a
+     * specific log offset.
+     *
+     * <p>Note: this method must be called under the kvLock to ensure the consistency between the
+     * returned state and the log offset.
+     */
+    @GuardedBy("kvLock")
+    public TabletState getTabletState() {
+        return new TabletState(
+                flushedLogOffset,
+                rowCount == ROW_COUNT_DISABLED ? null : rowCount,
+                autoIncrementManager.getCurrentIDRanges());
+    }
+
+    /**
+     * Put the KvRecordBatch into the kv storage with default DEFAULT mode.
+     *
+     * <p>This is a convenience method that calls {@link #putAsLeader(KvRecordBatch, int[],
+     * MergeMode)} with {@link MergeMode#DEFAULT}.
      *
      * @param kvRecords the kv records to put into
      * @param targetColumns the target columns to put, null if put all columns
      */
     public LogAppendInfo putAsLeader(KvRecordBatch kvRecords, @Nullable int[] targetColumns)
             throws Exception {
+        return putAsLeader(kvRecords, targetColumns, MergeMode.DEFAULT);
+    }
+
+    /**
+     * Put the KvRecordBatch into the kv storage, and return the appended wal log info.
+     *
+     * <p>Schema Evolution Handling:
+     *
+     * <p>We don't allow shema of input kv records to be larger than the latest schema id known by
+     * the tablet. Besides, we currently only support ADD COLUMN LAST operation, so the input row or
+     * old row must have same or fewer columns than latest schema. This helps to simplify the schema
+     * change handling.
+     *
+     * <p>1. We write the kv records into KvStore without converting it into latest schema for
+     * performance consideration. We have mechanisms that writer client dynamically use latest
+     * schema for writing records.
+     *
+     * <p>2. We always use the latest schema for writing WAL logs, because it anyway happens
+     * deserialization&serialization to convert the compacted format into Arrow format.
+     *
+     * @param kvRecords the kv records to put into
+     * @param targetColumns the target columns to put, null if put all columns
+     * @param mergeMode the merge mode (DEFAULT or OVERWRITE)
+     */
+    public LogAppendInfo putAsLeader(
+            KvRecordBatch kvRecords, @Nullable int[] targetColumns, MergeMode mergeMode)
+            throws Exception {
         return inWriteLock(
                 kvLock,
                 () -> {
                     rocksDBKv.checkIfRocksDBClosed();
-                    short schemaId = kvRecords.schemaId();
-                    RowMerger currentMerger = rowMerger.configureTargetColumns(targetColumns);
-                    RowType rowType = schema.getRowType();
-                    WalBuilder walBuilder = createWalBuilder(schemaId, rowType);
+
+                    SchemaInfo schemaInfo = schemaGetter.getLatestSchemaInfo();
+                    Schema latestSchema = schemaInfo.getSchema();
+                    short latestSchemaId = (short) schemaInfo.getSchemaId();
+                    validateSchemaId(kvRecords.schemaId(), latestSchemaId);
+
+                    AutoIncrementUpdater currentAutoIncrementUpdater =
+                            autoIncrementManager.getUpdaterForSchema(kvFormat, latestSchemaId);
+
+                    // Validate targetColumns doesn't contain auto-increment column
+                    currentAutoIncrementUpdater.validateTargetColumns(targetColumns);
+
+                    // Determine the row merger based on mergeMode:
+                    // - DEFAULT: Use the configured merge engine (rowMerger)
+                    // - OVERWRITE: Bypass merge engine, use pre-created overwriteRowMerger
+                    //   to directly replace values (for undo recovery scenarios)
+                    // We only support ADD COLUMN, so targetColumns is fine to be used directly.
+                    RowMerger currentMerger =
+                            (mergeMode == MergeMode.OVERWRITE)
+                                    ? overwriteRowMerger.configureTargetColumns(
+                                            targetColumns, latestSchemaId, latestSchema)
+                                    : rowMerger.configureTargetColumns(
+                                            targetColumns, latestSchemaId, latestSchema);
+
+                    RowType latestRowType = latestSchema.getRowType();
+                    WalBuilder walBuilder = createWalBuilder(latestSchemaId, latestRowType);
                     walBuilder.setWriterState(kvRecords.writerId(), kvRecords.batchSequence());
+                    // we only support ADD COLUMN LAST, so the BinaryRow after RowMerger is
+                    // only has fewer ending columns than latest schema, so we pad nulls to
+                    // the end of the BinaryRow to get the latest schema row.
+                    PaddingRow latestSchemaRow = new PaddingRow(latestRowType.getFieldCount());
                     // get offset to track the offset corresponded to the kv record
                     long logEndOffsetOfPrevBatch = logTablet.localLogEndOffset();
-                    DataType[] fieldTypes = rowType.getChildren().toArray(new DataType[0]);
-                    try {
-                        long logOffset = logEndOffsetOfPrevBatch;
 
-                        // TODO: reuse the read context and decoder
-                        KvRecordBatch.ReadContext readContext =
-                                KvRecordReadContext.createReadContext(kvFormat, fieldTypes);
-                        ValueDecoder valueDecoder =
-                                new ValueDecoder(readContext.getRowDecoder(schemaId));
-                        for (KvRecord kvRecord : kvRecords.records(readContext)) {
-                            byte[] keyBytes = BytesUtils.toArray(kvRecord.getKey());
-                            KvPreWriteBuffer.Key key = KvPreWriteBuffer.Key.of(keyBytes);
-                            if (kvRecord.getRow() == null) {
-                                DeleteBehavior deleteBehavior = rowMerger.deleteBehavior();
-                                if (deleteBehavior == DeleteBehavior.IGNORE) {
-                                    // skip delete rows if the merger doesn't support yet
-                                    continue;
-                                } else if (deleteBehavior == DeleteBehavior.DISABLE) {
-                                    throw new DeletionDisabledException(
-                                            "Delete operations are disabled for this table. "
-                                                    + "The table.delete.behavior is set to 'disable'.");
-                                }
-                                // it's for deletion
-                                byte[] oldValue = getFromBufferOrKv(key);
-                                if (oldValue == null) {
-                                    // there might be large amount of such deletion, so we don't log
-                                    LOG.debug(
-                                            "The specific key can't be found in kv tablet although the kv record is for deletion, "
-                                                    + "ignore it directly as it doesn't exist in the kv tablet yet.");
-                                } else {
-                                    BinaryRow oldRow = valueDecoder.decodeValue(oldValue).row;
-                                    BinaryRow newRow = currentMerger.delete(oldRow);
-                                    // if newRow is null, it means the row should be deleted
-                                    if (newRow == null) {
-                                        walBuilder.append(ChangeType.DELETE, oldRow);
-                                        kvPreWriteBuffer.delete(key, logOffset++);
-                                    } else {
-                                        // otherwise, it's a partial update, should produce -U,+U
-                                        walBuilder.append(ChangeType.UPDATE_BEFORE, oldRow);
-                                        walBuilder.append(ChangeType.UPDATE_AFTER, newRow);
-                                        kvPreWriteBuffer.put(
-                                                key,
-                                                ValueEncoder.encodeValue(schemaId, newRow),
-                                                logOffset + 1);
-                                        logOffset += 2;
-                                    }
-                                }
-                            } else {
-                                // upsert operation
-                                byte[] oldValue = getFromBufferOrKv(key);
-                                // it's update
-                                if (oldValue != null) {
-                                    BinaryRow oldRow = valueDecoder.decodeValue(oldValue).row;
-                                    BinaryRow newRow =
-                                            currentMerger.merge(oldRow, kvRecord.getRow());
-                                    if (newRow == oldRow) {
-                                        // newRow is the same to oldRow, means nothing
-                                        // happens (no update/delete), and input should be ignored
-                                        continue;
-                                    }
-                                    walBuilder.append(ChangeType.UPDATE_BEFORE, oldRow);
-                                    walBuilder.append(ChangeType.UPDATE_AFTER, newRow);
-                                    // logOffset is for -U, logOffset + 1 is for +U, we need to use
-                                    // the log offset for +U
-                                    kvPreWriteBuffer.put(
-                                            key,
-                                            ValueEncoder.encodeValue(schemaId, newRow),
-                                            logOffset + 1);
-                                    logOffset += 2;
-                                } else {
-                                    // it's insert
-                                    // TODO: we should add guarantees that all non-specified columns
-                                    //  of the input row are set to null.
-                                    BinaryRow newRow = kvRecord.getRow();
-                                    walBuilder.append(ChangeType.INSERT, newRow);
-                                    kvPreWriteBuffer.put(
-                                            key,
-                                            ValueEncoder.encodeValue(schemaId, newRow),
-                                            logOffset++);
-                                }
-                            }
-                        }
+                    try {
+                        processKvRecords(
+                                kvRecords,
+                                kvRecords.schemaId(),
+                                currentMerger,
+                                currentAutoIncrementUpdater,
+                                walBuilder,
+                                latestSchemaRow,
+                                logEndOffsetOfPrevBatch);
 
                         // There will be a situation that these batches of kvRecordBatch have not
                         // generated any CDC logs, for example, when client attempts to delete
@@ -384,6 +447,187 @@ public final class KvTablet {
                 });
     }
 
+    private void validateSchemaId(short schemaIdOfNewData, short latestSchemaId) {
+        if (schemaIdOfNewData > latestSchemaId || schemaIdOfNewData < 0) {
+            throw new SchemaNotExistException(
+                    "Invalid schema id: "
+                            + schemaIdOfNewData
+                            + ", latest schema id: "
+                            + latestSchemaId);
+        }
+    }
+
+    private void processKvRecords(
+            KvRecordBatch kvRecords,
+            short schemaIdOfNewData,
+            RowMerger currentMerger,
+            AutoIncrementUpdater autoIncrementUpdater,
+            WalBuilder walBuilder,
+            PaddingRow latestSchemaRow,
+            long startLogOffset)
+            throws Exception {
+        long logOffset = startLogOffset;
+
+        // TODO: reuse the read context and decoder
+        KvRecordBatch.ReadContext readContext =
+                KvRecordReadContext.createReadContext(kvFormat, schemaGetter);
+        ValueDecoder valueDecoder = new ValueDecoder(schemaGetter, kvFormat);
+
+        for (KvRecord kvRecord : kvRecords.records(readContext)) {
+            byte[] keyBytes = BytesUtils.toArray(kvRecord.getKey());
+            KvPreWriteBuffer.Key key = KvPreWriteBuffer.Key.of(keyBytes);
+            BinaryRow row = kvRecord.getRow();
+            BinaryValue currentValue = row == null ? null : new BinaryValue(schemaIdOfNewData, row);
+
+            if (currentValue == null) {
+                logOffset =
+                        processDeletion(
+                                key,
+                                currentMerger,
+                                valueDecoder,
+                                walBuilder,
+                                latestSchemaRow,
+                                logOffset);
+            } else {
+                logOffset =
+                        processUpsert(
+                                key,
+                                currentValue,
+                                currentMerger,
+                                autoIncrementUpdater,
+                                valueDecoder,
+                                walBuilder,
+                                latestSchemaRow,
+                                logOffset);
+            }
+        }
+    }
+
+    private long processDeletion(
+            KvPreWriteBuffer.Key key,
+            RowMerger currentMerger,
+            ValueDecoder valueDecoder,
+            WalBuilder walBuilder,
+            PaddingRow latestSchemaRow,
+            long logOffset)
+            throws Exception {
+        DeleteBehavior deleteBehavior = currentMerger.deleteBehavior();
+        if (deleteBehavior == DeleteBehavior.IGNORE) {
+            // skip delete rows if the merger doesn't support yet
+            return logOffset;
+        } else if (deleteBehavior == DeleteBehavior.DISABLE) {
+            throw new DeletionDisabledException(
+                    "Delete operations are disabled for this table. "
+                            + "The table.delete.behavior is set to 'disable'.");
+        }
+
+        byte[] oldValueBytes = getFromBufferOrKv(key);
+        if (oldValueBytes == null) {
+            LOG.debug(
+                    "The specific key can't be found in kv tablet although the kv record is for deletion, "
+                            + "ignore it directly as it doesn't exist in the kv tablet yet.");
+            return logOffset;
+        }
+
+        BinaryValue oldValue = valueDecoder.decodeValue(oldValueBytes);
+        BinaryValue newValue = currentMerger.delete(oldValue);
+
+        // if newValue is null, it means the row should be deleted
+        if (newValue == null) {
+            return applyDelete(key, oldValue, walBuilder, latestSchemaRow, logOffset);
+        } else {
+            return applyUpdate(key, oldValue, newValue, walBuilder, latestSchemaRow, logOffset);
+        }
+    }
+
+    private long processUpsert(
+            KvPreWriteBuffer.Key key,
+            BinaryValue currentValue,
+            RowMerger currentMerger,
+            AutoIncrementUpdater autoIncrementUpdater,
+            ValueDecoder valueDecoder,
+            WalBuilder walBuilder,
+            PaddingRow latestSchemaRow,
+            long logOffset)
+            throws Exception {
+        // Optimization: IN WAL mode，when using DefaultRowMerger (full update, not partial update)
+        // and there is no auto-increment column, we can skip fetching old value for better
+        // performance since the result always reflects the new value. In this case, both INSERT and
+        // UPDATE will produce UPDATE_AFTER.
+        if (changelogImage == ChangelogImage.WAL
+                && !autoIncrementUpdater.hasAutoIncrement()
+                && currentMerger instanceof DefaultRowMerger) {
+            return applyUpdate(key, null, currentValue, walBuilder, latestSchemaRow, logOffset);
+        }
+
+        byte[] oldValueBytes = getFromBufferOrKv(key);
+        if (oldValueBytes == null) {
+            return applyInsert(
+                    key,
+                    currentValue,
+                    walBuilder,
+                    latestSchemaRow,
+                    logOffset,
+                    autoIncrementUpdater);
+        }
+
+        BinaryValue oldValue = valueDecoder.decodeValue(oldValueBytes);
+        BinaryValue newValue = currentMerger.merge(oldValue, currentValue);
+
+        if (newValue == oldValue) {
+            // no actual change, skip this record
+            return logOffset;
+        }
+
+        return applyUpdate(key, oldValue, newValue, walBuilder, latestSchemaRow, logOffset);
+    }
+
+    private long applyDelete(
+            KvPreWriteBuffer.Key key,
+            BinaryValue oldValue,
+            WalBuilder walBuilder,
+            PaddingRow latestSchemaRow,
+            long logOffset)
+            throws Exception {
+        walBuilder.append(ChangeType.DELETE, latestSchemaRow.replaceRow(oldValue.row));
+        kvPreWriteBuffer.delete(key, logOffset);
+        return logOffset + 1;
+    }
+
+    private long applyInsert(
+            KvPreWriteBuffer.Key key,
+            BinaryValue currentValue,
+            WalBuilder walBuilder,
+            PaddingRow latestSchemaRow,
+            long logOffset,
+            AutoIncrementUpdater autoIncrementUpdater)
+            throws Exception {
+        BinaryValue newValue = autoIncrementUpdater.updateAutoIncrementColumns(currentValue);
+        walBuilder.append(ChangeType.INSERT, latestSchemaRow.replaceRow(newValue.row));
+        kvPreWriteBuffer.insert(key, newValue.encodeValue(), logOffset);
+        return logOffset + 1;
+    }
+
+    private long applyUpdate(
+            KvPreWriteBuffer.Key key,
+            BinaryValue oldValue,
+            BinaryValue newValue,
+            WalBuilder walBuilder,
+            PaddingRow latestSchemaRow,
+            long logOffset)
+            throws Exception {
+        if (changelogImage == ChangelogImage.WAL) {
+            walBuilder.append(ChangeType.UPDATE_AFTER, latestSchemaRow.replaceRow(newValue.row));
+            kvPreWriteBuffer.update(key, newValue.encodeValue(), logOffset);
+            return logOffset + 1;
+        } else {
+            walBuilder.append(ChangeType.UPDATE_BEFORE, latestSchemaRow.replaceRow(oldValue.row));
+            walBuilder.append(ChangeType.UPDATE_AFTER, latestSchemaRow.replaceRow(newValue.row));
+            kvPreWriteBuffer.update(key, newValue.encodeValue(), logOffset + 1);
+            return logOffset + 2;
+        }
+    }
+
     private WalBuilder createWalBuilder(int schemaId, RowType rowType) throws Exception {
         switch (logFormat) {
             case INDEXED:
@@ -396,6 +640,8 @@ public final class KvTablet {
                             "Primary Key Table with COMPACTED kv format doesn't support INDEXED cdc log format.");
                 }
                 return new IndexWalBuilder(schemaId, memorySegmentPool);
+            case COMPACTED:
+                return new CompactedWalBuilder(schemaId, rowType, memorySegmentPool);
             case ARROW:
                 return new ArrowWalBuilder(
                         schemaId,
@@ -432,8 +678,13 @@ public final class KvTablet {
                                 tableBucket);
                     } else {
                         try {
-                            kvPreWriteBuffer.flush(exclusiveUpToLogOffset);
+                            int rowCountDiff = kvPreWriteBuffer.flush(exclusiveUpToLogOffset);
                             flushedLogOffset = exclusiveUpToLogOffset;
+                            if (rowCount != ROW_COUNT_DISABLED) {
+                                // row count is enabled, we update the row count after flush.
+                                long currentRowCount = rowCount;
+                                rowCount = currentRowCount + rowCountDiff;
+                            }
                         } catch (Throwable t) {
                             fatalErrorHandler.onFatalError(
                                     new KvStorageException("Failed to flush kv pre-write buffer."));
@@ -443,12 +694,18 @@ public final class KvTablet {
     }
 
     /** put key,value,logOffset into pre-write buffer directly. */
-    void putToPreWriteBuffer(byte[] key, @Nullable byte[] value, long logOffset) {
+    void putToPreWriteBuffer(
+            ChangeType changeType, byte[] key, @Nullable byte[] value, long logOffset) {
         KvPreWriteBuffer.Key wrapKey = KvPreWriteBuffer.Key.of(key);
-        if (value == null) {
+        if (changeType == ChangeType.DELETE && value == null) {
             kvPreWriteBuffer.delete(wrapKey, logOffset);
+        } else if (changeType == ChangeType.INSERT) {
+            kvPreWriteBuffer.insert(wrapKey, value, logOffset);
+        } else if (changeType == ChangeType.UPDATE_AFTER) {
+            kvPreWriteBuffer.update(wrapKey, value, logOffset);
         } else {
-            kvPreWriteBuffer.put(wrapKey, value, logOffset);
+            throw new IllegalArgumentException(
+                    "Unsupported change type for putToPreWriteBuffer: " + changeType);
         }
     }
 
@@ -500,7 +757,10 @@ public final class KvTablet {
     }
 
     public KvBatchWriter createKvBatchWriter() {
-        return rocksDBKv.newWriteBatch(writeBatchSize);
+        return rocksDBKv.newWriteBatch(
+                writeBatchSize,
+                serverMetricGroup.kvFlushCount(),
+                serverMetricGroup.kvFlushLatencyHistogram());
     }
 
     public void close() throws Exception {
@@ -511,6 +771,8 @@ public final class KvTablet {
                     if (isClosed) {
                         return;
                     }
+                    // Note: RocksDB metrics lifecycle is managed by TableMetricGroup
+                    // No need to close it here
                     if (rocksDBKv != null) {
                         rocksDBKv.close();
                     }
