@@ -44,6 +44,7 @@ import javax.annotation.concurrent.NotThreadSafe;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.file.NoSuchFileException;
 import java.util.Optional;
 
 import static org.apache.fluss.record.LogRecordBatchFormat.V0_RECORD_BATCH_HEADER_SIZE;
@@ -172,6 +173,24 @@ public final class LogSegment {
         timeIndex().resize(size);
     }
 
+    public void sanityCheck() throws IOException {
+        if (!lazyOffsetIndex.file().exists()) {
+            throw new NoSuchFileException(
+                    "Offset index file "
+                            + lazyOffsetIndex.file().getAbsolutePath()
+                            + " does not exist.");
+        }
+        lazyOffsetIndex.get().sanityCheck();
+
+        if (!lazyTimeIndex.file().exists()) {
+            throw new NoSuchFileException(
+                    "Time index file "
+                            + lazyTimeIndex.file().getAbsolutePath()
+                            + " does not exist.");
+        }
+        lazyTimeIndex.get().sanityCheck();
+    }
+
     /**
      * The maximum timestamp we see so far.
      *
@@ -284,7 +303,7 @@ public final class LogSegment {
      * Run recovery on the given segment. This will rebuild the index from the log file and lop off
      * any invalid bytes from the end of the log and index.
      */
-    public int recover() throws Exception {
+    public int recover() throws IOException {
         offsetIndex().reset();
         timeIndex().reset();
         int validBytes = 0;
