@@ -100,7 +100,7 @@ public final class LogTablet {
     private final Scheduler scheduler;
     private final ScheduledFuture<?> writerExpireCheck;
     private final LogFormat logFormat;
-    private final int tieredLogLocalSegments;
+    private volatile int tieredLogLocalSegments;
     private final Clock clock;
     private final boolean isChangeLog;
 
@@ -120,6 +120,8 @@ public final class LogTablet {
     // tracking the log size in remote storage
     private volatile long remoteLogSize = 0;
 
+    // tracking if the data lake enabled
+    private volatile boolean isDataLakeEnabled = false;
     // tracking the log start/end offset in lakehouse storage
     private volatile long lakeTableSnapshotId = -1;
     // note: currently, for primary key table, the log start offset nerve be updated
@@ -231,6 +233,10 @@ public final class LogTablet {
         return localLog.getTableBucket();
     }
 
+    public long getRowCount() {
+        return getHighWatermark() - logStartOffset();
+    }
+
     public long getHighWatermark() {
         return highWatermarkMetadata.getMessageOffset();
     }
@@ -239,6 +245,10 @@ public final class LogTablet {
         return localLog.getLocalLogEndOffsetMetadata();
     }
 
+    public boolean isDataLakeEnabled() {
+        return isDataLakeEnabled;
+    }
+
     public long getLakeTableSnapshotId() {
         return lakeTableSnapshotId;
     }
@@ -516,6 +526,18 @@ public final class LogTablet {
         }
     }
 
+    public void updateIsDataLakeEnabled(boolean isDataLakeEnabled) {
+        this.isDataLakeEnabled = isDataLakeEnabled;
+    }
+
+    public void updateTieredLogLocalSegments(int tieredLogLocalSegments) {
+        this.tieredLogLocalSegments = tieredLogLocalSegments;
+    }
+
+    public int getTieredLogLocalSegments() {
+        return tieredLogLocalSegments;
+    }
+
     public void updateLakeTableSnapshotId(long snapshotId) {
         if (snapshotId > this.lakeTableSnapshotId) {
             this.lakeTableSnapshotId = snapshotId;
@@ -1277,13 +1299,13 @@ public final class LogTablet {
         Map<Long, WriterAppendInfo> loadedWriters = new HashMap<>();
         for (LogRecordBatch batch : records.batches()) {
             if (batch.hasWriterId()) {
-                updateWriterAppendInfo(writerStateManager, batch, loadedWriters, true);
+                updateWriterAppendInfo(writerStateManager, batch, loadedWriters, false);
             }
         }
         loadedWriters.values().forEach(writerStateManager::update);
     }
 
-    private static void deleteWriterSnapshots(
+    public static void deleteWriterSnapshots(
             List<LogSegment> segments, WriterStateManager writerStateManager) throws IOException {
         for (LogSegment segment : segments) {
             writerStateManager.removeAndDeleteSnapshot(segment.getBaseOffset());
