@@ -27,11 +27,13 @@ import org.apache.fluss.metrics.NoOpCounter;
 import org.apache.fluss.metrics.ThreadSafeSimpleCounter;
 import org.apache.fluss.metrics.groups.AbstractMetricGroup;
 import org.apache.fluss.metrics.registry.MetricRegistry;
+import org.apache.fluss.server.kv.rocksdb.RocksDBStatistics;
+import org.apache.fluss.utils.MapUtils;
 
 import javax.annotation.Nullable;
 
-import java.util.HashMap;
 import java.util.Map;
+import java.util.stream.Stream;
 
 import static org.apache.fluss.metrics.utils.MetricGroupUtils.makeScope;
 
@@ -41,7 +43,7 @@ import static org.apache.fluss.metrics.utils.MetricGroupUtils.makeScope;
  */
 public class TableMetricGroup extends AbstractMetricGroup {
 
-    private final Map<TableBucket, BucketMetricGroup> buckets = new HashMap<>();
+    private final Map<TableBucket, BucketMetricGroup> buckets = MapUtils.newConcurrentHashMap();
 
     private final TablePath tablePath;
 
@@ -70,6 +72,8 @@ public class TableMetricGroup extends AbstractMetricGroup {
         if (isKvTable) {
             kvMetrics = new KvMetricGroup(this);
             logMetrics = new LogMetricGroup(this, TabletType.CDC_LOG);
+            // Register RocksDB aggregated metrics for kv tables
+            registerRocksDBMetrics();
         } else {
             // otherwise, create log produce metrics
             kvMetrics = null;
@@ -236,17 +240,156 @@ public class TableMetricGroup extends AbstractMetricGroup {
 
     public void removeBucketMetricGroup(TableBucket tableBucket) {
         BucketMetricGroup metricGroup = buckets.remove(tableBucket);
-        metricGroup.close();
+        if (metricGroup != null) {
+            // BucketMetricGroup.close() will automatically clean up RocksDB statistics
+            metricGroup.close();
+        }
     }
 
     public int bucketGroupsCount() {
         return buckets.size();
     }
 
+    public java.util.Collection<BucketMetricGroup> getBucketMetricGroups() {
+        return buckets.values();
+    }
+
+    /**
+     * Get all RocksDB statistics from bucket metric groups for table-level and server-level
+     * aggregation.
+     *
+     * <p>This method dynamically collects statistics from all buckets, allowing automatic cleanup
+     * when buckets are removed without maintaining a separate map.
+     *
+     * @return stream of RocksDB statistics from all buckets in this table
+     */
+    public Stream<RocksDBStatistics> allRocksDBStatistics() {
+        return buckets.values().stream()
+                .map(BucketMetricGroup::getRocksDBStatistics)
+                .filter(stats -> stats != null);
+    }
+
     public TabletServerMetricGroup getServerMetricGroup() {
         return (TabletServerMetricGroup) parent;
     }
 
+    /**
+     * Register RocksDB aggregated metrics at table level. These metrics aggregate values from all
+     * buckets of this table.
+     *
+     * <p>This method is called once during TableMetricGroup construction for KV tables.
+     */
+    private void registerRocksDBMetrics() {
+        // Max aggregation metrics - track the maximum value across all buckets
+        gauge(
+                MetricNames.ROCKSDB_WRITE_STALL_MICROS_MAX,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getWriteStallMicros)
+                                .max()
+                                .orElse(0L));
+        gauge(
+                MetricNames.ROCKSDB_GET_LATENCY_MICROS_MAX,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getGetLatencyMicros)
+                                .max()
+                                .orElse(0L));
+        gauge(
+                MetricNames.ROCKSDB_WRITE_LATENCY_MICROS_MAX,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getWriteLatencyMicros)
+                                .max()
+                                .orElse(0L));
+        gauge(
+                MetricNames.ROCKSDB_NUM_FILES_AT_LEVEL0_MAX,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getNumFilesAtLevel0)
+                                .max()
+                                .orElse(0L));
+        gauge(
+                MetricNames.ROCKSDB_FLUSH_PENDING_MAX,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getFlushPending)
+                                .max()
+                                .orElse(0L));
+        gauge(
+                MetricNames.ROCKSDB_COMPACTION_PENDING_MAX,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getCompactionPending)
+                                .max()
+                                .orElse(0L));
+        gauge(
+                MetricNames.ROCKSDB_COMPACTION_TIME_MICROS_MAX,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getCompactionTimeMicros)
+                                .max()
+                                .orElse(0L));
+
+        // Sum aggregation metrics - track the total value across all buckets
+        gauge(
+                MetricNames.ROCKSDB_BYTES_READ_TOTAL,
+                () -> allRocksDBStatistics().mapToLong(RocksDBStatistics::getBytesRead).sum());
+        gauge(
+                MetricNames.ROCKSDB_BYTES_WRITTEN_TOTAL,
+                () -> allRocksDBStatistics().mapToLong(RocksDBStatistics::getBytesWritten).sum());
+        gauge(
+                MetricNames.ROCKSDB_FLUSH_BYTES_WRITTEN_TOTAL,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getFlushBytesWritten)
+                                .sum());
+        gauge(
+                MetricNames.ROCKSDB_COMPACTION_BYTES_READ_TOTAL,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getCompactionBytesRead)
+                                .sum());
+        gauge(
+                MetricNames.ROCKSDB_COMPACTION_BYTES_WRITTEN_TOTAL,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getCompactionBytesWritten)
+                                .sum());
+
+        // Fine-grained memory metrics - track memory usage by component type
+        gauge(
+                MetricNames.ROCKSDB_MEMTABLE_MEMORY_USAGE_TOTAL,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getMemTableMemoryUsage)
+                                .sum());
+        gauge(
+                MetricNames.ROCKSDB_MEMTABLE_UNFLUSHED_MEMORY_USAGE_TOTAL,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getMemTableUnFlushedMemoryUsage)
+                                .sum());
+        gauge(
+                MetricNames.ROCKSDB_TABLE_READERS_MEMORY_USAGE_TOTAL,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getTableReadersMemoryUsage)
+                                .sum());
+        gauge(
+                MetricNames.ROCKSDB_BLOCK_CACHE_MEMORY_USAGE_TOTAL,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getBlockCacheMemoryUsage)
+                                .sum());
+        gauge(
+                MetricNames.ROCKSDB_BLOCK_CACHE_PINNED_USAGE_TOTAL,
+                () ->
+                        allRocksDBStatistics()
+                                .mapToLong(RocksDBStatistics::getBlockCachePinnedUsage)
+                                .sum());
+    }
+
     /** Metric group for specific kind of tablet of a table. */
     private static class TabletMetricGroup extends AbstractMetricGroup {
         private final TabletType tabletType;
