@@ -32,6 +32,8 @@ import org.apache.fluss.rpc.entity.ResultForBucket;
 import org.apache.fluss.rpc.gateway.TabletServerGateway;
 import org.apache.fluss.rpc.messages.FetchLogRequest;
 import org.apache.fluss.rpc.messages.FetchLogResponse;
+import org.apache.fluss.rpc.messages.GetTableStatsRequest;
+import org.apache.fluss.rpc.messages.GetTableStatsResponse;
 import org.apache.fluss.rpc.messages.InitWriterRequest;
 import org.apache.fluss.rpc.messages.InitWriterResponse;
 import org.apache.fluss.rpc.messages.LimitScanRequest;
@@ -62,6 +64,7 @@ import org.apache.fluss.rpc.messages.UpdateMetadataRequest;
 import org.apache.fluss.rpc.messages.UpdateMetadataResponse;
 import org.apache.fluss.rpc.protocol.ApiError;
 import org.apache.fluss.rpc.protocol.Errors;
+import org.apache.fluss.rpc.protocol.MergeMode;
 import org.apache.fluss.security.acl.OperationType;
 import org.apache.fluss.security.acl.Resource;
 import org.apache.fluss.server.DynamicConfigManager;
@@ -70,6 +73,7 @@ import org.apache.fluss.server.authorizer.Authorizer;
 import org.apache.fluss.server.coordinator.MetadataManager;
 import org.apache.fluss.server.entity.FetchReqInfo;
 import org.apache.fluss.server.entity.NotifyLeaderAndIsrData;
+import org.apache.fluss.server.entity.UserContext;
 import org.apache.fluss.server.log.FetchParams;
 import org.apache.fluss.server.log.ListOffsetsParam;
 import org.apache.fluss.server.metadata.TabletServerMetadataCache;
@@ -87,6 +91,7 @@ import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.ExecutorService;
 import java.util.function.BiFunction;
 import java.util.stream.Collectors;
 
@@ -103,9 +108,11 @@ import static org.apache.fluss.server.utils.ServerRpcMessageUtils.getNotifySnaps
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.getProduceLogData;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.getPutKvData;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.getStopReplicaData;
+import static org.apache.fluss.server.utils.ServerRpcMessageUtils.getTableStatsRequestData;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.getTargetColumns;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.getUpdateMetadataRequestData;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.makeFetchLogResponse;
+import static org.apache.fluss.server.utils.ServerRpcMessageUtils.makeGetTableStatsResponse;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.makeInitWriterResponse;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.makeLimitScanResponse;
 import static org.apache.fluss.server.utils.ServerRpcMessageUtils.makeListOffsetsResponse;
@@ -134,14 +141,16 @@ public final class TabletService extends RpcServiceBase implements TabletServerG
             TabletServerMetadataCache metadataCache,
             MetadataManager metadataManager,
             @Nullable Authorizer authorizer,
-            DynamicConfigManager dynamicConfigManager) {
+            DynamicConfigManager dynamicConfigManager,
+            ExecutorService ioExecutor) {
         super(
                 remoteFileSystem,
                 ServerType.TABLET_SERVER,
                 zkClient,
                 metadataManager,
                 authorizer,
-                dynamicConfigManager);
+                dynamicConfigManager,
+                ioExecutor);
         this.serviceName = "server-" + serverId;
         this.replicaManager = replicaManager;
         this.metadataCache = metadataCache;
@@ -166,6 +175,7 @@ public final class TabletService extends RpcServiceBase implements TabletServerG
                 request.getTimeoutMs(),
                 request.getAcks(),
                 produceLogData,
+                new UserContext(currentSession().getPrincipal()),
                 bucketResponseMap -> response.complete(makeProduceLogResponse(bucketResponseMap)));
         return response;
     }
@@ -190,6 +200,7 @@ public final class TabletService extends RpcServiceBase implements TabletServerG
         replicaManager.fetchLogRecords(
                 fetchParams,
                 interesting,
+                new UserContext(currentSession().getPrincipal()),
                 fetchResponseMap ->
                         response.complete(
                                 makeFetchLogResponse(fetchResponseMap, errorResponseMap)));
@@ -218,12 +229,19 @@ public final class TabletService extends RpcServiceBase implements TabletServerG
         authorizeTable(WRITE, request.getTableId());
 
         Map<TableBucket, KvRecordBatch> putKvData = getPutKvData(request);
+        // Get mergeMode from request, default to DEFAULT if not set
+        MergeMode mergeMode =
+                request.hasAggMode()
+                        ? MergeMode.fromValue(request.getAggMode())
+                        : MergeMode.DEFAULT;
         CompletableFuture<PutKvResponse> response = new CompletableFuture<>();
         replicaManager.putRecordsToKv(
                 request.getTimeoutMs(),
                 request.getAcks(),
                 putKvData,
                 getTargetColumns(request),
+                mergeMode,
+                currentSession().getApiVersion(),
                 bucketResponse -> response.complete(makePutKvResponse(bucketResponse)));
         return response;
     }
@@ -232,17 +250,29 @@ public final class TabletService extends RpcServiceBase implements TabletServerG
     public CompletableFuture<LookupResponse> lookup(LookupRequest request) {
         Map<TableBucket, List<byte[]>> lookupData = toLookupData(request);
         Map<TableBucket, LookupResultForBucket> errorResponseMap = new HashMap<>();
-        Map<TableBucket, List<byte[]>> interesting =
-                authorizeRequestData(
-                        READ, lookupData, errorResponseMap, LookupResultForBucket::new);
-        if (interesting.isEmpty()) {
-            return CompletableFuture.completedFuture(makeLookupResponse(errorResponseMap));
-        }
-
         CompletableFuture<LookupResponse> response = new CompletableFuture<>();
-        replicaManager.lookups(
-                lookupData,
-                value -> response.complete(makeLookupResponse(value, errorResponseMap)));
+
+        if (request.hasInsertIfNotExists() && request.isInsertIfNotExists()) {
+            authorizeTable(WRITE, request.getTableId());
+            replicaManager.lookups(
+                    request.isInsertIfNotExists(),
+                    request.getTimeoutMs(),
+                    request.getAcks(),
+                    lookupData,
+                    currentSession().getApiVersion(),
+                    value -> response.complete(makeLookupResponse(value, errorResponseMap)));
+        } else {
+            Map<TableBucket, List<byte[]>> interesting =
+                    authorizeRequestData(
+                            READ, lookupData, errorResponseMap, LookupResultForBucket::new);
+            if (interesting.isEmpty()) {
+                return CompletableFuture.completedFuture(makeLookupResponse(errorResponseMap));
+            }
+            replicaManager.lookups(
+                    lookupData,
+                    currentSession().getApiVersion(),
+                    value -> response.complete(makeLookupResponse(value, errorResponseMap)));
+        }
         return response;
     }
 
@@ -260,6 +290,7 @@ public final class TabletService extends RpcServiceBase implements TabletServerG
         CompletableFuture<PrefixLookupResponse> response = new CompletableFuture<>();
         replicaManager.prefixLookups(
                 prefixLookupData,
+                currentSession().getApiVersion(),
                 value -> response.complete(makePrefixLookupResponse(value, errorResponseMap)));
         return response;
     }
@@ -279,6 +310,17 @@ public final class TabletService extends RpcServiceBase implements TabletServerG
         return response;
     }
 
+    @Override
+    public CompletableFuture<GetTableStatsResponse> getTableStats(GetTableStatsRequest request) {
+        authorizeTable(READ, request.getTableId());
+
+        CompletableFuture<GetTableStatsResponse> response = new CompletableFuture<>();
+        replicaManager.getTableStats(
+                getTableStatsRequestData(request),
+                result -> response.complete(makeGetTableStatsResponse(result)));
+        return response;
+    }
+
     @Override
     public CompletableFuture<NotifyLeaderAndIsrResponse> notifyLeaderAndIsr(
             NotifyLeaderAndIsrRequest notifyLeaderAndIsrRequest) {
@@ -380,7 +422,8 @@ public final class TabletService extends RpcServiceBase implements TabletServerG
         return response;
     }
 
-    private void authorizeTable(OperationType operationType, long tableId) {
+    @Override
+    public void authorizeTable(OperationType operationType, long tableId) {
         if (authorizer != null) {
             TablePath tablePath = metadataCache.getTablePath(tableId).orElse(null);
             if (tablePath == null) {
@@ -390,15 +433,7 @@ public final class TabletService extends RpcServiceBase implements TabletServerG
                                         + "metadata cache in the server is not updated yet.",
                                 serviceName, tableId));
             }
-            if (!authorizer.isAuthorized(
-                    currentSession(), operationType, Resource.table(tablePath))) {
-                throw new AuthorizationException(
-                        String.format(
-                                "No permission to %s table %s in database %s",
-                                operationType,
-                                tablePath.getTableName(),
-                                tablePath.getDatabaseName()));
-            }
+            authorizeTable(operationType, tablePath);
         }
     }
 
