@@ -24,7 +24,14 @@ import org.apache.fluss.metadata.TablePartition;
 import org.apache.fluss.metadata.TablePath;
 import org.apache.fluss.security.acl.Resource;
 import org.apache.fluss.security.acl.ResourceType;
+import org.apache.fluss.server.zk.data.lake.LakeTable;
+import org.apache.fluss.server.zk.data.lake.LakeTableJsonSerde;
+import org.apache.fluss.server.zk.data.lease.KvSnapshotLeaseMetadata;
+import org.apache.fluss.server.zk.data.lease.KvSnapshotLeaseMetadataJsonSerde;
+import org.apache.fluss.server.zk.data.producer.ProducerOffsets;
+import org.apache.fluss.server.zk.data.producer.ProducerOffsetsJsonSerde;
 import org.apache.fluss.utils.json.JsonSerdeUtils;
+import org.apache.fluss.utils.types.Tuple2;
 
 import javax.annotation.Nullable;
 
@@ -130,6 +137,7 @@ public final class ZkData {
      * <p>/metadata/databases/[databaseName]/tables/[tableName]/schemas
      */
     public static final class SchemasZNode {
+
         public static String path(TablePath tablePath) {
             return TableZNode.path(tablePath) + "/schemas";
         }
@@ -142,6 +150,32 @@ public final class ZkData {
      * <p>/metadata/databases/[databaseName]/tables/[tableName]/schemas/[schemaId]
      */
     public static final class SchemaZNode {
+
+        @Nullable
+        public static Tuple2<TablePath, Integer> parsePath(String zkPath) {
+            String splitter = "/schemas/";
+            if (!zkPath.contains(splitter)) {
+                return null;
+            }
+
+            String[] split = zkPath.split(splitter);
+            if (split.length != 2) {
+                return null;
+            }
+            TablePath tablePath = TableZNode.parsePath(split[0]);
+            if (tablePath == null) {
+                return null;
+            }
+
+            int schemaId;
+            try {
+                schemaId = Integer.parseInt(split[1]);
+                return Tuple2.of(tablePath, schemaId);
+            } catch (NumberFormatException e) {
+                return null;
+            }
+        }
+
         public static String path(TablePath tablePath, int schemaId) {
             return SchemasZNode.path(tablePath) + "/" + schemaId;
         }
@@ -222,6 +256,28 @@ public final class ZkData {
         }
     }
 
+    /**
+     * The znode for auto increment columns of a table. The znode path is:
+     *
+     * <p>/metadata/databases/[databaseName]/tables/[tableName]/auto_inc
+     */
+    public static final class AutoIncrementColumnsZNode {
+        public static String path(TablePath tablePath) {
+            return TableZNode.path(tablePath) + "/auto_inc";
+        }
+    }
+
+    /**
+     * The znode for auto increment column. The znode path is:
+     *
+     * <p>/metadata/databases/[databaseName]/tables/[tableName]/auto_inc/col_[columnId]
+     */
+    public static final class AutoIncrementColumnZNode {
+        public static String path(TablePath tablePath, int columnId) {
+            return AutoIncrementColumnsZNode.path(tablePath) + String.format("/col_%d", columnId);
+        }
+    }
+
     /**
      * The znode used to generate a sequence unique id for a partition. The znode path is:
      *
@@ -536,22 +592,69 @@ public final class ZkData {
     }
 
     /**
-     * The znode for the info of lake data for a table. The znode path is:
+     * The znode for lake table snapshot information. The znode path is:
      *
      * <p>/tabletservers/tables/[tableId]/laketable
+     *
+     * <p>This znode stores {@link LakeTable} in:
+     *
+     * <ul>
+     *   <li>Version 1 (legacy): Full snapshot data stored directly in ZK
+     *   <li>Version 2 (current): A list of snapshot metadata, with metadata file path stored in ZK,
+     *       actual data in remote file
+     * </ul>
      */
     public static final class LakeTableZNode {
+        /**
+         * Returns the ZK path for the lake table znode of the given table.
+         *
+         * @param tableId the table ID
+         * @return the ZK path
+         */
         public static String path(long tableId) {
             return TableIdZNode.path(tableId) + "/laketable";
         }
 
-        public static byte[] encode(LakeTableSnapshot lakeTableSnapshot) {
-            return JsonSerdeUtils.writeValueAsBytes(
-                    lakeTableSnapshot, LakeTableSnapshotJsonSerde.INSTANCE);
+        /**
+         * Encodes a LakeTable to JSON bytes for storage in ZK.
+         *
+         * @param lakeTable the LakeTable to encode
+         * @return the encoded bytes
+         */
+        public static byte[] encode(LakeTable lakeTable) {
+            return JsonSerdeUtils.writeValueAsBytes(lakeTable, LakeTableJsonSerde.INSTANCE);
         }
 
-        public static LakeTableSnapshot decode(byte[] json) {
-            return JsonSerdeUtils.readValue(json, LakeTableSnapshotJsonSerde.INSTANCE);
+        /**
+         * Decodes JSON bytes from ZK to a LakeTable.
+         *
+         * <p>This method handles both version 1 (legacy) and version 2 (current) formats
+         * automatically through {@link LakeTableJsonSerde}.
+         *
+         * @param json the JSON bytes from ZK
+         * @return the decoded LakeTable
+         */
+        public static LakeTable decode(byte[] json) {
+            return JsonSerdeUtils.readValue(json, LakeTableJsonSerde.INSTANCE);
+        }
+    }
+
+    /**
+     * The znode for server tags. The znode path is:
+     *
+     * <p>/tabletServers/server_tags
+     */
+    public static final class ServerTagsZNode {
+        public static String path() {
+            return "/tabletservers/server_tags";
+        }
+
+        public static byte[] encode(ServerTags serverTag) {
+            return JsonSerdeUtils.writeValueAsBytes(serverTag, ServerTagsJsonSerde.INSTANCE);
+        }
+
+        public static ServerTags decode(byte[] json) {
+            return JsonSerdeUtils.readValue(json, ServerTagsJsonSerde.INSTANCE);
         }
     }
 
@@ -718,4 +821,125 @@ public final class ZkData {
             return new byte[0];
         }
     }
+
+    // ------------------------------------------------------------------------------------------
+    // ZNodes under "/cluster/"
+    // ------------------------------------------------------------------------------------------
+
+    /**
+     * The znode for rebalance. The znode path is:
+     *
+     * <p>/cluster/rebalance
+     */
+    public static final class RebalanceZNode {
+        public static String path() {
+            return "/cluster/rebalance";
+        }
+
+        public static byte[] encode(RebalanceTask rebalanceTask) {
+            return JsonSerdeUtils.writeValueAsBytes(rebalanceTask, RebalanceTaskJsonSerde.INSTANCE);
+        }
+
+        public static RebalanceTask decode(byte[] json) {
+            return JsonSerdeUtils.readValue(json, RebalanceTaskJsonSerde.INSTANCE);
+        }
+    }
+
+    // ------------------------------------------------------------------------------------------
+    // ZNodes under "/producers/"
+    // ------------------------------------------------------------------------------------------
+
+    /**
+     * The znode for producers. This is the root node for all producer offset snapshots. The znode
+     * path is:
+     *
+     * <p>/producers
+     */
+    public static final class ProducersZNode {
+        public static String path() {
+            return "/producers";
+        }
+    }
+
+    /**
+     * The znode for a specific producer's offset snapshot. The znode path is:
+     *
+     * <p>/producers/[producerId]
+     *
+     * <p>This znode stores {@link ProducerOffsets} which contains:
+     *
+     * <ul>
+     *   <li>expiration_time: TTL for automatic cleanup
+     *   <li>tables: List of table offset metadata with paths to remote offset files
+     * </ul>
+     *
+     * <p>The actual offset data is stored in remote storage (e.g., OSS/S3) and referenced by the
+     * file paths in the metadata.
+     */
+    public static final class ProducerIdZNode {
+        /**
+         * Returns the ZK path for the producer snapshot znode.
+         *
+         * @param producerId the producer ID (typically Flink job ID)
+         * @return the ZK path
+         */
+        public static String path(String producerId) {
+            return ProducersZNode.path() + "/" + producerId;
+        }
+
+        /**
+         * Encodes a ProducerOffsets to JSON bytes for storage in ZK.
+         *
+         * @param producerOffsets the ProducerOffsets to encode
+         * @return the encoded bytes
+         */
+        public static byte[] encode(ProducerOffsets producerOffsets) {
+            return JsonSerdeUtils.writeValueAsBytes(
+                    producerOffsets, ProducerOffsetsJsonSerde.INSTANCE);
+        }
+
+        /**
+         * Decodes JSON bytes from ZK to a ProducerOffsets.
+         *
+         * @param json the JSON bytes from ZK
+         * @return the decoded ProducerOffsets
+         */
+        public static ProducerOffsets decode(byte[] json) {
+            return JsonSerdeUtils.readValue(json, ProducerOffsetsJsonSerde.INSTANCE);
+        }
+    }
+
+    // ------------------------------------------------------------------------------------------
+    // ZNodes for Consumers.
+    // ------------------------------------------------------------------------------------------
+
+    /** The root znode for leases. It will record all the info of fluss leases. */
+    public static final class LeasesNode {
+        public static String path() {
+            return "/leases";
+        }
+    }
+
+    /** The root znode for kv snapshot leases. */
+    public static final class KvSnapshotLeasesZNode {
+        public static String path() {
+            return LeasesNode.path() + "/kv_snapshot";
+        }
+    }
+
+    /** The znode for kv snapshot lease zk data. */
+    public static final class KvSnapshotLeaseZNode {
+        public static String path(String leaseId) {
+            return KvSnapshotLeasesZNode.path() + "/" + leaseId;
+        }
+
+        public static byte[] encode(KvSnapshotLeaseMetadata kvSnapshotLeaseMetadata) {
+            return JsonSerdeUtils.writeValueAsBytes(
+                    kvSnapshotLeaseMetadata, KvSnapshotLeaseMetadataJsonSerde.INSTANCE);
+        }
+
+        public static KvSnapshotLeaseMetadata decode(byte[] json) {
+            return JsonSerdeUtils.readValue(json, KvSnapshotLeaseMetadataJsonSerde.INSTANCE);
+        }
+    }
 }
