@@ -480,7 +480,8 @@ class KvTabletSnapshotTargetTest {
                         sharedKvFileRegistry,
                         Collections.emptyList(),
                         snapshotHandleStore,
-                        executor);
+                        executor,
+                        (consumeKvSnapshotForBucket) -> false); //  only retain the latest snapshot.
 
         RocksIncrementalSnapshot rocksIncrementalSnapshot =
                 createIncrementalSnapshot(snapshotFailType);
@@ -500,7 +501,7 @@ class KvTabletSnapshotTargetTest {
                 executor,
                 cancelStreamRegistry,
                 testingSnapshotIdCounter,
-                logOffsetGenerator::get,
+                this::getCurrentTabletState,
                 updateMinRetainOffsetConsumer::set,
                 bucketLeaderEpochSupplier,
                 coordinatorEpochSupplier,
@@ -538,7 +539,7 @@ class KvTabletSnapshotTargetTest {
                 executor,
                 cancelStreamRegistry,
                 testingSnapshotIdCounter,
-                logOffsetGenerator::get,
+                this::getCurrentTabletState,
                 updateMinRetainOffsetConsumer::set,
                 bucketLeaderEpochSupplier,
                 coordinatorEpochSupplier,
@@ -546,6 +547,10 @@ class KvTabletSnapshotTargetTest {
                 0L);
     }
 
+    private TabletState getCurrentTabletState() {
+        return new TabletState(logOffsetGenerator.get(), null, null);
+    }
+
     private RocksIncrementalSnapshot createIncrementalSnapshot(SnapshotFailType snapshotFailType)
             throws IOException {
         long lastCompletedSnapshotId = -1L;
@@ -625,10 +630,11 @@ class KvTabletSnapshotTargetTest {
             this.snapshotFailType = snapshotFailType;
         }
 
+        @Override
         public SnapshotResultSupplier asyncSnapshot(
                 NativeRocksDBSnapshotResources snapshotResources,
                 long snapshotId,
-                long logOffset,
+                TabletState tabletState,
                 @Nonnull SnapshotLocation snapshotLocation) {
             if (snapshotFailType == SnapshotFailType.SYNC_PHASE) {
                 throw new FlussRuntimeException("Fail in snapshot sync phase.");
@@ -638,7 +644,7 @@ class KvTabletSnapshotTargetTest {
                 };
             } else {
                 return super.asyncSnapshot(
-                        snapshotResources, snapshotId, logOffset, snapshotLocation);
+                        snapshotResources, snapshotId, tabletState, snapshotLocation);
             }
         }
 
@@ -655,10 +661,20 @@ class KvTabletSnapshotTargetTest {
 
     private class TestingSnapshotIDCounter implements SequenceIDCounter {
 
+        @Override
+        public long getCurrent() {
+            return snapshotIdGenerator.get();
+        }
+
         @Override
         public long getAndIncrement() {
             return snapshotIdGenerator.getAndIncrement();
         }
+
+        @Override
+        public long getAndAdd(Long delta) {
+            return snapshotIdGenerator.getAndAdd(delta);
+        }
     }
 
     private enum SnapshotFailType {
