@@ -21,6 +21,8 @@ import org.apache.fluss.config.ConfigOptions;
 import org.apache.fluss.exception.OutOfOrderSequenceException;
 import org.apache.fluss.metadata.LogFormat;
 import org.apache.fluss.metadata.PhysicalTablePath;
+import org.apache.fluss.metadata.SchemaGetter;
+import org.apache.fluss.metadata.SchemaInfo;
 import org.apache.fluss.metadata.TableBucket;
 import org.apache.fluss.metadata.TablePath;
 import org.apache.fluss.record.ChangeType;
@@ -29,6 +31,8 @@ import org.apache.fluss.record.KvRecordTestUtils;
 import org.apache.fluss.record.LogRecordBatch;
 import org.apache.fluss.record.LogRecords;
 import org.apache.fluss.record.MemoryLogRecords;
+import org.apache.fluss.record.ProjectionPushdownCache;
+import org.apache.fluss.rpc.protocol.MergeMode;
 import org.apache.fluss.server.entity.NotifyLeaderAndIsrData;
 import org.apache.fluss.server.kv.KvTablet;
 import org.apache.fluss.server.kv.snapshot.CompletedSnapshot;
@@ -41,6 +45,7 @@ import org.apache.fluss.server.zk.NOPErrorHandler;
 import org.apache.fluss.server.zk.data.LeaderAndIsr;
 import org.apache.fluss.testutils.DataTestUtils;
 import org.apache.fluss.testutils.common.ManuallyTriggeredScheduledExecutorService;
+import org.apache.fluss.types.RowType;
 import org.apache.fluss.utils.types.Tuple2;
 
 import org.junit.jupiter.api.Test;
@@ -70,11 +75,15 @@ import static org.apache.fluss.record.TestData.DATA1_TABLE_ID;
 import static org.apache.fluss.record.TestData.DATA1_TABLE_ID_PK;
 import static org.apache.fluss.record.TestData.DATA1_TABLE_PATH;
 import static org.apache.fluss.record.TestData.DATA1_TABLE_PATH_PK;
+import static org.apache.fluss.record.TestData.DATA2;
+import static org.apache.fluss.record.TestData.DATA2_ROW_TYPE;
+import static org.apache.fluss.record.TestData.DATA2_SCHEMA;
 import static org.apache.fluss.record.TestData.DEFAULT_SCHEMA_ID;
 import static org.apache.fluss.server.coordinator.CoordinatorContext.INITIAL_COORDINATOR_EPOCH;
 import static org.apache.fluss.server.zk.data.LeaderAndIsr.INITIAL_LEADER_EPOCH;
 import static org.apache.fluss.testutils.DataTestUtils.assertLogRecordsEquals;
 import static org.apache.fluss.testutils.DataTestUtils.createBasicMemoryLogRecords;
+import static org.apache.fluss.testutils.DataTestUtils.createRecordsWithoutBaseLogOffset;
 import static org.apache.fluss.testutils.DataTestUtils.genKvRecordBatch;
 import static org.apache.fluss.testutils.DataTestUtils.genKvRecords;
 import static org.apache.fluss.testutils.DataTestUtils.genMemoryLogRecordsByObject;
@@ -116,6 +125,8 @@ final class ReplicaTest extends ReplicaTestBase {
     void testAppendRecordsToLeader() throws Exception {
         Replica logReplica =
                 makeLogReplica(DATA1_PHYSICAL_TABLE_PATH, new TableBucket(DATA1_TABLE_ID, 1));
+        SchemaGetter schemaGetter = logReplica.getSchemaGetter();
+
         makeLogReplicaAsLeader(logReplica);
 
         MemoryLogRecords mr = genMemoryLogRecordsByObject(DATA1);
@@ -128,10 +139,48 @@ final class ReplicaTest extends ReplicaTestBase {
                         (int)
                                 conf.get(ConfigOptions.CLIENT_SCANNER_LOG_FETCH_MAX_BYTES)
                                         .getBytes());
+        ProjectionPushdownCache projectionCache = new ProjectionPushdownCache();
         fetchParams.setCurrentFetch(
-                DATA1_TABLE_ID, 0, Integer.MAX_VALUE, DATA1_ROW_TYPE, DEFAULT_COMPRESSION, null);
+                DATA1_TABLE_ID,
+                0,
+                Integer.MAX_VALUE,
+                schemaGetter,
+                DEFAULT_COMPRESSION,
+                null,
+                projectionCache);
         LogReadInfo logReadInfo = logReplica.fetchRecords(fetchParams);
-        assertLogRecordsEquals(DATA1_ROW_TYPE, logReadInfo.getFetchedData().getRecords(), DATA1);
+        assertLogRecordsEquals(
+                DATA1_ROW_TYPE, logReadInfo.getFetchedData().getRecords(), DATA1, schemaGetter);
+
+        // schema evolution.
+        serverMetadataCache.updateLatestSchema(
+                DATA1_TABLE_ID, new SchemaInfo(DATA2_SCHEMA, (short) 2));
+        mr =
+                createRecordsWithoutBaseLogOffset(
+                        DATA2_ROW_TYPE,
+                        2,
+                        0,
+                        System.currentTimeMillis(),
+                        CURRENT_LOG_MAGIC_VALUE,
+                        DATA2,
+                        LogFormat.ARROW);
+        appendInfo = logReplica.appendRecordsToLeader(mr, 0);
+        assertThat(appendInfo.shallowCount()).isEqualTo(1);
+        fetchParams.setCurrentFetch(
+                DATA1_TABLE_ID,
+                appendInfo.firstOffset(),
+                Integer.MAX_VALUE,
+                schemaGetter,
+                DEFAULT_COMPRESSION,
+                null,
+                projectionCache);
+        logReadInfo = logReplica.fetchRecords(fetchParams);
+        assertLogRecordsEquals(
+                2, DATA2_ROW_TYPE, logReadInfo.getFetchedData().getRecords(), DATA2, schemaGetter);
+
+        // read with old schema id.
+        assertLogRecordsEquals(
+                DATA1_ROW_TYPE, logReadInfo.getFetchedData().getRecords(), DATA2, schemaGetter);
     }
 
     @Test
@@ -222,6 +271,7 @@ final class ReplicaTest extends ReplicaTestBase {
 
         assertThatLogRecords(fetchRecords(kvReplica, 4))
                 .withSchema(DATA1_ROW_TYPE)
+                .withSchemaGetter(kvReplica.getSchemaGetter())
                 .isEqualTo(expected);
     }
 
@@ -229,6 +279,7 @@ final class ReplicaTest extends ReplicaTestBase {
     void testPutRecordsToLeader() throws Exception {
         Replica kvReplica =
                 makeKvReplica(DATA1_PHYSICAL_TABLE_PATH_PK, new TableBucket(DATA1_TABLE_ID_PK, 1));
+        SchemaGetter schemaGetter = kvReplica.getSchemaGetter();
         makeKvReplicaAsLeader(kvReplica);
 
         // two records in a batch with same key, should also generate +I/-U/+U
@@ -290,6 +341,7 @@ final class ReplicaTest extends ReplicaTestBase {
                                 new Object[] {5, "b4"}));
         assertThatLogRecords(fetchRecords(kvReplica, currentOffset))
                 .withSchema(DATA1_ROW_TYPE)
+                .withSchemaGetter(schemaGetter)
                 .isEqualTo(expected);
         currentOffset += 5;
 
@@ -315,6 +367,7 @@ final class ReplicaTest extends ReplicaTestBase {
                                 new Object[] {6, "b4"}));
         assertThatLogRecords(fetchRecords(kvReplica, currentOffset))
                 .withSchema(DATA1_ROW_TYPE)
+                .withSchemaGetter(schemaGetter)
                 .isEqualTo(expected);
         currentOffset += 3;
 
@@ -343,6 +396,35 @@ final class ReplicaTest extends ReplicaTestBase {
                         Arrays.asList(new Object[] {1, "a1"}, new Object[] {1, "aaa"}));
         assertThatLogRecords(fetchRecords(kvReplica, currentOffset))
                 .withSchema(DATA1_ROW_TYPE)
+                .withSchemaGetter(schemaGetter)
+                .isEqualTo(expected);
+
+        // test schema change
+        currentOffset += 2;
+        short newSchemaId = 2;
+        serverMetadataCache.updateLatestSchema(
+                DATA1_TABLE_ID, new SchemaInfo(DATA2_SCHEMA, newSchemaId));
+        KvRecordTestUtils.KvRecordBatchFactory kvRecordBatchFactory2 =
+                KvRecordTestUtils.KvRecordBatchFactory.of(newSchemaId);
+        KvRecordTestUtils.KvRecordFactory kvRecordFactory2 =
+                KvRecordTestUtils.KvRecordFactory.of(DATA2_ROW_TYPE);
+        kvRecords =
+                kvRecordBatchFactory2.ofRecords(
+                        kvRecordFactory2.ofRecord("k1", null),
+                        kvRecordFactory2.ofRecord("k1", new Object[] {1, "aaaa", "bbb"}));
+        logAppendInfo = putRecordsToLeader(kvReplica, kvRecords);
+        assertThat(logAppendInfo.lastOffset()).isEqualTo(16);
+        expected =
+                logRecords(
+                        (short) 2,
+                        DATA2_ROW_TYPE,
+                        currentOffset,
+                        Arrays.asList(ChangeType.DELETE, ChangeType.INSERT),
+                        Arrays.asList(
+                                new Object[] {1, "aaa", null}, new Object[] {1, "aaaa", "bbb"}));
+        assertThatLogRecords(fetchRecords(kvReplica, currentOffset))
+                .withSchema(DATA2_ROW_TYPE)
+                .withSchemaGetter(schemaGetter)
                 .isEqualTo(expected);
     }
 
@@ -369,8 +451,8 @@ final class ReplicaTest extends ReplicaTestBase {
                         Tuple2.of("k2", new Object[] {3, "b1"}));
         putRecordsToLeader(kvReplica, kvRecords);
 
-        // trigger one snapshot,
-        scheduledExecutorService.triggerNonPeriodicScheduledTask();
+        // trigger one snapshot (task has been scheduled after becoming leader)
+        scheduledExecutorService.triggerAllNonPeriodicTasks();
 
         // wait until the snapshot 0 success
         CompletedSnapshot completedSnapshot0 =
@@ -391,7 +473,7 @@ final class ReplicaTest extends ReplicaTestBase {
         putRecordsToLeader(kvReplica, kvRecords);
 
         // trigger next checkpoint
-        scheduledExecutorService.triggerNonPeriodicScheduledTask();
+        scheduledExecutorService.triggerNextNonPeriodicScheduledTask(Duration.ofSeconds(30));
         // wait until the snapshot 1 success
         CompletedSnapshot completedSnapshot1 =
                 kvSnapshotStore.waitUntilSnapshotComplete(tableBucket, 1);
@@ -431,8 +513,8 @@ final class ReplicaTest extends ReplicaTestBase {
                         Tuple2.of("k3", new Object[] {5, "k3"}));
         putRecordsToLeader(kvReplica, kvRecords);
 
-        // trigger another one snapshot,
-        scheduledExecutorService.triggerNonPeriodicScheduledTask();
+        // trigger another one snapshot (task has been scheduled after becoming leader)
+        scheduledExecutorService.triggerAllNonPeriodicTasks();
         //  wait until the snapshot 2 success
         CompletedSnapshot completedSnapshot2 =
                 kvSnapshotStore.waitUntilSnapshotComplete(tableBucket, 2);
@@ -502,8 +584,8 @@ final class ReplicaTest extends ReplicaTestBase {
                         Tuple2.of("k2", new Object[] {2, "b"}));
         putRecordsToLeader(kvReplica, kvRecords);
 
-        // trigger first snapshot
-        scheduledExecutorService.triggerNonPeriodicScheduledTask();
+        // trigger first snapshot (task has been scheduled after becoming leader)
+        scheduledExecutorService.triggerAllNonPeriodicTasks();
         kvSnapshotStore.waitUntilSnapshotComplete(tableBucket, 0);
 
         // put more data and create second snapshot
@@ -513,8 +595,8 @@ final class ReplicaTest extends ReplicaTestBase {
                         Tuple2.of("k3", new Object[] {4, "d"}));
         putRecordsToLeader(kvReplica, kvRecords);
 
-        // trigger second snapshot
-        scheduledExecutorService.triggerNonPeriodicScheduledTask();
+        // trigger second snapshot (may need to wait the task being scheduled)
+        scheduledExecutorService.triggerNextNonPeriodicScheduledTask(Duration.ofSeconds(30));
         kvSnapshotStore.waitUntilSnapshotComplete(tableBucket, 1);
 
         // put more data and create third snapshot (this will be the broken one)
@@ -524,8 +606,8 @@ final class ReplicaTest extends ReplicaTestBase {
                         Tuple2.of("k5", new Object[] {6, "f"}));
         putRecordsToLeader(kvReplica, kvRecords);
 
-        // trigger third snapshot
-        scheduledExecutorService.triggerNonPeriodicScheduledTask();
+        // trigger third snapshot (may need to wait the task being scheduled)
+        scheduledExecutorService.triggerNextNonPeriodicScheduledTask(Duration.ofSeconds(30));
         CompletedSnapshot snapshot2 = kvSnapshotStore.waitUntilSnapshotComplete(tableBucket, 2);
 
         // verify that snapshot2 is the latest one before we break it
@@ -607,12 +689,8 @@ final class ReplicaTest extends ReplicaTestBase {
                 getKeyValuePairs(genKvRecords(new Object[] {1, "a"}, new Object[] {2, "b"}));
         verifyGetKeyValues(kvTablet, expectedKeyValues);
 
-        // We have to remove the first scheduled snapshot task since it's for the previous kv tablet
-        // whose rocksdb has been dropped.
-        scheduledExecutorService.removeNonPeriodicScheduledTask();
-
-        // trigger one snapshot,
-        scheduledExecutorService.triggerNonPeriodicScheduledTask();
+        // trigger first snapshot (task has been scheduled after becoming leader)
+        scheduledExecutorService.triggerAllNonPeriodicTasks();
         // wait until the snapshot success
         kvSnapshotStore.waitUntilSnapshotComplete(tableBucket, 0);
 
@@ -631,6 +709,65 @@ final class ReplicaTest extends ReplicaTestBase {
                                 new Object[] {3, "c"}));
         kvTablet = kvReplica.getKvTablet();
         verifyGetKeyValues(kvTablet, expectedKeyValues);
+
+        // test recover with schema evolution.
+        short newSchemaId = 2;
+        // trigger second snapshot (task has been scheduled after becoming leader again)
+        scheduledExecutorService.triggerAllNonPeriodicTasks();
+        // wait until the snapshot success
+        kvSnapshotStore.waitUntilSnapshotComplete(tableBucket, 1);
+        // write data with old schema
+        putRecordsToLeader(
+                kvReplica,
+                DataTestUtils.genKvRecordBatch(new Object[] {4, "555"}, new Object[] {3, "d"}));
+        // update schema.
+        zkClient.registerSchema(DATA1_TABLE_PATH_PK, DATA2_SCHEMA, newSchemaId);
+        serverMetadataCache.updateLatestSchema(
+                DATA1_TABLE_ID, new SchemaInfo(DATA2_SCHEMA, newSchemaId));
+        // write data with new schema
+        putRecordsToLeader(
+                kvReplica,
+                DataTestUtils.genKvRecordBatch(
+                        newSchemaId,
+                        DATA2_ROW_TYPE,
+                        new Object[] {5, "555", "666"},
+                        new Object[] {4, "555", "666"}));
+        expectedKeyValues =
+                getKeyValuePairs(genKvRecords(new Object[] {1, "a"}, new Object[] {2, "bbb"}));
+        expectedKeyValues.addAll(
+                getKeyValuePairs(
+                        newSchemaId,
+                        genKvRecords(
+                                DATA2_ROW_TYPE,
+                                new Object[] {3, "d", null},
+                                new Object[] {4, "555", "666"},
+                                new Object[] {5, "555", "666"})));
+        // restore again and apply the schema.
+        makeKvReplicaAsLeader(kvReplica, 4);
+        kvTablet = kvReplica.getKvTablet();
+        verifyGetKeyValues(kvTablet, expectedKeyValues);
+    }
+
+    @Test
+    void testUpdateIsDataLakeEnabled() throws Exception {
+        Replica logReplica =
+                makeLogReplica(DATA1_PHYSICAL_TABLE_PATH, new TableBucket(DATA1_TABLE_ID, 1));
+        makeLogReplicaAsLeader(logReplica);
+
+        // initial state should be false
+        assertThat(logReplica.getLogTablet().isDataLakeEnabled()).isFalse();
+
+        // update to true
+        logReplica.updateIsDataLakeEnabled(true);
+        assertThat(logReplica.getLogTablet().isDataLakeEnabled()).isTrue();
+
+        // update with same value should not change anything (no-op)
+        logReplica.updateIsDataLakeEnabled(true);
+        assertThat(logReplica.getLogTablet().isDataLakeEnabled()).isTrue();
+
+        // update to false
+        logReplica.updateIsDataLakeEnabled(false);
+        assertThat(logReplica.getLogTablet().isDataLakeEnabled()).isFalse();
     }
 
     private void makeLogReplicaAsLeader(Replica replica) throws Exception {
@@ -696,18 +833,29 @@ final class ReplicaTest extends ReplicaTestBase {
                 replica.getTableBucket().getTableId(),
                 offset,
                 Integer.MAX_VALUE,
-                replica.getRowType(),
+                replica.getSchemaGetter(),
                 DEFAULT_COMPRESSION,
-                null);
+                null,
+                new ProjectionPushdownCache());
         LogReadInfo logReadInfo = replica.fetchRecords(fetchParams);
         return logReadInfo.getFetchedData().getRecords();
     }
 
     private static MemoryLogRecords logRecords(
             long baseOffset, List<ChangeType> changeTypes, List<Object[]> values) throws Exception {
+        return logRecords(DEFAULT_SCHEMA_ID, DATA1_ROW_TYPE, baseOffset, changeTypes, values);
+    }
+
+    private static MemoryLogRecords logRecords(
+            short schemaId,
+            RowType rowType,
+            long baseOffset,
+            List<ChangeType> changeTypes,
+            List<Object[]> values)
+            throws Exception {
         return createBasicMemoryLogRecords(
-                DATA1_ROW_TYPE,
-                DEFAULT_SCHEMA_ID,
+                rowType,
+                schemaId,
                 baseOffset,
                 -1L,
                 CURRENT_LOG_MAGIC_VALUE,
@@ -721,7 +869,9 @@ final class ReplicaTest extends ReplicaTestBase {
 
     private LogAppendInfo putRecordsToLeader(
             Replica replica, KvRecordBatch kvRecords, int[] targetColumns) throws Exception {
-        LogAppendInfo logAppendInfo = replica.putRecordsToLeader(kvRecords, targetColumns, 0);
+        // Use DEFAULT mode as default for tests
+        LogAppendInfo logAppendInfo =
+                replica.putRecordsToLeader(kvRecords, targetColumns, MergeMode.DEFAULT, 0);
         KvTablet kvTablet = checkNotNull(replica.getKvTablet());
         // flush to make data visible
         kvTablet.flush(replica.getLocalLogEndOffset(), NOPErrorHandler.INSTANCE);
