@@ -20,15 +20,20 @@ package org.apache.fluss.server.tablet;
 import org.apache.fluss.config.ConfigOptions;
 import org.apache.fluss.exception.FlussRuntimeException;
 import org.apache.fluss.exception.InvalidRequiredAcksException;
+import org.apache.fluss.metadata.KvFormat;
 import org.apache.fluss.metadata.LogFormat;
 import org.apache.fluss.metadata.PhysicalTablePath;
 import org.apache.fluss.metadata.Schema;
+import org.apache.fluss.metadata.SchemaGetter;
 import org.apache.fluss.metadata.TableBucket;
 import org.apache.fluss.metadata.TableDescriptor;
 import org.apache.fluss.metadata.TablePath;
 import org.apache.fluss.record.DefaultKvRecordBatch;
 import org.apache.fluss.record.DefaultValueRecordBatch;
+import org.apache.fluss.record.TestingSchemaGetter;
+import org.apache.fluss.row.InternalRow;
 import org.apache.fluss.row.encode.CompactedKeyEncoder;
+import org.apache.fluss.row.encode.ValueDecoder;
 import org.apache.fluss.row.encode.ValueEncoder;
 import org.apache.fluss.rpc.gateway.TabletServerGateway;
 import org.apache.fluss.rpc.messages.FetchLogResponse;
@@ -92,6 +97,7 @@ import static org.apache.fluss.record.TestData.DATA1_TABLE_PATH;
 import static org.apache.fluss.record.TestData.DATA1_TABLE_PATH_PK;
 import static org.apache.fluss.record.TestData.DATA_1_WITH_KEY_AND_VALUE;
 import static org.apache.fluss.record.TestData.DEFAULT_SCHEMA_ID;
+import static org.apache.fluss.record.TestData.TEST_SCHEMA_GETTER;
 import static org.apache.fluss.server.testutils.KvTestUtils.assertLookupResponse;
 import static org.apache.fluss.server.testutils.KvTestUtils.assertPrefixLookupResponse;
 import static org.apache.fluss.server.testutils.RpcMessageTestUtils.assertFetchLogResponse;
@@ -219,6 +225,7 @@ public class TabletServiceITCase {
         long tableId =
                 createTable(FLUSS_CLUSTER_EXTENSION, DATA1_TABLE_PATH, DATA1_TABLE_DESCRIPTOR);
         TableBucket tb = new TableBucket(tableId, 0);
+        SchemaGetter schemaGetter = new TestingSchemaGetter(1, DATA1_SCHEMA);
 
         FLUSS_CLUSTER_EXTENSION.waitUntilAllReplicaReady(tb);
 
@@ -297,6 +304,7 @@ public class TabletServiceITCase {
                         .fetchLog(newFetchLogRequest(-1, tableId, 0, 10L, new int[] {0}))
                         .get(),
                 DATA1_ROW_TYPE.project(new int[] {0}),
+                schemaGetter,
                 tableId,
                 0,
                 20L,
@@ -312,6 +320,7 @@ public class TabletServiceITCase {
                         .fetchLog(newFetchLogRequest(-1, tableId, 0, 15L, new int[] {1}))
                         .get(),
                 DATA1_ROW_TYPE.project(new int[] {1}),
+                schemaGetter,
                 tableId,
                 0,
                 20L,
@@ -740,6 +749,7 @@ public class TabletServiceITCase {
         assertLimitScanResponse(
                 logLeaderGateWay.limitScan(newLimitScanRequest(logTableId, 0, 10)).get(),
                 DATA1_ROW_TYPE,
+                TEST_SCHEMA_GETTER,
                 expected2);
     }
 
@@ -966,4 +976,100 @@ public class TabletServiceITCase {
         return ServerRpcMessageUtils.makeNotifyLeaderAndIsrRequest(
                 0, Collections.singletonList(reqForBucket));
     }
+
+    @Test
+    void testLookupWithInsertIfNotExists() throws Exception {
+        long tableId =
+                createTable(
+                        FLUSS_CLUSTER_EXTENSION, DATA1_TABLE_PATH_PK, DATA1_TABLE_DESCRIPTOR_PK);
+        TableBucket tb = new TableBucket(tableId, 0);
+        FLUSS_CLUSTER_EXTENSION.waitUntilAllReplicaReady(tb);
+
+        int leader = FLUSS_CLUSTER_EXTENSION.waitAndGetLeader(tb);
+        TabletServerGateway gateway = FLUSS_CLUSTER_EXTENSION.newTabletServerClientForNode(leader);
+        CompactedKeyEncoder keyEncoder = new CompactedKeyEncoder(DATA1_ROW_TYPE, new int[] {0});
+
+        byte[] key1 = keyEncoder.encodeKey(row(new Object[] {100}));
+        byte[] key2 = keyEncoder.encodeKey(row(new Object[] {200}));
+        byte[] key3 = keyEncoder.encodeKey(row(new Object[] {300}));
+
+        // Insert missing keys
+        PbLookupRespForBucket resp1 =
+                gateway.lookup(newLookupRequest(tableId, 0, true, 20000, 1, key1, key2))
+                        .get()
+                        .getBucketsRespAt(0);
+        assertThat(resp1.hasErrorCode()).isFalse();
+        byte[] value1 = resp1.getValuesList().get(0).getValues();
+        byte[] value2 = resp1.getValuesList().get(1).getValues();
+        assertThat(value1).isNotNull();
+        assertThat(value2).isNotNull();
+
+        // Existing keys return same values
+        PbLookupRespForBucket resp2 =
+                gateway.lookup(newLookupRequest(tableId, 0, true, 20000, 1, key1, key2))
+                        .get()
+                        .getBucketsRespAt(0);
+        assertThat(resp2.getValuesList().get(0).getValues()).isEqualTo(value1);
+        assertThat(resp2.getValuesList().get(1).getValues()).isEqualTo(value2);
+
+        // Mixed: key1 exists, key3 missing
+        PbLookupRespForBucket resp3 =
+                gateway.lookup(newLookupRequest(tableId, 0, true, 20000, 1, key1, key3))
+                        .get()
+                        .getBucketsRespAt(0);
+        assertThat(resp3.getValuesList().get(0).getValues()).isEqualTo(value1);
+        assertThat(resp3.getValuesList().get(1).getValues()).isNotNull();
+    }
+
+    @Test
+    void testLookupWithInsertIfNotExistsAutoIncrement() throws Exception {
+        TablePath tablePath = TablePath.of("test_db_1", "test_auto_increment_t1");
+        Schema schema =
+                Schema.newBuilder()
+                        .column("a", DataTypes.INT())
+                        .column("b", DataTypes.BIGINT())
+                        .withComment("auto increment column")
+                        .column("c", DataTypes.STRING())
+                        .enableAutoIncrement("b")
+                        .primaryKey("a")
+                        .build();
+        RowType rowType = schema.getRowType();
+        TableDescriptor descriptor =
+                TableDescriptor.builder().schema(schema).distributedBy(1, "a").build();
+        long tableId = createTable(FLUSS_CLUSTER_EXTENSION, tablePath, descriptor);
+        TableBucket tb = new TableBucket(tableId, 0);
+        FLUSS_CLUSTER_EXTENSION.waitUntilAllReplicaReady(tb);
+
+        int leader = FLUSS_CLUSTER_EXTENSION.waitAndGetLeader(tb);
+        TabletServerGateway gateway = FLUSS_CLUSTER_EXTENSION.newTabletServerClientForNode(leader);
+        CompactedKeyEncoder keyEncoder = new CompactedKeyEncoder(rowType, new int[] {0});
+        TestingSchemaGetter schemaGetter = new TestingSchemaGetter(DEFAULT_SCHEMA_ID, schema);
+        ValueDecoder valueDecoder = new ValueDecoder(schemaGetter, KvFormat.COMPACTED);
+
+        byte[] key1 = keyEncoder.encodeKey(row(new Object[] {100}));
+        byte[] key2 = keyEncoder.encodeKey(row(new Object[] {200}));
+        byte[] key3 = keyEncoder.encodeKey(row(new Object[] {300}));
+
+        // Insert key1, key2 - auto-increment: 1, 2
+        PbLookupRespForBucket resp1 =
+                gateway.lookup(newLookupRequest(tableId, 0, true, 20000, 1, key1, key2))
+                        .get()
+                        .getBucketsRespAt(0);
+        assertThat(resp1.hasErrorCode()).isFalse();
+        InternalRow row1 = valueDecoder.decodeValue(resp1.getValuesList().get(0).getValues()).row;
+        InternalRow row2 = valueDecoder.decodeValue(resp1.getValuesList().get(1).getValues()).row;
+        assertThat(row1.getLong(1)).isEqualTo(1L);
+        assertThat(row2.getLong(1)).isEqualTo(2L);
+
+        // Mixed: key1 exists (unchanged), key3 missing (gets 3)
+        PbLookupRespForBucket resp2 =
+                gateway.lookup(newLookupRequest(tableId, 0, true, 20000, 1, key1, key3))
+                        .get()
+                        .getBucketsRespAt(0);
+        InternalRow existingRow =
+                valueDecoder.decodeValue(resp2.getValuesList().get(0).getValues()).row;
+        InternalRow newRow = valueDecoder.decodeValue(resp2.getValuesList().get(1).getValues()).row;
+        assertThat(existingRow.getLong(1)).isEqualTo(1L);
+        assertThat(newRow.getLong(1)).isEqualTo(3L);
+    }
 }
