@@ -36,14 +36,16 @@ import org.apache.fluss.rpc.messages.GetFileSystemSecurityTokenRequest;
 import org.apache.fluss.rpc.messages.GetFileSystemSecurityTokenResponse;
 import org.apache.fluss.rpc.messages.GetKvSnapshotMetadataRequest;
 import org.apache.fluss.rpc.messages.GetKvSnapshotMetadataResponse;
+import org.apache.fluss.rpc.messages.GetLakeSnapshotRequest;
+import org.apache.fluss.rpc.messages.GetLakeSnapshotResponse;
 import org.apache.fluss.rpc.messages.GetLatestKvSnapshotsRequest;
 import org.apache.fluss.rpc.messages.GetLatestKvSnapshotsResponse;
-import org.apache.fluss.rpc.messages.GetLatestLakeSnapshotRequest;
-import org.apache.fluss.rpc.messages.GetLatestLakeSnapshotResponse;
 import org.apache.fluss.rpc.messages.GetTableInfoRequest;
 import org.apache.fluss.rpc.messages.GetTableInfoResponse;
 import org.apache.fluss.rpc.messages.GetTableSchemaRequest;
 import org.apache.fluss.rpc.messages.GetTableSchemaResponse;
+import org.apache.fluss.rpc.messages.GetTableStatsRequest;
+import org.apache.fluss.rpc.messages.GetTableStatsResponse;
 import org.apache.fluss.rpc.messages.InitWriterRequest;
 import org.apache.fluss.rpc.messages.InitWriterResponse;
 import org.apache.fluss.rpc.messages.LimitScanRequest;
@@ -87,6 +89,7 @@ import org.apache.fluss.rpc.messages.TableExistsRequest;
 import org.apache.fluss.rpc.messages.TableExistsResponse;
 import org.apache.fluss.rpc.messages.UpdateMetadataRequest;
 import org.apache.fluss.rpc.messages.UpdateMetadataResponse;
+import org.apache.fluss.rpc.protocol.ApiKeys;
 import org.apache.fluss.server.entity.FetchReqInfo;
 import org.apache.fluss.utils.types.Tuple2;
 
@@ -97,7 +100,7 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.Queue;
+import java.util.Set;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentLinkedDeque;
 import java.util.concurrent.atomic.AtomicLong;
@@ -113,16 +116,22 @@ public class TestTabletServerGateway implements TabletServerGateway {
 
     // Use concurrent queue for storing request and related completable future response so that
     // requests may be queried from a different thread.
-    private final Queue<Tuple2<ApiMessage, CompletableFuture<?>>> requests =
+    private final ConcurrentLinkedDeque<Tuple2<ApiMessage, CompletableFuture<?>>> requests =
             new ConcurrentLinkedDeque<>();
+    private final Set<ApiKeys> ignoreApiKeys;
 
-    public TestTabletServerGateway(boolean alwaysFail) {
+    public TestTabletServerGateway(boolean alwaysFail, Set<ApiKeys> ignoreApiKeys) {
         this.alwaysFail = alwaysFail;
+        this.ignoreApiKeys = ignoreApiKeys;
     }
 
     @Override
     public CompletableFuture<UpdateMetadataResponse> updateMetadata(UpdateMetadataRequest request) {
-        return CompletableFuture.completedFuture(null);
+        CompletableFuture<UpdateMetadataResponse> response = new CompletableFuture<>();
+        if (!ignoreApiKeys.contains(ApiKeys.UPDATE_METADATA)) {
+            requests.add(Tuple2.of(request, response));
+        }
+        return response;
     }
 
     @Override
@@ -150,9 +159,9 @@ public class TestTabletServerGateway implements TabletServerGateway {
     }
 
     @Override
-    public CompletableFuture<GetLatestLakeSnapshotResponse> getLatestLakeSnapshot(
-            GetLatestLakeSnapshotRequest request) {
-        throw new UnsupportedOperationException();
+    public CompletableFuture<GetLakeSnapshotResponse> getLakeSnapshot(
+            GetLakeSnapshotRequest request) {
+        return null;
     }
 
     @Override
@@ -177,7 +186,9 @@ public class TestTabletServerGateway implements TabletServerGateway {
 
     @Override
     public CompletableFuture<PutKvResponse> putKv(PutKvRequest request) {
-        return null;
+        CompletableFuture<PutKvResponse> response = new CompletableFuture<>();
+        requests.add(Tuple2.of(request, response));
+        return response;
     }
 
     @Override
@@ -187,7 +198,7 @@ public class TestTabletServerGateway implements TabletServerGateway {
 
     @Override
     public CompletableFuture<PrefixLookupResponse> prefixLookup(PrefixLookupRequest request) {
-        throw new UnsupportedOperationException();
+        return null;
     }
 
     @Override
@@ -195,6 +206,11 @@ public class TestTabletServerGateway implements TabletServerGateway {
         return null;
     }
 
+    @Override
+    public CompletableFuture<GetTableStatsResponse> getTableStats(GetTableStatsRequest request) {
+        return null;
+    }
+
     @Override
     public CompletableFuture<ListOffsetsResponse> listOffsets(ListOffsetsRequest request) {
         return null;
